@@ -1,7 +1,7 @@
 
 //OpenSCADA system module Archive.DBArch file: arch.cpp
 /***************************************************************************
- *   Copyright (C) 2007 by Roman Savochenko                                *
+ *   Copyright (C) 2007-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,7 +18,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
- 
+
 #include <sys/stat.h>
 #include <signal.h>
 #include <getopt.h>
@@ -32,32 +32,32 @@
 
 //*************************************************
 //* Modul info!                                   *
-#define MOD_ID      "DBArch"
-#define MOD_NAME    "To DB archivator"
-#define MOD_TYPE    "Archive"
-#define VER_TYPE    VER_ARH
-#define VERSION     "0.6.0"
-#define AUTORS      "Roman Savochenko"
-#define DESCRIPTION "The Archive module. Allow functions for messages and values arhiving to DB."
-#define LICENSE     "GPL"
+#define MOD_ID		"DBArch"
+#define MOD_NAME	"To DB archivator"
+#define MOD_TYPE	"Archive"
+#define VER_TYPE	VER_ARH
+#define VERSION		"0.7.0"
+#define AUTORS		"Roman Savochenko"
+#define DESCRIPTION	"The Archive module. Allow functions for messages and values arhiving to DB."
+#define LICENSE		"GPL"
 //*************************************************
 
 DBArch::ModArch *DBArch::mod;
 
 extern "C"
-{ 
+{
     TModule::SAt module( int n_mod )
     {
 	if( n_mod==0 ) 	return TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE);
-	return TModule::SAt("");    
+	return TModule::SAt("");
     }
-    
+
     TModule *attach( const TModule::SAt &AtMod, const string &source )
     {
 	if( AtMod == TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE) ) 
 	    return new DBArch::ModArch( source );
 	return NULL;
-    }    
+    }
 }
 
 using namespace DBArch;
@@ -67,38 +67,38 @@ using namespace DBArch;
 //*************************************************
 ModArch::ModArch( const string &name)
 {
-    mId 	= MOD_ID;
+    mId		= MOD_ID;
     mName	= MOD_NAME;
-    mType  	= MOD_TYPE;
-    mVers      	= VERSION;
-    mAutor    	= AUTORS;
-    mDescr  	= DESCRIPTION;
-    mLicense   	= LICENSE;
-    mSource    	= name;
-    
-    mod 	= this;
+    mType	= MOD_TYPE;
+    mVers	= VERSION;
+    mAutor	= AUTORS;
+    mDescr	= DESCRIPTION;
+    mLicense	= LICENSE;
+    mSource	= name;
+
+    mod		= this;
 }
 
 void ModArch::postEnable( int flag )
 {
     TModule::postEnable( flag );
-    
+
     if( flag&TCntrNode::NodeConnect )
     {
 	//- Add self DB-fields for messages archive -
-        owner().messE().fldAdd( new TFld("DBArchSize",_("Archive size (hours)"),TFld::Real,TFld::NoFlag,"20.5","24") );
+	owner().messE().fldAdd( new TFld("DBArchSize",_("Archive size (hours)"),TFld::Real,TFld::NoFlag,"20.5","24") );
 
 	//- Add self DB-fields for value archive -
 	owner().valE().fldAdd( new TFld("DBArchSize",_("Archive size (hours)"),TFld::Real,TFld::NoFlag,"20.5","24") );
-	    
-        //- Archivators info table DB structure -
+
+	//- Archivators info table DB structure -
 	el_arch.fldAdd( new TFld("TBL",_("Table"),TFld::String,TCfg::Key,"50") );
 	el_arch.fldAdd( new TFld("BEGIN",_("Begin"),TFld::String,TFld::NoFlag,"20") );
-        el_arch.fldAdd( new TFld("END",_("End"),TFld::String,TFld::NoFlag,"20") );
-        el_arch.fldAdd( new TFld("PRM1",_("Parameter 1"),TFld::String,TFld::NoFlag,"20") );
-        el_arch.fldAdd( new TFld("PRM2",_("Parameter 2"),TFld::String,TFld::NoFlag,"20") );
-        el_arch.fldAdd( new TFld("PRM3",_("Parameter 3"),TFld::String,TFld::NoFlag,"20") );
-    
+	el_arch.fldAdd( new TFld("END",_("End"),TFld::String,TFld::NoFlag,"20") );
+	el_arch.fldAdd( new TFld("PRM1",_("Parameter 1"),TFld::String,TFld::NoFlag,"20") );
+	el_arch.fldAdd( new TFld("PRM2",_("Parameter 2"),TFld::String,TFld::NoFlag,"20") );
+	el_arch.fldAdd( new TFld("PRM3",_("Parameter 3"),TFld::String,TFld::NoFlag,"20") );
+
 	//- Message DB archive DB structure -
 	el_mess.fldAdd( new TFld("TM",_("Time (c)"),TFld::Integer,TCfg::Key,"10") );
 	el_mess.fldAdd( new TFld("CATEG",_("Category"),TFld::String,TCfg::Key,"100") );
@@ -109,16 +109,16 @@ void ModArch::postEnable( int flag )
 	el_vl_int.fldAdd( new TFld("TM",_("Time (c)"),TFld::Integer,TCfg::Key,"10") );
 	el_vl_int.fldAdd( new TFld("TMU",_("Time (uc)"),TFld::Integer,TCfg::Key,"10") );
 	el_vl_int.fldAdd( new TFld("VAL",_("Value"),TFld::Integer,TFld::NoFlag,"10") );
- 	
+
 	//- Real value DB archive DB structure -
 	el_vl_real.fldAdd( new TFld("TM",_("Time (c)"),TFld::Integer,TCfg::Key,"10") );
 	el_vl_real.fldAdd( new TFld("TMU",_("Time (uc)"),TFld::Integer,TCfg::Key,"10") );
 	el_vl_real.fldAdd( new TFld("VAL",_("Value"),TFld::Real,TFld::NoFlag,"30.15") );
 
 	//- String value DB archive DB structure -
- 	el_vl_str.fldAdd( new TFld("TM",_("Time (c)"),TFld::Integer,TCfg::Key,"10") );
+	el_vl_str.fldAdd( new TFld("TM",_("Time (c)"),TFld::Integer,TCfg::Key,"10") );
 	el_vl_str.fldAdd( new TFld("TMU",_("Time (uc)"),TFld::Integer,TCfg::Key,"10") );
-	el_vl_str.fldAdd( new TFld("VAL",_("Value"),TFld::String,TFld::NoFlag,"255") );        
+	el_vl_str.fldAdd( new TFld("VAL",_("Value"),TFld::String,TFld::NoFlag,"255") );
     }
 }
 
@@ -143,7 +143,7 @@ void ModArch::load_()
 {
     //- Load parameters from command line -
     int next_opt;
-    char *short_opt="h";
+    const char *short_opt="h";
     struct option long_opt[] =
     {
 	{"help"    ,0,NULL,'h'},
@@ -188,7 +188,7 @@ void ModArch::cntrCmdProc( XMLNode *opt )
     //- Get page info -
     if( opt->name() == "info" )
     {
-        TTipArchivator::cntrCmdProc(opt);
+	TTipArchivator::cntrCmdProc(opt);
 	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","5");
 	return;
     }
