@@ -1,7 +1,7 @@
 
 //OpenSCADA system module Archive.DBArch file: mess.cpp
 /***************************************************************************
- *   Copyright (C) 2007 by Roman Savochenko                                *
+ *   Copyright (C) 2007-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -19,7 +19,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-#include <sys/time.h> 
+#include <sys/time.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <dirent.h>
@@ -34,8 +34,8 @@ using namespace DBArch;
 //************************************************
 //* DBArch::ModMArch - Messages archivator       *
 //************************************************
-ModMArch::ModMArch( const string &iid, const string &idb, TElem *cf_el ) : 
-    TMArchivator(iid,idb,cf_el), m_beg(0), m_end(0), tm_calc(0.0), 
+ModMArch::ModMArch( const string &iid, const string &idb, TElem *cf_el ) :
+    TMArchivator(iid,idb,cf_el), m_beg(0), m_end(0), tm_calc(0.0),
     m_max_size(cfg("DBArchSize").getRd())
 {
     setAddr("*.*");
@@ -54,12 +54,12 @@ void ModMArch::postDisable( int flag )
 	if( flag )
 	{
 	    //- Remove info record -
-    	    TConfig cfg(&mod->archEl());
+	    TConfig cfg(&mod->archEl());
 	    cfg.cfg("TBL").setS(archTbl());
 	    SYS->db().at().dataDel(addr()+"."+mod->mainTbl(),"",cfg);
-		
+
 	    //- Remove archive's DB table -
-    	    SYS->db().at().open( addr()+"."+archTbl() );
+	    SYS->db().at().open( addr()+"."+archTbl() );
 	    SYS->db().at().close( addr()+"."+archTbl(), true );
 	}
     }
@@ -72,7 +72,7 @@ void ModMArch::load_( )
 
     //- Init address to DB -
     if( addr().empty() ) setAddr("*.*");
-    
+
     //- Load message archive parameters -
     TConfig cfg(&mod->archEl());
     cfg.cfg("TBL").setS(archTbl());
@@ -83,7 +83,7 @@ void ModMArch::load_( )
 	//-- Check for delete archivator table --
 	if( m_end <= (time(NULL)-(time_t)(maxSize()*3600.)) )
 	{
-    	    SYS->db().at().open(addr()+"."+archTbl());
+	    SYS->db().at().open(addr()+"."+archTbl());
 	    SYS->db().at().close(addr()+"."+archTbl(),true);
 	    m_beg = m_end = 0;
 	}
@@ -91,7 +91,7 @@ void ModMArch::load_( )
 }
 
 void ModMArch::start( )
-{    
+{
     run_st = true;
 }
 
@@ -113,16 +113,16 @@ time_t ModMArch::end()
 void ModMArch::put( vector<TMess::SRec> &mess )
 {
     if(!run_st) throw TError(nodePath().c_str(),_("Archive no started!"));
-    
+
     AutoHD<TTable> tbl = SYS->db().at().open(addr()+"."+archTbl(),true);
-    if( tbl.freeStat() ) return;	    
-    
+    if( tbl.freeStat() ) return;
+
     TConfig cfg(&mod->messEl());
     unsigned long long t_cnt = SYS->shrtCnt();
     for( unsigned i_m = 0; i_m < mess.size(); i_m++)
     {
-	if( !chkMessOK(mess[i_m].categ,mess[i_m].level) ) continue;	
-	
+	if( !chkMessOK(mess[i_m].categ,mess[i_m].level) ) continue;
+
 	//- Put record to DB -
 	cfg.cfg("TM").setI(mess[i_m].time);
 	cfg.cfg("CATEG").setS(mess[i_m].categ);
@@ -133,7 +133,7 @@ void ModMArch::put( vector<TMess::SRec> &mess )
 	m_beg=m_beg?vmin(m_beg,mess[i_m].time):mess[i_m].time;
 	m_end=m_end?vmax(m_end,mess[i_m].time):mess[i_m].time;
     }
-    
+
     //- Archive size limit process -
     if( (m_end-m_beg) > (time_t)(maxSize()*3600.) )
     {
@@ -147,7 +147,7 @@ void ModMArch::put( vector<TMess::SRec> &mess )
     }
     tbl.free();
     SYS->db().at().close(addr()+"."+archTbl());
-    
+
     //- Update archive info -
     cfg.setElem(&mod->archEl());
     cfg.cfgViewAll(false);
@@ -155,20 +155,20 @@ void ModMArch::put( vector<TMess::SRec> &mess )
     cfg.cfg("BEGIN").setS(TSYS::int2str(m_beg),true);
     cfg.cfg("END").setS(TSYS::int2str(m_end),true);
     SYS->db().at().dataSet(addr()+"."+mod->mainTbl(),"",cfg);
-    
+
     tm_calc = 1.0e3*((double)(SYS->shrtCnt()-t_cnt))/((double)SYS->sysClk());
 }
 
 void ModMArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const string &category, char level )
 {
-    if(!run_st) throw TError(nodePath().c_str(),_("Archive no started!"));
+    if( !run_st ) throw TError(nodePath().c_str(),_("Archive no started!"));
 
     b_tm = vmax(b_tm,begin());
     e_tm = vmin(e_tm,end());
     if( e_tm <= b_tm ) return;
-    
+
     TConfig cfg(&mod->messEl());
-    //- Get values from DB -    
+    //- Get values from DB -
     for( time_t t_c = b_tm; t_c <= e_tm; t_c++ )
     {
 	cfg.cfg("TM").setI(t_c);
@@ -178,17 +178,17 @@ void ModMArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const s
 	    if( rc.level >= level && TMess::chkPattern(rc.categ,category) )
 	    {
 		bool equal = false;
-	        int i_p = mess.size();
-	        for( int i_m = mess.size()-1; i_m >= 0; i_m-- )
-	        {
-	            if( mess[i_m].time > rc.time )   i_p = i_m;
-	            else if( rc.time == mess[i_m].time && rc.level == mess[i_m].level && rc.mess == mess[i_m].mess )
-                    { equal = true; break; }
-                    else if( mess[i_m].time < rc.time ) break;
-        	}
-                if( !equal )
-		{ 
-		    mess.insert(mess.begin()+i_p,rc);    
+		int i_p = mess.size();
+		for( int i_m = mess.size()-1; i_m >= 0; i_m-- )
+		{
+		    if( mess[i_m].time > rc.time )   i_p = i_m;
+		    else if( rc.time == mess[i_m].time && rc.level == mess[i_m].level && rc.mess == mess[i_m].mess )
+		    { equal = true; break; }
+		    else if( mess[i_m].time < rc.time ) break;
+		}
+		if( !equal )
+		{
+		    mess.insert(mess.begin()+i_p,rc);
 		    if( mess.size() >= TArchiveS::max_req_mess ) return;
 		}
 	    }
@@ -199,15 +199,14 @@ void ModMArch::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const s
 
 void ModMArch::cntrCmdProc( XMLNode *opt )
 {
-    string grp = owner().owner().subId();    
     //- Get page info -
     if( opt->name() == "info" )
     {
-        TMArchivator::cntrCmdProc(opt);
-	ctrMkNode("fld",opt,-1,"/prm/st/tarch",_("Archiving time (msek)"),0444,"root",grp.c_str(),1,"tp","real");
-	if( ctrMkNode("area",opt,1,"/bs",_("Additional options"),0444,"root",grp.c_str()) )
-	    ctrMkNode("fld",opt,-1,"/bs/sz",cfg("DBArchSize").fld().descr(),0664,"root",grp.c_str(),1,"tp","real");
-        return;
+	TMArchivator::cntrCmdProc(opt);
+	ctrMkNode("fld",opt,-1,"/prm/st/tarch",_("Archiving time (msek)"),0444,"root","Archive",1,"tp","real");
+	if( ctrMkNode("area",opt,1,"/bs",_("Additional options"),0444,"root","Archive") )
+	    ctrMkNode("fld",opt,-1,"/bs/sz",cfg("DBArchSize").fld().descr(),0664,"root","Archive",1,"tp","real");
+	return;
     }
 
     //- Process command to page -
@@ -215,8 +214,8 @@ void ModMArch::cntrCmdProc( XMLNode *opt )
     if( a_path == "/prm/st/tarch" && ctrChkNode(opt) ) 	opt->setText(TSYS::real2str(tm_calc,6));
     else if( a_path == "/bs/sz" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",grp.c_str(),SEQ_RD) ) opt->setText(TSYS::real2str( maxSize() ));
-	if( ctrChkNode(opt,"set",0664,"root",grp.c_str(),SEQ_WR) ) setMaxSize( atof(opt->text().c_str()) );
+	if( ctrChkNode(opt,"get",0664,"root","Archive",SEQ_RD) ) opt->setText(TSYS::real2str( maxSize() ));
+	if( ctrChkNode(opt,"set",0664,"root","Archive",SEQ_WR) ) setMaxSize( atof(opt->text().c_str()) );
     }
     else TMArchivator::cntrCmdProc(opt);
 }
