@@ -1,7 +1,7 @@
 
 //OpenSCADA system module BD.DBF file: dbf_mod.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -29,20 +29,21 @@
 
 #include <tsys.h>
 #include <terror.h>
+#include <string.h>
 #include <tmess.h>
 #include "dbf.h"
 #include "dbf_mod.h"
 
 //************************************************
 //* Modul info!                                  *
-#define MOD_ID      "DBF"
-#define MOD_NAME    "DB DBF"
-#define MOD_TYPE    "BD"
-#define VER_TYPE    VER_BD
-#define VERSION     "1.8.1"
-#define AUTORS      "Roman Savochenko"
-#define DESCRIPTION "BD modul. Allow support of the *.dbf files, version 3.0."
-#define LICENSE     "GPL"
+#define MOD_ID		"DBF"
+#define MOD_NAME	"DB DBF"
+#define MOD_TYPE	"BD"
+#define VER_TYPE	VER_BD
+#define VERSION		"1.9.0"
+#define AUTORS		"Roman Savochenko"
+#define DESCRIPTION	"BD modul. Allow support of the *.dbf files, version 3.0."
+#define LICENSE		"GPL"
 //************************************************
 
 BDDBF::BDMod *BDDBF::mod;
@@ -51,14 +52,14 @@ extern "C"
 {
     TModule::SAt module( int n_mod )
     {
-	if( n_mod==0 )	return TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE);
+	if( n_mod==0 ) return TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE);
 	return TModule::SAt("");
     }
 
     TModule *attach( const TModule::SAt &AtMod, const string &source )
     {
 	if( AtMod == TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE) )
-    	    return new BDDBF::BDMod( source );
+	    return new BDDBF::BDMod( source );
 	return NULL;
     }
 }
@@ -68,17 +69,17 @@ using namespace BDDBF;
 //************************************************
 //* BDDBF::BDMod                                 *
 //************************************************
-BDMod::BDMod( string name ) 
+BDMod::BDMod( string name )
 {
-    mId 	= MOD_ID;
-    mName       = MOD_NAME;
-    mType  	= MOD_TYPE; 
-    mVers      	= VERSION; 
-    mAutor    	= AUTORS;
-    mDescr  	= DESCRIPTION; 
-    mLicense   	= LICENSE; 
-    mSource    	= name;
-    
+    mId		= MOD_ID;
+    mName	= MOD_NAME;
+    mType	= MOD_TYPE;
+    mVers	= VERSION;
+    mAutor	= AUTORS;
+    mDescr	= DESCRIPTION;
+    mLicense	= LICENSE;
+    mSource	= name;
+
     mod		= this;
 }
 
@@ -108,7 +109,7 @@ void BDMod::load_( )
 {
     //- Load parameters from command line -
     int next_opt;
-    char *short_opt = "h";
+    const char *short_opt = "h";
     struct option long_opt[] = 
     {
 	{"help", 0, NULL,'h'},
@@ -126,7 +127,7 @@ void BDMod::load_( )
 	}
     }
     while ( next_opt != -1 );
-    
+
     //- Load parameters from config file -
 }
 
@@ -140,25 +141,25 @@ MBD::MBD( string iid, TElem *cf_el ) : TBD(iid,cf_el)
 
 MBD::~MBD(  )
 {
-    
+
 }
 
 void MBD::postDisable(int flag)
 {
     TBD::postDisable(flag);
-    
+
     if( flag && owner().fullDeleteDB() )
-        if(rmdir(addr().c_str()) != 0)
-    	    mess_warning(nodePath().c_str(),_("Delete DB error!"));
+	if(rmdir(addr().c_str()) != 0)
+	    mess_warning(nodePath().c_str(),_("Delete DB error!"));
 }
 
 void MBD::enable( )
 {
     char   buf[STR_BUF_LEN];
-    
+
     char *rez = getcwd(buf,sizeof(buf));
     if( chdir(addr().c_str()) != 0 && mkdir(addr().c_str(),S_IRWXU|S_IRGRP|S_IROTH) != 0 )
-        throw TError(TSYS::DBInit,nodePath().c_str(),_("Error create DB directory <%s>!"),addr().c_str());
+	throw TError(TSYS::DBInit,nodePath().c_str(),_("Error create DB directory <%s>!"),addr().c_str());
     if( rez && chdir(buf) ) throw TError(TSYS::DBInit,nodePath().c_str(),_("Restore previous directory as curent is error."));
 
     TBD::enable( );
@@ -170,13 +171,13 @@ void MBD::allowList( vector<string> &list )
     list.clear();
 
     struct stat file_stat;
-    dirent *scan_dirent;    
+    dirent *scan_dirent;
     DIR *IdDir = opendir(addr().c_str());
     if(IdDir == NULL) return;
     while((scan_dirent = readdir(IdDir)) != NULL)
     {
 	nfile = scan_dirent->d_name;
-        if( nfile == ".." || nfile == "." || 
+	if( nfile == ".." || nfile == "." ||
 	    nfile.rfind(".") == string::npos || nfile.substr(nfile.rfind(".")) != ".dbf" ) continue;
 	stat((addr()+"/"+nfile).c_str(),&file_stat);
 	if( (file_stat.st_mode&S_IFMT) != S_IFREG ) continue;
@@ -186,12 +187,27 @@ void MBD::allowList( vector<string> &list )
 }
 
 TTable *MBD::openTable( const string &nm, bool create )
-{    
+{
     if( !enableStat() )
 	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Error open table <%s>. DB disabled."),nm.c_str());
     return new MTable(nm,this,create);
 }
 
+void MBD::cntrCmdProc( XMLNode *opt )
+{
+    //Get page info
+    if( opt->name() == "info" )
+    {
+	TBD::cntrCmdProc(opt);
+	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","BD",2,
+	    "tp","str","help",
+	    _("For DBF address DB is directory which contain files of tables (*.dbf).\n"
+	      "For example: /opt/dbf ."));
+	return;
+    }
+    TBD::cntrCmdProc(opt);
+}
+
 //************************************************
 //* BDDBF::MTable                                *
 //************************************************
@@ -207,7 +223,7 @@ MTable::MTable(const string &inm, MBD *iown, bool create) :
 
     codepage = owner().codePage().size()?owner().codePage():Mess->charset();
     n_table = owner().addr()+'/'+tbl_nm;
-    
+
     basa = new TBasaDBF(  );
     if( basa->LoadFile( (char *)n_table.c_str() ) == -1 && !create )
     {
@@ -240,45 +256,45 @@ void MTable::postDisable(int flag)
 bool MTable::fieldSeek( int i_ln, TConfig &cfg )
 {
     int i_clm;
-    
+
     ResAlloc res(m_res,false);
-    
-    i_ln = findKeyLine(cfg,i_ln);    
+
+    i_ln = findKeyLine(cfg,i_ln);
     if( i_ln < 0 ) return false;
-    
+
     //- Get config fields list -
     vector<string> cf_el;
     cfg.cfgList(cf_el);
-    
-    //- Write data to cfg - 
+
+    //- Write data to cfg -
     for( int i_cf = 0; i_cf < cf_el.size(); i_cf++ )
     {
 	TCfg &e_cfg = cfg.cfg(cf_el[i_cf]);
-	
+
 	//-- Find collumn --
 	db_str_rec *fld_rec;
 	for(i_clm = 0;(fld_rec = basa->getField(i_clm)) != NULL;i_clm++)
 	    if( cf_el[i_cf].substr(0,10) == fld_rec->name ) break;
-    	if(fld_rec == NULL) continue;
-	
+	if(fld_rec == NULL) continue;
+
 	//-- Get table volume --
 	string val;
 	if( basa->GetFieldIt( i_ln, i_clm, val ) < 0) 
 	    throw TError(TSYS::DBInernal,nodePath().c_str(),_("Cell error!"));
-	
+
 	//-- Write value --
 	switch(e_cfg.fld().type())
 	{
 	    case TFld::String:
 	    {
-    		//--- Remove spaces from end ---
+		//--- Remove spaces from end ---
 		int i;
-    		for(i = val.size(); i > 0; i--) if(val[i-1]!=' ') break;
-    		if(i != (int)val.size()) val.resize(i);
-	    
-    		e_cfg.setS(Mess->codeConvIn(codepage.c_str(),val));
+		for(i = val.size(); i > 0; i--) if(val[i-1]!=' ') break;
+		if(i != (int)val.size()) val.resize(i);
+
+		e_cfg.setS(Mess->codeConvIn(codepage.c_str(),val));
 		break;
-    	    }
+	    }
 	    case TFld::Integer:	e_cfg.setI(atoi(val.c_str()));	break;
 	    case TFld::Real:    e_cfg.setR(atof(val.c_str()));	break;
 	    case TFld::Boolean:	e_cfg.setB((val.c_str()[0] == 'T')?true:false);	break;
@@ -289,54 +305,54 @@ bool MTable::fieldSeek( int i_ln, TConfig &cfg )
 }
 
 void MTable::fieldGet( TConfig &cfg )
-{    
+{
     int i_ln, i_clm;
 
     //- Alloc resource -
     ResAlloc res(m_res,false);
-    
+
     //- Get key line -
-    i_ln = findKeyLine( cfg );    
+    i_ln = findKeyLine( cfg );
     if( i_ln < 0 ) throw TError(TSYS::DBRowNoPresent,nodePath().c_str(),_("Field no present!"));
-    
+
     //- Get config fields list -
     vector<string> cf_el;
     cfg.cfgList(cf_el);
-    
+
     //- Write data to cfg -
     for( int i_cf = 0; i_cf < cf_el.size(); i_cf++ )
     {
 	TCfg &e_cfg = cfg.cfg(cf_el[i_cf]);
-	
+
 	//-- Find collumn --
 	db_str_rec *fld_rec;
 	for(i_clm = 0;(fld_rec = basa->getField(i_clm)) != NULL;i_clm++)
 	    if( cf_el[i_cf].substr(0,10) == fld_rec->name ) break;
-    	if(fld_rec == NULL) continue;
-	
+	if(fld_rec == NULL) continue;
+
 	//-- Get table volume --
 	string val;
 	if( basa->GetFieldIt( i_ln, i_clm, val ) < 0) 
 	    throw TError(TSYS::DBInernal,nodePath().c_str(),_("Cell error!"));	
-	
+
 	//-- Write value --
 	switch(e_cfg.fld().type())
 	{
 	    case TFld::String:
 	    {
-    		//--- Remove spaces from end ---
+		//--- Remove spaces from end ---
 		int i;
-    		for(i = val.size(); i > 0; i--) if(val[i-1]!=' ') break;
-    		if(i != (int)val.size()) val.resize(i);
-	    
-    		e_cfg.setS(Mess->codeConvIn(codepage.c_str(),val));
+		for(i = val.size(); i > 0; i--) if(val[i-1]!=' ') break;
+		if(i != (int)val.size()) val.resize(i);
+
+		e_cfg.setS(Mess->codeConvIn(codepage.c_str(),val));
 		break;
-    	    }
+	    }
 	    case TFld::Integer:	e_cfg.setI(atoi(val.c_str()));	break;
 	    case TFld::Real:    e_cfg.setR(atof(val.c_str()));	break;
 	    case TFld::Boolean:	e_cfg.setB((val.c_str()[0] == 'T')?true:false);	break;
 	}
-    }    
+    }
 }
 
 void MTable::fieldSet( TConfig &cfg )
@@ -349,24 +365,24 @@ void MTable::fieldSet( TConfig &cfg )
     //- Get config fields list -
     vector<string> cf_el;
     cfg.cfgList(cf_el);
-    
+
     //- Check and fix structure of table -
     for( int i_cf = 0; i_cf < cf_el.size(); i_cf++ )
     {
 	TCfg &e_cfg = cfg.cfg(cf_el[i_cf]);
-	
+
 	//-- Find collumn --
 	db_str_rec *fld_rec;
 	for(i_clm = 0;(fld_rec = basa->getField(i_clm)) != NULL;i_clm++)
 	    if( cf_el[i_cf].substr(0,10) == fld_rec->name ) break;
-	if(fld_rec == NULL) 
+	if(fld_rec == NULL)
 	{
 	    //-- Create new collumn --
 	    db_str_rec n_rec;
-	    
-	    fieldPrmSet( e_cfg, n_rec );    
+
+	    fieldPrmSet( e_cfg, n_rec );
 	    if( basa->addField(i_cf,&n_rec) < 0 )
-		throw TError(TSYS::DBInernal,nodePath().c_str(),_("Column error!")); 	    
+		throw TError(TSYS::DBInernal,nodePath().c_str(),_("Column error!"));
 	}
 	else
 	{
@@ -375,21 +391,21 @@ void MTable::fieldSet( TConfig &cfg )
 	    {
 		case TFld::String:
 		    if( fld_rec->tip_fild == 'C' && e_cfg.fld().len() == fld_rec->len_fild )	continue;
-		    break;		    
-		case TFld::Integer:	
-		    if( fld_rec->tip_fild == 'N' && e_cfg.fld().len() == fld_rec->len_fild )	continue;  
 		    break;
-		case TFld::Real:    
+		case TFld::Integer:
+		    if( fld_rec->tip_fild == 'N' && e_cfg.fld().len() == fld_rec->len_fild )	continue;
+		    break;
+		case TFld::Real:
 		    if( fld_rec->tip_fild == 'N' && e_cfg.fld().len() == fld_rec->len_fild &&
-			    e_cfg.fld().dec() == fld_rec->dec_field )continue;  
+			    e_cfg.fld().dec() == fld_rec->dec_field )continue;
 		    break;
-		case TFld::Boolean:	
-		    if( fld_rec->tip_fild == 'L' )	continue;   
+		case TFld::Boolean:
+		    if( fld_rec->tip_fild == 'L' )	continue;
 		    break;
 	    }
-	    
+
 	    db_str_rec n_rec;
-	    
+
 	    fieldPrmSet( e_cfg, n_rec );
 	    if( basa->setField(i_clm,&n_rec) < 0 ) 
 		throw TError(TSYS::DBInernal,nodePath().c_str(),_("Column error!"));
@@ -403,11 +419,11 @@ void MTable::fieldSet( TConfig &cfg )
 	for( i_cf = 0; i_cf < cf_el.size(); i_cf++ )
 	    if( cf_el[i_cf].substr(0,10) == fld_rec->name ) break;
 	if( i_cf >= cf_el.size() )
-	    if( basa->DelField(i_clm) < 0 ) 
+	    if( basa->DelField(i_clm) < 0 )
 		throw TError(TSYS::DBClose,nodePath().c_str(),_("Delete field error!"));
-    }    
+    }
     //- Get key line -
-    i_ln = findKeyLine( cfg );    
+    i_ln = findKeyLine( cfg );
     if( i_ln < 0 ) i_ln = basa->CreateItems(-1);
     //- Write data to bd -
     for( int i_cf = 0; i_cf < cf_el.size(); i_cf++ )
@@ -422,14 +438,14 @@ void MTable::fieldSet( TConfig &cfg )
 
 	//-- Prepare value --
 	string val;
-       	switch(e_cfg.fld().type())
+	switch(e_cfg.fld().type())
 	{
-	    case TFld::String:	val = Mess->codeConvOut(codepage,e_cfg.getS());	break;		    
+	    case TFld::String:	val = Mess->codeConvOut(codepage,e_cfg.getS());	break;
 	    case TFld::Integer:	val = SYS->int2str(e_cfg.getI());break;
-	    case TFld::Real:    
+	    case TFld::Real:
 	    {
-		char str[200];	    
-		snprintf(str,sizeof(str),"%*.*f",fld_rec->len_fild,fld_rec->dec_field,e_cfg.getR());	
+		char str[200];
+		snprintf(str,sizeof(str),"%*.*f",fld_rec->len_fild,fld_rec->dec_field,e_cfg.getR());
 		val = str;
 		break;
 	    }
@@ -438,9 +454,9 @@ void MTable::fieldSet( TConfig &cfg )
 	
 	//-- Set table volume --
 	if( basa->ModifiFieldIt( i_ln, i_clm,val.c_str() ) < 0 )
-	    throw TError(TSYS::DBInernal,nodePath().c_str(),_("Cell error!"));	    
-    }    
-    
+	    throw TError(TSYS::DBInernal,nodePath().c_str(),_("Cell error!"));
+    }
+
     m_modify = true;
 }
 
@@ -448,32 +464,32 @@ void MTable::fieldDel( TConfig &cfg )
 {
     //- Alloc resource -
     ResAlloc res(m_res,true);
-    
+
     //- Get key line -
     bool i_ok = false;
-    int i_ln;     
+    int i_ln;
     while((i_ln = findKeyLine(cfg)) >= 0)
     {
 	if( basa->DeleteItems(i_ln,1) < 0 )
 	    throw TError(TSYS::DBInernal,nodePath().c_str(),_("Line error!"));
-		
+
 	i_ok = true;
 	m_modify = true;
-    }    
+    }
     if( !i_ok ) throw TError(TSYS::DBInernal,nodePath().c_str(),_("Field no present!"));
 }
 
 int MTable::findKeyLine( TConfig &cfg, int cnt )
 {
     int i_ln, i_clm, i_cnt = 0;
-    
+
     //- Get config fields list -
     vector<string> cf_el;
     cfg.cfgList(cf_el);
     //- Left only keys into list -
     for( int i_cf = 0; i_cf < cf_el.size(); )
 	if( cfg.cfg(cf_el[i_cf]).fld().flg()&TCfg::Key ) i_cf++;
-    	else cf_el.erase(cf_el.begin()+i_cf);
+	else cf_el.erase(cf_el.begin()+i_cf);
 
     //- Find want field -
     for( i_ln = 0; i_ln < basa->GetCountItems(  ); i_ln++ )
@@ -484,7 +500,7 @@ int MTable::findKeyLine( TConfig &cfg, int cnt )
 	    if( !cfg.cfg(cf_el[i_cf]).getS().size() )
 	    {
 		cnt_key++;
-		continue;		
+		continue;
 	    }
 	    //string key = cfg.cfg(cf_el[i_cf]).name();
 	    //- Check key -
@@ -510,37 +526,37 @@ int MTable::findKeyLine( TConfig &cfg, int cnt )
 	    }
 	    cnt_key++;
 	}
-	if(cnt_key && cnt <= i_cnt++) break;	
+	if(cnt_key && cnt <= i_cnt++) break;
     }
     if(i_ln >= basa->GetCountItems(  )) return -1;
-    
+
     return i_ln;
 }
 
 void MTable::fieldPrmSet( TCfg &e_cfg, db_str_rec &n_rec )
 {
     memset(&n_rec,0,sizeof(db_str_rec));
-    
+
     strncpy(n_rec.name,e_cfg.name().c_str(),10);
     switch(e_cfg.fld().type())
     {
 	case TFld::String:
 	    n_rec.tip_fild  = 'C';
 	    n_rec.len_fild  = (e_cfg.fld().len()>255)?255:e_cfg.fld().len();
-	    n_rec.dec_field = 0; 
-	    break;		    
-	case TFld::Integer:     
-	    n_rec.tip_fild = 'N'; 
-	    n_rec.len_fild = (e_cfg.fld().len() == 0)?5:(e_cfg.fld().len()>255)?255:e_cfg.fld().len();
-	    n_rec.dec_field = 0; 
+	    n_rec.dec_field = 0;
 	    break;
-	case TFld::Real:    
-	    n_rec.tip_fild = 'N'; 
+	case TFld::Integer:
+	    n_rec.tip_fild = 'N';
+	    n_rec.len_fild = (e_cfg.fld().len() == 0)?5:(e_cfg.fld().len()>255)?255:e_cfg.fld().len();
+	    n_rec.dec_field = 0;
+	    break;
+	case TFld::Real:
+	    n_rec.tip_fild = 'N';
 	    n_rec.len_fild = (e_cfg.fld().len() == 0)?7:(e_cfg.fld().len()>255)?255:e_cfg.fld().len();
 	    n_rec.dec_field = (e_cfg.fld().dec() == 0)?2:(e_cfg.fld().dec()>255)?255:e_cfg.fld().dec();
 	    break;
 	case TFld::Boolean:
-	    n_rec.tip_fild  = 'L'; 
+	    n_rec.tip_fild  = 'L';
 	    n_rec.len_fild  = 1;
 	    n_rec.dec_field = 0;
 	    break;
