@@ -1,7 +1,7 @@
 
 //OpenSCADA system module BD.DBF file: dbf_mod.h
 /***************************************************************************
- *   Copyright (C) 2003-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -52,52 +52,56 @@ struct Shd
 
 //************************************************
 //* BDDBF::MTable                                *
-//************************************************ 
-class MBD;    
+//************************************************
+class MBD;
 class MTable : public TTable
 {
     public:
 	//Public methods
 	MTable(const string &name, MBD *iown, bool create );
 	~MTable(  );
-	    
+
 	bool fieldSeek( int row, TConfig &cfg );
 	void fieldGet( TConfig &cfg );
 	void fieldSet( TConfig &cfg );
 	void fieldDel( TConfig &cfg );
-	    
+
 	MBD &owner() { return (MBD&)TTable::owner(); }
-	    
+
     private:
 	//Private methods
 	void postDisable(int flag);
-	void save( );	    
+	void save( );
 	int  findKeyLine( TConfig &cfg, int cnt = 0 );
 	void fieldPrmSet( TCfg &e_cfg, db_str_rec &n_rec );
-	    
+
 	//Private atributes
 	string n_table;
 	string codepage;
 	TBasaDBF *basa;
-	    
+
 	Res    m_res;
 	bool   m_modify;
 };
  
 //************************************************
 //* BDDBF::MBD                                   *
-//************************************************ 
-class BDMod;    
+//************************************************
+class BDMod;
 class MBD : public TBD
 {
     public:
 	//Public methods
 	MBD( string name, TElem *cf_el );
 	~MBD(  );
-	    
+
 	void enable( );
 	void allowList( vector<string> &list );
-	    
+
+    protected:
+	//Protected methods
+	void cntrCmdProc( XMLNode *opt );       //Control interface command process
+
     private:
 	//Private methods
 	void postDisable(int flag);
@@ -106,18 +110,18 @@ class MBD : public TBD
 
 //************************************************
 //* BDDBF::BDMod                                 *
-//************************************************ 
+//************************************************
 class BDMod : public TTipBD
 {
     public:
 	//Public methods
 	BDMod( string name );
 	~BDMod(  );
-    
+
     protected:
 	//Protected methods
 	void load_( );
-	    
+
     private:
 	//Private methods
 	TBD *openBD( const string &iid );
