@@ -1,7 +1,7 @@
 
 //OpenSCADA system module BD.FireBird file: firebird.cpp
 /***************************************************************************
- *   Copyright (C) 2007 by Roman Savochenko                                *
+ *   Copyright (C) 2007-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -29,14 +29,14 @@
 
 //******************************************************************************
 //* Modul info!                                                                *
-#define MOD_ID      "FireBird"
-#define MOD_NAME    "DB FireBird"
-#define MOD_TYPE    "BD"
-#define VER_TYPE    VER_BD
-#define VERSION     "0.7.0"
-#define AUTORS      "Roman Savochenko"
-#define DESCRIPTION "DB modul. Allow support of the DB FireBird."
-#define LICENSE     "GPL"
+#define MOD_ID		"FireBird"
+#define MOD_NAME	"DB FireBird"
+#define MOD_TYPE	"BD"
+#define VER_TYPE	VER_BD
+#define VERSION		"0.8.0"
+#define AUTORS		"Roman Savochenko"
+#define DESCRIPTION	"DB modul. Allow support of the DB FireBird."
+#define LICENSE		"GPL"
 //******************************************************************************
 
 FireBird::BDMod *FireBird::mod;
@@ -60,20 +60,20 @@ extern "C"
 using namespace FireBird;
 
 //*************************************************
-//* FireBird::BDMod                               *
+//* FireBird::BDMod				  *
 //*************************************************
 BDMod::BDMod(string name)
 {
     mId 	= MOD_ID;
     mName	= MOD_NAME;
-    mType  	= MOD_TYPE;
-    mVers      	= VERSION;
-    mAutor    	= AUTORS;
-    mDescr  	= DESCRIPTION;
-    mLicense   	= LICENSE;
-    mSource    	= name;
+    mType	= MOD_TYPE;
+    mVers	= VERSION;
+    mAutor	= AUTORS;
+    mDescr	= DESCRIPTION;
+    mLicense	= LICENSE;
+    mSource	= name;
 
-    mod 	= this;
+    mod		= this;
 }
 
 BDMod::~BDMod()
@@ -82,10 +82,10 @@ BDMod::~BDMod()
 }
 
 TBD *BDMod::openBD( const string &iid )
-{    
+{
     return new MBD(iid,&owner().openDB_E());
 }
-	    
+
 string BDMod::optDescr( )
 {
     char buf[STR_BUF_LEN];
@@ -97,7 +97,7 @@ string BDMod::optDescr( )
 	MOD_TYPE,MOD_ID,nodePath().c_str());
 
     return buf;
-}			
+}
 
 void BDMod::load_( )
 {
@@ -120,7 +120,7 @@ void BDMod::load_( )
 	    case -1 : break;
 	}
     } while(next_opt != -1);
-    
+
     //- Load parameters from config file -=
 
 }
@@ -130,11 +130,12 @@ string BDMod::sqlReqCode( const string &req, char symb )
     string sout = req;
     for( int i_sz = 0; i_sz < sout.size(); i_sz++ )
 	if( sout[i_sz] == symb ) sout.replace(i_sz++,1,2,symb);
-    return sout; 	
+    return sout;
 }
 
+
 //************************************************
-//* FireBird::MBD                                *
+//* FireBird::MBD				 *
 //************************************************
 MBD::MBD( const string &iid, TElem *cf_el ) : TBD(iid,cf_el), hdb(0)
 {
@@ -149,18 +150,18 @@ MBD::~MBD( )
 void MBD::postDisable(int flag)
 {
     TBD::postDisable(flag);
-    
+
     if( flag && owner().fullDeleteDB() )
-    {     
+    {
 	//- Attach to DB -
 	//-- Prepare database parameter buffer (DPB) --
 	char  *dpb = (char *)malloc(50);
 	short dpb_length = 0;
 	isc_modify_dpb(&dpb, &dpb_length, isc_dpb_user_name, user.c_str(),user.size());
-	isc_modify_dpb(&dpb, &dpb_length, isc_dpb_password, pass.c_str(),pass.size());    
-	
+	isc_modify_dpb(&dpb, &dpb_length, isc_dpb_password, pass.c_str(),pass.size());
+
 	ISC_STATUS_ARRAY status;
-	isc_tr_handle	trans = 0;	
+	isc_tr_handle trans = 0;
 	if( isc_attach_database( status, 0, fdb.c_str(), &hdb, dpb_length, dpb) ) return;
 	if( isc_drop_database(status, &hdb) ) 
 	    throw TError(TSYS::DBOpen,nodePath().c_str(),_("Drop DB '%s' error: %s"),fdb.c_str(),getErr(status).c_str());
@@ -172,7 +173,7 @@ void MBD::enable( )
     if( enableStat() )  return;
 
     //- DB address parsing -
-    fdb   = TSYS::strSepParse(addr(),0,';');    
+    fdb   = TSYS::strSepParse(addr(),0,';');
     user  = TSYS::strSepParse(addr(),1,';');
     pass  = TSYS::strSepParse(addr(),2,';');
     cd_pg = codePage().size()?codePage():Mess->charset();
@@ -188,13 +189,13 @@ void MBD::enable( )
     if( isc_attach_database( status, 0, fdb.c_str(), &hdb, dpb_length, dpb) )
     {
 	//--- Make try for create DB ---
-	isc_tr_handle	trans = 0;
+	isc_tr_handle trans = 0;
 	if( isc_dsql_execute_immediate(status, &hdb, &trans, 0, 
 		    ("CREATE DATABASE '"+fdb+"' USER '"+user+"' PASSWORD '"+pass+"'").c_str(), 3, NULL) )
 	    throw TError(TSYS::DBOpen,nodePath().c_str(),_("Create DB '%s' error: %s"),fdb.c_str(),getErr(status).c_str());
     }
-    
-    TBD::enable( );    
+
+    TBD::enable( );
 }
 
 void MBD::disable( )
@@ -207,7 +208,7 @@ void MBD::disable( )
     ISC_STATUS_ARRAY status;
     isc_detach_database( status, &hdb );
     hdb = 0;
-}	
+}
 
 void MBD::allowList( vector<string> &list )
 {
@@ -224,39 +225,39 @@ void MBD::allowList( vector<string> &list )
 TTable *MBD::openTable( const string &inm, bool create )
 {
     if( !enableStat() )
-        throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Error open table <%s>. DB disabled."),inm.c_str());
-	    
+	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Error open table <%s>. DB disabled."),inm.c_str());
+
     return new MTable(inm,this,create);
 }
 
 string MBD::getErr( ISC_STATUS_ARRAY status )
 {
-    string err;    
+    string err;
     char msg[512];
-    
+
     err=TSYS::int2str(isc_sqlcode(status))+":";
     while( fb_interpret( msg, sizeof(msg), (const ISC_STATUS**)&status ) )
 	err += string("-")+msg;
-    
+
     return err;
 }
 
 void MBD::transOpen( isc_tr_handle *trans )
 {
     ISC_STATUS_ARRAY status;
-    
+
     ResAlloc res(conn_res,true);
-    if( !trans || *trans )	return;
+    if( !trans || *trans ) return;
     if( isc_start_transaction(status, trans, 1, &hdb, 0, NULL) )
-        throw TError(TSYS::DBRequest,nodePath().c_str(),_("Start trasaction error: %s"),getErr(status).c_str());
+	throw TError(TSYS::DBRequest,nodePath().c_str(),_("Start trasaction error: %s"),getErr(status).c_str());
 }
 
 void MBD::transCommit( isc_tr_handle *trans )
 {
     ISC_STATUS_ARRAY status;
-    
+
     ResAlloc res(conn_res,true);
-    if( !trans || !(*trans) )	return;
+    if( !trans || !(*trans) ) return;
     if( isc_commit_transaction(status, trans) )
 	throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL close transaction error: %s"),getErr(status).c_str());
     *trans = 0;
@@ -268,9 +269,9 @@ void MBD::sqlReq( const string &ireq, vector< vector<string> > *tbl )
 }
 
 void MBD::sqlReq( isc_tr_handle *itrans, const string &ireq, vector< vector<string> > *tbl )
-{    
+{
     if( tbl ) tbl->clear();
-    if(!enableStat())	return; 
+    if(!enableStat()) return;
 
     //printf("TEST 20: %s\n",ireq.c_str());
 
@@ -283,42 +284,42 @@ void MBD::sqlReq( isc_tr_handle *itrans, const string &ireq, vector< vector<stri
     int      dtBufLen = 0;
     isc_stmt_handle stmt = 0;
     isc_tr_handle trans = (itrans && *itrans) ? (*itrans) : 0;
-    ISC_STATUS_ARRAY status;       
+    ISC_STATUS_ARRAY status;
 
     try
     {
-    	//- Prepare statement -
-    	if( isc_dsql_allocate_statement(status, &hdb, &stmt) )
-    	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Allocate statement error: %s"),getErr(status).c_str());
-    	if( !trans && isc_start_transaction(status, &trans, 1, &hdb, 0, NULL) )
-    	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Start trasaction error: %s"),getErr(status).c_str());
-    	//- Prepare output data structure -
-    	if( isc_dsql_prepare(status, &trans, &stmt, 0, Mess->codeConvOut(cd_pg.c_str(),ireq).c_str(), 3, NULL) )
-    	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL prepare error: %s"),getErr(status).c_str());
-    	if( isc_dsql_describe(status, &stmt, 1, out_sqlda) )
-    	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL describe error: %s"),getErr(status).c_str());
-    	if( out_sqlda->sqld > out_sqlda->sqln )
-    	{
-    	    int n = out_sqlda->sqld;
-    	    free(out_sqlda);
-    	    out_sqlda = (XSQLDA *)malloc(XSQLDA_LENGTH(n));
-    	    out_sqlda->sqln = n;
-    	    out_sqlda->version = SQLDA_VERSION1;
-    	    isc_dsql_describe(status, &stmt, 1, out_sqlda);
-    	}
-    	//-- Setup SQLDA --
+	//- Prepare statement -
+	if( isc_dsql_allocate_statement(status, &hdb, &stmt) )
+	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Allocate statement error: %s"),getErr(status).c_str());
+	if( !trans && isc_start_transaction(status, &trans, 1, &hdb, 0, NULL) )
+	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("Start trasaction error: %s"),getErr(status).c_str());
+	//- Prepare output data structure -
+	if( isc_dsql_prepare(status, &trans, &stmt, 0, Mess->codeConvOut(cd_pg.c_str(),ireq).c_str(), 3, NULL) )
+	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL prepare error: %s"),getErr(status).c_str());
+	if( isc_dsql_describe(status, &stmt, 1, out_sqlda) )
+	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL describe error: %s"),getErr(status).c_str());
+	if( out_sqlda->sqld > out_sqlda->sqln )
+	{
+	    int n = out_sqlda->sqld;
+	    free(out_sqlda);
+	    out_sqlda = (XSQLDA *)malloc(XSQLDA_LENGTH(n));
+	    out_sqlda->sqln = n;
+	    out_sqlda->version = SQLDA_VERSION1;
+	    isc_dsql_describe(status, &stmt, 1, out_sqlda);
+	}
+	//-- Setup SQLDA --
 	//--- Data buffer size calc ---
-        for( int i = 0; i < out_sqlda->sqld; i++ )
+	for( int i = 0; i < out_sqlda->sqld; i++ )
 	{
 	    dtBufLen += out_sqlda->sqlvar[i].sqllen + sizeof(short);
-	    if( ((out_sqlda->sqlvar[i].sqltype & ~1) == SQL_VARYING) ) 
+	    if( ((out_sqlda->sqlvar[i].sqltype & ~1) == SQL_VARYING) )
 		dtBufLen += sizeof(short);
 	}
 	if( dtBufLen )
 	{
-	    dtBuf = (char*)malloc(dtBufLen);	    
-    	    for( int i = 0, off = 0; i < out_sqlda->sqld; i++ )
-    	    {
+	    dtBuf = (char*)malloc(dtBufLen);
+	    for( int i = 0, off = 0; i < out_sqlda->sqld; i++ )
+	    {
 		out_sqlda->sqlvar[i].sqldata = dtBuf+off;
 		off += out_sqlda->sqlvar[i].sqllen;
 		if( ((out_sqlda->sqlvar[i].sqltype & ~1) == SQL_VARYING) )
@@ -327,39 +328,39 @@ void MBD::sqlReq( isc_tr_handle *itrans, const string &ireq, vector< vector<stri
 		off += sizeof(short);
 	    }
 	}
-	
-    	//-- Get data --
-    	if( isc_dsql_execute(status, &trans, &stmt, 1, NULL) )
-    	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL execute error: %s"),getErr(status).c_str());
+
+	//-- Get data --
+	if( isc_dsql_execute(status, &trans, &stmt, 1, NULL) )
+	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL execute error: %s"),getErr(status).c_str());
 	if( tbl && out_sqlda->sqld )
 	{
 	    //if( isc_dsql_set_cursor_name(status, &stmt,"dyn_cursor", 0) )
-    	    //	throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL open cursor error: %s"),getErr(status).c_str());	
+	    //  throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL open cursor error: %s"),getErr(status).c_str());
 	    vector<string> row;
-    	    long  fetch_stat;
-    	    while( (fetch_stat = isc_dsql_fetch(status, &stmt, 1, out_sqlda)) == 0 )
+	    long  fetch_stat;
+	    while( (fetch_stat = isc_dsql_fetch(status, &stmt, 1, out_sqlda)) == 0 )
 	    {
-	        //-- Add head --
+		//-- Add head --
 		if( row.empty() )
 		{
-		    for( int i = 0; i < out_sqlda->sqld; i++ )	    
+		    for( int i = 0; i < out_sqlda->sqld; i++ )
 			row.push_back(out_sqlda->sqlvar[i].sqlname);
-		    tbl->push_back(row);		
-		}   
-	        //-- Add row --
+		    tbl->push_back(row);
+		}
+		//-- Add row --
 		row.clear();
-    		for( int i = 0; i < out_sqlda->sqld; i++ )
-    		{
+		for( int i = 0; i < out_sqlda->sqld; i++ )
+		{
 		    XSQLVAR &var = out_sqlda->sqlvar[i];
 		    // Null handling.  If the column is nullable and null
-	    	    if( (var.sqltype & 1) && 
+		    if( (var.sqltype & 1) &&
 			    (*var.sqlind < 0) ) row.push_back("");
 		    switch( var.sqltype & ~1 )
 		    {
 			case SQL_TEXT:
 			    row.push_back(Mess->codeConvIn(cd_pg.c_str(),clrEndSpace(string(var.sqldata,var.sqllen))));
 			    break;
-	        	case SQL_VARYING:
+			case SQL_VARYING:
 			    row.push_back(Mess->codeConvIn(cd_pg.c_str(),string((char*)var.sqldata+sizeof(short),*(short*)var.sqldata)));
 			    break;
 			case SQL_SHORT:
@@ -378,13 +379,13 @@ void MBD::sqlReq( isc_tr_handle *itrans, const string &ireq, vector< vector<stri
 			    row.push_back(TSYS::real2str(*(double *)var.sqldata));
 			    break;
 			case SQL_BLOB:
-			{    
+			{
 			    //- Read blob data -
 			    ISC_QUAD blob_id = *((ISC_QUAD*)var.sqldata);
 			    ISC_STATUS blob_stat;
 			    isc_blob_handle blob_handle = 0;
 			    char blob_segment[STR_BUF_LEN];
-			    unsigned short actual_seg_len;			    
+			    unsigned short actual_seg_len;
 			    string bval;
 			    isc_open_blob2( status, &hdb, &trans, &blob_handle, &blob_id, 0, NULL );
 			    do
@@ -400,29 +401,29 @@ void MBD::sqlReq( isc_tr_handle *itrans, const string &ireq, vector< vector<stri
 			default:
 			    row.push_back("");
 		    }
-    		}
+		}
 		tbl->push_back(row);
-	    }	    
+	    }
 	}
-    	if( isc_dsql_free_statement(status, &stmt, DSQL_drop) )
+	if( isc_dsql_free_statement(status, &stmt, DSQL_drop) )
 	{
 	    stmt = 0;
-    	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL free statement error: %s"),getErr(status).c_str());
+	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL free statement error: %s"),getErr(status).c_str());
 	}
-    	if( (!itrans || !(*itrans)) && isc_commit_transaction(status, &trans) )
+	if( (!itrans || !(*itrans)) && isc_commit_transaction(status, &trans) )
 	{
 	    stmt = trans = 0;
-    	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL close transaction error: %s"),getErr(status).c_str());
+	    throw TError(TSYS::DBRequest,nodePath().c_str(),_("DSQL close transaction error: %s"),getErr(status).c_str());
 	}
     }catch(...)
     {
 	if( stmt ) isc_dsql_free_statement(status, &stmt, DSQL_drop);
-	if( trans && (!itrans || !(*itrans)) )	isc_commit_transaction(status, &trans);
+	if( trans && (!itrans || !(*itrans)) ) isc_commit_transaction(status, &trans);
 	free(out_sqlda);
 	if(dtBuf) free(dtBuf);
 	throw;
     }
-	
+
     free(out_sqlda);
     if(dtBuf) free(dtBuf);
 }
@@ -434,13 +435,31 @@ string MBD::clrEndSpace( const string &vl )
     return vl.substr(0,i+1);
 }
 
+void MBD::cntrCmdProc( XMLNode *opt )
+{
+    //Get page info
+    if( opt->name() == "info" )
+    {
+	TBD::cntrCmdProc(opt);
+	ctrMkNode("fld",opt,-1,"/prm/cfg/addr",cfg("ADDR").fld().descr(),0664,"root","BD",2,
+	    "tp","str","help",
+	    _("FireBird address to DB writed as: [<file>;<user>;<pass>].\n"
+	      "Where:\n"
+	      "  file - full DB file;\n"
+	      "  user - DB user;\n"
+	      "  pass - password of DB user."));
+	return;
+    }
+    TBD::cntrCmdProc(opt);
+}
+
 //************************************************
-//* FireBird::Table                              *
+//* FireBird::Table				 *
 //************************************************
 MTable::MTable(string inm, MBD *iown, bool create ) : TTable(inm), trans(0)
 {
-    setNodePrev(iown);    
-    
+    setNodePrev(iown);
+
     if( create )
     {
 	string req = "EXECUTE BLOCK AS BEGIN "
@@ -450,7 +469,7 @@ MTable::MTable(string inm, MBD *iown, bool create ) : TTable(inm), trans(0)
 	owner().sqlReq(req);
     }
     //- Get table structure description -
-    getStructDB( tblStrct );    
+    getStructDB( tblStrct );
     if( tblStrct.size() <= 1 )
 	throw TError(TSYS::DBOpenTable,nodePath().c_str(),_("Table '%s' no present."),name().c_str());
 }
@@ -461,10 +480,10 @@ MTable::~MTable(  )
 }
 
 void MTable::postDisable(int flag)
-{    
+{
     owner().transCommit(&trans);
     if( flag )
-    {	
+    {
 	try
 	{ owner().sqlReq("DROP TABLE \""+mod->sqlReqCode(name(),'"')+"\""); }
 	catch(TError err) { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
@@ -492,7 +511,7 @@ void MTable::getStructDB( vector< vector<string> > &tblStrct )
 	    int i_k;
 	    for( i_k = 1; i_k < keyLst.size(); i_k++ )
 		if( tblStrct[i_f][0] == keyLst[i_k][0] ) break;
-	    tblStrct[i_f].push_back( (i_k<keyLst.size()) ? keyLst[i_k][1] : "" );		    
+	    tblStrct[i_f].push_back( (i_k<keyLst.size()) ? keyLst[i_k][1] : "" );
 	}
     }
 }
@@ -500,17 +519,17 @@ void MTable::getStructDB( vector< vector<string> > &tblStrct )
 void MTable::fieldStruct( TConfig &cfg )
 {
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty."));
-    
+
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
-        string sid = tblStrct[i_fld][0];
-        if( cfg.cfgPresent(sid) ) continue;
-	
+	string sid = tblStrct[i_fld][0];
+	if( cfg.cfgPresent(sid) ) continue;
+
 	int flg = (tblStrct[i_fld][3]=="PRIMARY KEY") ? (int)TCfg::Key : (int)TFld::NoFlag;
 	if( tblStrct[i_fld][1] == "37" )
-    	    cfg.elem().fldAdd( new TFld(sid.c_str(),sid.c_str(),TFld::String,flg,tblStrct[i_fld][2].c_str()) );
+	    cfg.elem().fldAdd( new TFld(sid.c_str(),sid.c_str(),TFld::String,flg,tblStrct[i_fld][2].c_str()) );
 	else if( tblStrct[i_fld][1] == "261" )
-    	    cfg.elem().fldAdd( new TFld(sid.c_str(),sid.c_str(),TFld::String,flg,"1048576") );
+	    cfg.elem().fldAdd( new TFld(sid.c_str(),sid.c_str(),TFld::String,flg,"1048576") );
 	else if( tblStrct[i_fld][1] == "8" )
 	    cfg.elem().fldAdd( new TFld(sid.c_str(),sid.c_str(),TFld::Integer,flg) );
 	else if( tblStrct[i_fld][1] == "27" )
@@ -523,7 +542,7 @@ void MTable::fieldStruct( TConfig &cfg )
 bool MTable::fieldSeek( int row, TConfig &cfg )
 {
     vector< vector<string> > tbl;
-    
+
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty."));
 
     owner().transCommit(&trans);
@@ -535,23 +554,23 @@ bool MTable::fieldSeek( int row, TConfig &cfg )
     bool next = false;
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
-        string sid = tblStrct[i_fld][0];
-        if( !cfg.cfgPresent(sid) ) continue;
+	string sid = tblStrct[i_fld][0];
+	if( !cfg.cfgPresent(sid) ) continue;
 	TCfg &u_cfg = cfg.cfg(sid);
-    	if( u_cfg.fld().flg()&TCfg::Key && u_cfg.getS().size() )
-    	{
+	if( u_cfg.fld().flg()&TCfg::Key && u_cfg.getS().size() )
+	{
 	    if( !next ) next = true; 
 	    else req_where=req_where+"AND ";
 	    req_where=req_where+"\""+mod->sqlReqCode(sid,'"')+"\"='"+mod->sqlReqCode(u_cfg.getS())+"' ";
 	}
 	else if( u_cfg.fld().flg()&TCfg::Key || u_cfg.view() )
-        {
+	{
 	    if( first_sel ) req=req+"\""+mod->sqlReqCode(sid,'"')+"\"";
 	    else req=req+",\""+mod->sqlReqCode(sid,'"')+"\"";
 	    first_sel = false;
 	}
     }
-    
+
     //- Request -
     req = req+" FROM \""+mod->sqlReqCode(name(),'"')+"\" "+((next)?req_where:"");
     owner().sqlReq( req, &tbl );
@@ -577,7 +596,7 @@ bool MTable::fieldSeek( int row, TConfig &cfg )
 void MTable::fieldGet( TConfig &cfg )
 {
     vector< vector<string> > tbl;
-    
+
     if( tblStrct.empty() ) throw TError(TSYS::DBTableEmpty,nodePath().c_str(),_("Table is empty."));
 
     owner().transCommit(&trans);
@@ -590,7 +609,7 @@ void MTable::fieldGet( TConfig &cfg )
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
     {
 	if( !cfg.cfgPresent(tblStrct[i_fld][0]) ) continue;
-        TCfg &u_cfg = cfg.cfg(tblStrct[i_fld][0]);
+	TCfg &u_cfg = cfg.cfg(tblStrct[i_fld][0]);
 	if( u_cfg.fld().flg()&TCfg::Key )
 	{
 	    if( !next_wr ) next_wr = true; else req_where=req_where+"AND ";
@@ -599,17 +618,17 @@ void MTable::fieldGet( TConfig &cfg )
 	}
 	else if( u_cfg.view() )
 	{
-            if( first_sel ) req=req+"\""+mod->sqlReqCode(tblStrct[i_fld][0],'"')+"\"";
-		    else req=req+",\""+mod->sqlReqCode(tblStrct[i_fld][0],'"')+"\"";
+	    if( first_sel ) req=req+"\""+mod->sqlReqCode(tblStrct[i_fld][0],'"')+"\"";
+	    else req=req+",\""+mod->sqlReqCode(tblStrct[i_fld][0],'"')+"\"";
 	    first_sel = false;
 	}
     }
     req = req+" FROM \""+mod->sqlReqCode(name(),'"')+"\" WHERE "+req_where;
-    
+
     //- Query -
     owner().sqlReq( req, &tbl );
     if( tbl.size() < 2 ) throw TError(TSYS::DBRowNoPresent,nodePath().c_str(),_("Row no present."));
-    
+
     //- Processing of query -
     for( int i_fld = 0; i_fld < tbl[0].size(); i_fld++ )
     {
@@ -629,27 +648,27 @@ void MTable::fieldGet( TConfig &cfg )
 void MTable::fieldSet( TConfig &cfg )
 {
     vector< vector<string> > tbl;
-        
+
     if( tblStrct.empty() ) fieldFix(cfg);
 
     owner().transOpen(&trans);
     //- Get config fields list -
     vector<string> cf_el;
-    cfg.cfgList(cf_el);    
-    
+    cfg.cfgList(cf_el);
+
     //- Get present fields list -
     string req_where = "WHERE ";
     //-- Add key list to queue --
     bool next = false;
     for( int i_el = 0; i_el < cf_el.size(); i_el++ )
     {
-        TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
-        if( !(u_cfg.fld().flg()&TCfg::Key) ) continue;
+	TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
+	if( !(u_cfg.fld().flg()&TCfg::Key) ) continue;
 	if( !next ) next = true; else req_where=req_where+"AND ";
 	req_where=req_where+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\"='"+
 				 mod->sqlReqCode(u_cfg.getS())+"' ";
-    }    
-    
+    }
+
     //- Prepare query -
     //-- Add new record --
     string reqi = "INSERT INTO \""+mod->sqlReqCode(name(),'"')+"\" ";
@@ -660,13 +679,13 @@ void MTable::fieldSet( TConfig &cfg )
 	TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	if( !(u_cfg.fld().flg()&TCfg::Key) && !u_cfg.view() ) continue;
 
-	if( !next ) next = true; 
-	else 
+	if( !next ) next = true;
+	else
 	{
 	    ins_name=ins_name+",";
 	    ins_value=ins_value+",";
 	}
-	ins_name=ins_name+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\" ";		    
+	ins_name=ins_name+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\" ";
 	string val;
 	switch(u_cfg.fld().type())
 	{
@@ -685,10 +704,10 @@ void MTable::fieldSet( TConfig &cfg )
 	string requ = "UPDATE \""+mod->sqlReqCode(name(),'"')+"\" SET ";
 	next = false;
 	for( int i_el = 0; i_el < cf_el.size(); i_el++ )
-        {
-            TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
+	{
+	    TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	    if( u_cfg.fld().flg()&TCfg::Key || !u_cfg.view() ) continue;
-			     
+
 	    if( !next ) next = true; else requ=requ+",";
 	    string val;
 	    switch(u_cfg.fld().type())
@@ -700,10 +719,10 @@ void MTable::fieldSet( TConfig &cfg )
 	    }
 	    requ=requ+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\"='"+mod->sqlReqCode(val)+"' ";
 	}
-    	requ = requ + req_where;
-    	try{ owner().sqlReq( &trans, requ ); }
-    	catch( TError err )	
-	{ 
+	requ = requ + req_where;
+	try{ owner().sqlReq( &trans, requ ); }
+	catch( TError err )
+	{
 	    fieldFix(cfg);
 	    try{ owner().sqlReq( &trans, reqi ); }
 	    catch(TError err)	{ owner().sqlReq( &trans, requ ); }
@@ -718,8 +737,8 @@ void MTable::fieldDel( TConfig &cfg )
     owner().transOpen(&trans);
     //- Get config fields list -
     vector<string> cf_el;
-    cfg.cfgList(cf_el);    
-    
+    cfg.cfgList(cf_el);
+
     //- Prepare request -
     string req = "DELETE FROM \""+mod->sqlReqCode(name(),'"')+"\" WHERE ";
     //-- Add key list to queue --
@@ -729,9 +748,9 @@ void MTable::fieldDel( TConfig &cfg )
 	TCfg &u_cfg = cfg.cfg(cf_el[i_el]);
 	if( u_cfg.fld().flg()&TCfg::Key && u_cfg.getS().size() )
 	{
-	    if( !next ) next = true; else req=req+"AND ";		
+	    if( !next ) next = true; else req=req+"AND ";
 	    req=req+"\""+mod->sqlReqCode(cf_el[i_el],'"')+"\"='"+
-		         mod->sqlReqCode(u_cfg.getS())+"' ";
+			 mod->sqlReqCode(u_cfg.getS())+"' ";
 	}
     }
     owner().sqlReq( &trans, req );
@@ -741,51 +760,51 @@ void MTable::fieldFix( TConfig &cfg )
 {
     bool next = false, next_key = false;
 
-    owner().transCommit(&trans);	
-    
+    owner().transCommit(&trans);
+
     //- Get config fields list -
     vector<string> cf_el;
     cfg.cfgList(cf_el);
-	    
+
     //- Prepare request for fix structure -
     string req = "ALTER TABLE \""+mod->sqlReqCode(name(),'"')+"\" "
     "DROP CONSTRAINT \"pk_"+mod->sqlReqCode(name(),'"')+"\", ";
-			    
+
     string pr_keys;
     for( int i_cf = 0; i_cf < cf_el.size(); i_cf++ )
     {
-        TCfg &u_cfg = cfg.cfg(cf_el[i_cf]);
-        //-- Check primary key --
-        if( u_cfg.fld().flg()&TCfg::Key )
-        {
-            if( !next_key ) next_key = true;
-            else pr_keys=pr_keys+",";
-            pr_keys=pr_keys+"\""+mod->sqlReqCode(u_cfg.name(),'"')+"\"";
-        }
-        int i_fld;
-        for( i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
-            if( cf_el[i_cf] == tblStrct[i_fld][0] )     break;
-        if( i_fld < tblStrct.size() )
-        {
-            string f_tp;
-            switch(u_cfg.fld().type())
-    	    {
-                case TFld::String:
-		    if( tblStrct[i_fld][1] == "37" && (u_cfg.fld().len() <= 255 || u_cfg.fld().flg()&TCfg::Key) && 
+	TCfg &u_cfg = cfg.cfg(cf_el[i_cf]);
+	//-- Check primary key --
+	if( u_cfg.fld().flg()&TCfg::Key )
+	{
+	    if( !next_key ) next_key = true;
+	    else pr_keys=pr_keys+",";
+	    pr_keys=pr_keys+"\""+mod->sqlReqCode(u_cfg.name(),'"')+"\"";
+	}
+	int i_fld;
+	for( i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
+	    if( cf_el[i_cf] == tblStrct[i_fld][0] )     break;
+	if( i_fld < tblStrct.size() )
+	{
+	    string f_tp;
+	    switch(u_cfg.fld().type())
+	    {
+		case TFld::String:
+		    if( tblStrct[i_fld][1] == "37" && (u_cfg.fld().len() <= 255 || u_cfg.fld().flg()&TCfg::Key) &&
 			    u_cfg.fld().len() == atoi(tblStrct[i_fld][2].c_str()) )
 			continue;
 		    else if( tblStrct[i_fld][1] == "261" && u_cfg.fld().len() > 255 )	continue;
-            	    break;
-        	case TFld::Integer:	if( tblStrct[i_fld][1] == "8" )	continue;	break;
-                case TFld::Real:	if( tblStrct[i_fld][1] == "27" )continue;	break;
-                case TFld::Boolean:	if( tblStrct[i_fld][1] == "7" ) continue;	break;
-            }
-    	    //-- Change field --
-            if( !next ) next = true; else req=req+",";
-            req=req+"DROP \""+mod->sqlReqCode(cf_el[i_cf],'"')+"\" ";
-        }
+		    break;
+		case TFld::Integer:	if( tblStrct[i_fld][1] == "8" )	continue;	break;
+		case TFld::Real:	if( tblStrct[i_fld][1] == "27" )continue;	break;
+		case TFld::Boolean:	if( tblStrct[i_fld][1] == "7" ) continue;	break;
+	    }
+	    //-- Change field --
+	    if( !next ) next = true; else req=req+",";
+	    req=req+"DROP \""+mod->sqlReqCode(cf_el[i_cf],'"')+"\" ";
+	}
 	if( !next ) next = true; else req=req+",";
-    	//-- Add field --
+	//-- Add field --
 	req=req+"ADD \""+mod->sqlReqCode(cf_el[i_cf],'"')+"\" ";
 	switch(u_cfg.fld().type())
 	{
@@ -795,30 +814,30 @@ void MTable::fieldFix( TConfig &cfg )
 		else  req=req+"BLOB SUB_TYPE TEXT DEFAULT '"+u_cfg.fld().def()+"' NOT NULL ";
 		break;
 	    case TFld::Integer:
- 		req=req+"INTEGER DEFAULT '"+TSYS::int2str(atoi(u_cfg.fld().def().c_str()))+"' NOT NULL ";
+		req=req+"INTEGER DEFAULT '"+TSYS::int2str(atoi(u_cfg.fld().def().c_str()))+"' NOT NULL ";
+		break;
+	    case TFld::Real:
+		req=req+"DOUBLE PRECISION DEFAULT '"+TSYS::real2str(atof(u_cfg.fld().def().c_str()))+"' NOT NULL ";
 		break;
-            case TFld::Real:
- 		req=req+"DOUBLE PRECISION DEFAULT '"+TSYS::real2str(atof(u_cfg.fld().def().c_str()))+"' NOT NULL ";
-                break;
 	    case TFld::Boolean:
-            	req=req+"SMALLINT DEFAULT '"+TSYS::int2str(atoi(u_cfg.fld().def().c_str()))+"' NOT NULL ";
-                break;
-    	}
+		req=req+"SMALLINT DEFAULT '"+TSYS::int2str(atoi(u_cfg.fld().def().c_str()))+"' NOT NULL ";
+		break;
+	}
     }
     //- DROP fields -
     for( int i_fld = 1; i_fld < tblStrct.size(); i_fld++ )
-        if( !cfg.cfgPresent(tblStrct[i_fld][0]) )
-        {
-            if( !next ) next = true; else req=req+",";
-    	    req=req+"DROP \""+mod->sqlReqCode(tblStrct[i_fld][0],'"')+"\" ";
-        }
+	if( !cfg.cfgPresent(tblStrct[i_fld][0]) )
+	{
+	    if( !next ) next = true; else req=req+",";
+	    req=req+"DROP \""+mod->sqlReqCode(tblStrct[i_fld][0],'"')+"\" ";
+	}
     req=req+", ADD CONSTRAINT \"pk_"+mod->sqlReqCode(name(),'"')+"\" PRIMARY KEY ("+pr_keys+") ";
 
     if( next )
     {
-        owner().sqlReq( req );
+	owner().sqlReq( req );
 	getStructDB( tblStrct );
     }
 
     owner().transOpen(&trans);
-}    
+}
