@@ -1,7 +1,7 @@
 
 //OpenSCADA system module DAQ.SNMP file: snmp.cpp
 /***************************************************************************
- *   Copyright (C) 2006-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2006-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -39,14 +39,14 @@
 
 //*************************************************
 //* Modul info!                                   *
-#define MOD_ID      "SNMP"
-#define MOD_NAME    "SNMP client"
-#define MOD_TYPE    "DAQ"
-#define VER_TYPE    VER_CNTR
-#define VERSION     "0.3.1"
-#define AUTORS      "Roman Savochenko"
-#define DESCRIPTION "Allow realising of SNMP client service."
-#define LICENSE     "GPL"
+#define MOD_ID		"SNMP"
+#define MOD_NAME	"SNMP client"
+#define MOD_TYPE	"DAQ"
+#define VER_TYPE	VER_CNTR
+#define VERSION		"0.3.2"
+#define AUTORS		"Roman Savochenko"
+#define DESCRIPTION	"Allow realising of SNMP client service."
+#define LICENSE		"GPL"
 //*************************************************
 
 SNMP_DAQ::TTpContr *SNMP_DAQ::mod;  //Pointer for direct access to module
@@ -61,7 +61,7 @@ extern "C"
 
     TModule *attach( const TModule::SAt &AtMod, const string &source )
     {
-    	if( AtMod == TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE) )
+	if( AtMod == TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE) )
 	    return new SNMP_DAQ::TTpContr( source );
 	return NULL;
     }
@@ -70,19 +70,19 @@ extern "C"
 using namespace SNMP_DAQ;
 
 //*************************************************
-//* TTpContr                                      * 
+//* TTpContr                                      *
 //*************************************************
-TTpContr::TTpContr( string name )  
+TTpContr::TTpContr( string name )
 {
-    mId 	= MOD_ID;
-    mName       = MOD_NAME;
-    mType  	= MOD_TYPE;
-    mVers      	= VERSION;
-    mAutor    	= AUTORS;
-    mDescr  	= DESCRIPTION;
-    mLicense   	= LICENSE;
-    mSource    	= name;    
-    
+    mId		= MOD_ID;
+    mName	= MOD_NAME;
+    mType	= MOD_TYPE;
+    mVers	= VERSION;
+    mAutor	= AUTORS;
+    mDescr	= DESCRIPTION;
+    mLicense	= LICENSE;
+    mSource	= name;
+
     mod		= this;
 
     //- Once init of Net-SNMP -
@@ -90,7 +90,7 @@ TTpContr::TTpContr( string name )
 }
 
 TTpContr::~TTpContr()
-{    
+{
 
 }
 
@@ -102,7 +102,7 @@ string TTpContr::optDescr( )
 	"======================= The module <%s:%s> options =======================\n"
 	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
 	MOD_TYPE,MOD_ID,nodePath().c_str());
-    
+
     return buf;
 }
 
@@ -110,7 +110,7 @@ void TTpContr::load_( )
 {
     //- Load parameters from command line -
     int next_opt;
-    char *short_opt="h";
+    const char *short_opt="h";
     struct option long_opt[] =
     {
 	{"help"    ,0,NULL,'h'},
@@ -130,7 +130,7 @@ void TTpContr::load_( )
 }
 
 void TTpContr::postEnable( int flag )
-{    
+{
     TModule::postEnable(flag);
 
     //- Controler's bd structure -
@@ -140,7 +140,7 @@ void TTpContr::postEnable( int flag )
     fldAdd( new TFld("ADDR",_("Remote host address"),TFld::String,TFld::NoFlag,"30","localhost") );
     fldAdd( new TFld("COMM",_("Server community"),TFld::String,TFld::NoFlag,"20","public") );
     fldAdd( new TFld("PATTR_LIM",_("Param's attributes limit"),TFld::Integer,TFld::NoFlag,"3","100") );
-    
+
     //- Parameter type bd structure -
     int t_prm = tpParmAdd("std","PRM_BD",_("Standard"));
     tpPrmAt(t_prm).fldAdd( new TFld("OID_LS",_("OID list (next line separated)"),TFld::String,TFld::FullText|TCfg::NoVal,"100","") );
@@ -158,7 +158,7 @@ TMdContr::TMdContr( string name_c, const string &daq_db, ::TElem *cfgelem) :
 	::TController(name_c,daq_db,cfgelem), prc_st(false), endrun_req(false), tm_gath(0),
 	m_per(cfg("PERIOD").getId()), m_prior(cfg("PRIOR").getId()), m_addr(cfg("ADDR").getSd()), m_comm(cfg("COMM").getSd()),
 	m_pattr_lim(cfg("PATTR_LIM").getId())
-{    
+{
     cfg("PRM_BD").setS("SNMPPrm_"+name_c);
 }
 
@@ -173,7 +173,7 @@ TParamContr *TMdContr::ParamAttach( const string &name, int type )
 }
 
 void TMdContr::start_( )
-{      
+{
     //- Start the gathering data task -
     if( !prc_st )
     {
@@ -184,27 +184,27 @@ void TMdContr::start_( )
 	    pthread_attr_setschedpolicy(&pthr_attr,SCHED_RR);
 	else pthread_attr_setschedpolicy(&pthr_attr,SCHED_OTHER);
 	prior.__sched_priority=m_prior;
-        pthread_attr_setschedparam(&pthr_attr,&prior);
-	
-        pthread_create(&procPthr,&pthr_attr,TMdContr::Task,this);
-        pthread_attr_destroy(&pthr_attr);
-        if( TSYS::eventWait(prc_st, true, nodePath()+"start",5) )
-            throw TError(nodePath().c_str(),_("Gathering task no started!"));    
+	pthread_attr_setschedparam(&pthr_attr,&prior);
+
+	pthread_create(&procPthr,&pthr_attr,TMdContr::Task,this);
+	pthread_attr_destroy(&pthr_attr);
+	if( TSYS::eventWait(prc_st, true, nodePath()+"start",5) )
+	    throw TError(nodePath().c_str(),_("Gathering task no started!"));
     }
 }
 
 void TMdContr::stop_( )
-{  
+{
     //- Stop the request and calc data task -
     if( prc_st )
     {
-        endrun_req = true;
-        pthread_kill( procPthr, SIGALRM );
-        if( TSYS::eventWait(prc_st,false,nodePath()+"stop",5) )
-            throw TError(nodePath().c_str(),_("Gathering task no stoped!"));
-        pthread_join( procPthr, NULL );
+	endrun_req = true;
+	pthread_kill( procPthr, SIGALRM );
+	if( TSYS::eventWait(prc_st,false,nodePath()+"stop",5) )
+	    throw TError(nodePath().c_str(),_("Gathering task no stoped!"));
+	pthread_join( procPthr, NULL );
     }
-} 
+}
 
 void TMdContr::prmEn( const string &id, bool val )
 {
@@ -212,8 +212,8 @@ void TMdContr::prmEn( const string &id, bool val )
 
     ResAlloc res(en_res,true);
     for( i_prm = 0; i_prm < p_hd.size(); i_prm++)
-        if( p_hd[i_prm].at().id() == id ) break;
-    
+	if( p_hd[i_prm].at().id() == id ) break;
+
     if( val && i_prm >= p_hd.size() )	p_hd.push_back(at(id));
     if( !val && i_prm < p_hd.size() )	p_hd.erase(p_hd.begin()+i_prm);
 }
@@ -224,9 +224,9 @@ void *TMdContr::Task( void *icntr )
     struct timespec get_tm;
     TMdContr &cntr = *(TMdContr *)icntr;
 
-    int	   el_cnt;    
-    string soid; 
- 
+    int		el_cnt;
+    string	soid;
+
     //- Start SNMP-net session -
     struct snmp_session session;
     struct snmp_pdu *response;
@@ -235,21 +235,21 @@ void *TMdContr::Task( void *icntr )
     session.version = SNMP_VERSION_1;
     session.community = (u_char*)cntr.m_comm.c_str();
     session.community_len = strlen((char *)session.community);
-    session.peername = (char *)cntr.m_addr.c_str();    
+    session.peername = (char *)cntr.m_addr.c_str();
     void *ss =  snmp_sess_open(&session);
-    if(!ss)	
+    if( !ss )
     {
 	mess_err(mod->nodePath().c_str(),"%s",_("Error SNMP session open."));
 	return NULL;
     }
-    
+
     cntr.endrun_req = false;
     cntr.prc_st = true;
-    
+
     while(!cntr.endrun_req)
-    {	
+    {
 	long long t_cnt = SYS->shrtCnt();
-	
+
 	//-- Update controller's data --
 	el_cnt = 0;
 	cntr.en_res.resRequestR( );
@@ -258,28 +258,28 @@ void *TMdContr::Task( void *icntr )
 	    {
 		oid oid_root[MAX_OID_LEN], oid_next[MAX_OID_LEN];
 		size_t oid_root_len = MAX_OID_LEN, oid_next_len = MAX_OID_LEN;
-	    
+
 		TMdPrm &cprm = cntr.p_hd[i_p].at();
-	    
+
 		for(int ioid = 0; ioid < cprm.lsOID().size();ioid++)
 		{
 		    oid_root_len=oid_next_len=cprm.lsOID()[ioid].size()/sizeof(oid);
 		    memmove(oid_root,cprm.lsOID()[ioid].c_str(),oid_root_len*sizeof(oid));
 		    memmove(oid_next,oid_root,oid_root_len*sizeof(oid));
-		    
+
 		    bool running = true;
 		    while(running && (el_cnt++) < cntr.pAttrLimit())
-		    {		    
+		    {
 			struct snmp_pdu *pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
 			snmp_add_null_var(pdu, oid_next, oid_next_len);
 			int status = snmp_sess_synch_response(ss,pdu,&response);
 			if(status == STAT_SUCCESS)
-	    		{
+			{
 			    if(response->errstat == SNMP_ERR_NOERROR)
 				for (vars = response->variables; vars; vars = vars->next_variable)
-	                	{
+				{
 				    if((vars->name_length < oid_root_len) || (memcmp(oid_root,vars->name,oid_root_len*sizeof(oid)) != 0))
-	                	    {
+				    {
 					running = 0;
 					continue;
 				    }
@@ -314,51 +314,51 @@ void *TMdContr::Task( void *icntr )
 					case ASN_COUNTER:
 					    cprm.vlAt(soid).at().setR(*(unsigned long*)vars->val.integer,0,true);
 					    break;
-					default:    
+					default:
 					    //printf("TEST 02: %xh\n",vars->type);
 					    //print_objid(vars->name,vars->name_length);
 					    //print_value(vars->name,vars->name_length,vars);
 					    break;
 				    }
-				    //print_variable(vars->name, vars->name_length, vars);				    
+				    //print_variable(vars->name, vars->name_length, vars);
 				    //--------------------
 				    if( vars->type!=SNMP_ENDOFMIBVIEW && vars->type!=SNMP_NOSUCHOBJECT && vars->type!=SNMP_NOSUCHINSTANCE )
-	                	    {
-                        		memmove((char*)oid_next,(char*)vars->name,vars->name_length*sizeof(oid));
-	                        	oid_next_len = vars->name_length;
+				    {
+					memmove((char*)oid_next,(char*)vars->name,vars->name_length*sizeof(oid));
+					oid_next_len = vars->name_length;
 				    }
 				    else running = 0;
 				}
-		    	    else running = 0;
+			    else running = 0;
 			}
 			else if (status == STAT_TIMEOUT)
 			{
 			    mess_err(mod->nodePath().c_str(),_("Timeout: No Response from %s."),session.peername);
-	        	    running = 0;
+			    running = 0;
 			}
 			else running = 0;
 			if(response) snmp_free_pdu(response);
-	    	    }
+		    }
 		}
 	    }
 	    catch(TError err)
-    	    { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
-	cntr.en_res.resReleaseR( );    
+	    { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
+	cntr.en_res.resReleaseR( );
 	cntr.tm_gath = 1.0e3*((double)(SYS->shrtCnt()-t_cnt))/((double)SYS->sysClk());
-    
-        //- Calc next work time and sleep -
-        clock_gettime(CLOCK_REALTIME,&get_tm);
-        work_tm = (((long long)get_tm.tv_sec*1000000000+get_tm.tv_nsec)/((long long)cntr.m_per*1000000000) + 1)*(long long)cntr.m_per*1000000000;
-	if(last_tm == work_tm)	work_tm+=(long long)cntr.m_per*1000000000;	//Fix early call
+
+	//- Calc next work time and sleep -
+	clock_gettime(CLOCK_REALTIME,&get_tm);
+	work_tm = (((long long)get_tm.tv_sec*1000000000+get_tm.tv_nsec)/((long long)cntr.period()*1000000000) + 1)*(long long)cntr.period()*1000000000;
+	if(last_tm == work_tm)	work_tm+=(long long)cntr.period()*1000000000;	//Fix early call
 	last_tm = work_tm;
-        get_tm.tv_sec = work_tm/1000000000; get_tm.tv_nsec = work_tm%1000000000;
-        clock_nanosleep(CLOCK_REALTIME,TIMER_ABSTIME,&get_tm,NULL);
+	get_tm.tv_sec = work_tm/1000000000; get_tm.tv_nsec = work_tm%1000000000;
+	clock_nanosleep(CLOCK_REALTIME,TIMER_ABSTIME,&get_tm,NULL);
     }
-    
+
     snmp_sess_close(ss);
-    
+
     cntr.prc_st = false;
-    
+
     return NULL;
 }
 
@@ -396,7 +396,7 @@ TMdPrm::TMdPrm( string name, TTipParam *tp_prm ) :
 
 TMdPrm::~TMdPrm( )
 {
-    nodeDelAll();    
+    nodeDelAll();
 }
 
 void TMdPrm::postEnable( int flag )
@@ -407,10 +407,10 @@ void TMdPrm::postEnable( int flag )
 
 void TMdPrm::enable()
 {
-    if( enableStat() )	return;    
+    if( enableStat() )	return;
 
     TParamContr::enable();
-            
+
     owner().prmEn( id(), true );
 }
 
@@ -419,9 +419,9 @@ void TMdPrm::disable()
     if( !enableStat() )  return;
 
     owner().prmEn( id(), false );
-    
+
     TParamContr::disable();
-    
+
     //- Set EVAL to parameter attributes -
     vector<string> ls;
     elem().fldList(ls);
@@ -432,26 +432,26 @@ void TMdPrm::disable()
 void TMdPrm::load_( )
 {
     TParamContr::load_();
-    
+
     parseOIDList(m_oid);
 }
 
 void TMdPrm::parseOIDList(const string &ioid)
 {
     m_oid = ioid;
-    
+
     oid tmpoid[MAX_OID_LEN];
     size_t tmpoid_len = MAX_OID_LEN;
-    
+
     ls_oid.clear();
-    
+
     string sel;
     for( int ioff = 0; (sel=TSYS::strSepParse(m_oid,0,'\n',&ioff)).size(); )
     {
 	tmpoid_len = MAX_OID_LEN;
 	if(snmp_parse_oid(sel.c_str(),tmpoid,&tmpoid_len))
 	    ls_oid.push_back(string((char*)tmpoid,tmpoid_len*sizeof(oid)));
-    }    
+    }
 }
 
 void TMdPrm::cntrCmdProc( XMLNode *opt )
@@ -467,13 +467,13 @@ void TMdPrm::cntrCmdProc( XMLNode *opt )
 	ctrMkNode("fld",opt,-1,"/prm/cfg/OID_LS",cfg("OID_LS").fld().descr(),enableStat()?0444:0664);
 	return;
     }
-    
+
     //- Process command to page -
     if( a_path == "/prm/cfg/OID_LS" && ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )
     {
 	if( enableStat() )	throw TError(nodePath().c_str(),"Parameter is enabled.");
 	parseOIDList(opt->text());
-    }	
+    }
     else TParamContr::cntrCmdProc(opt);
 }
 
