@@ -1,7 +1,7 @@
 
 //OpenSCADA system module DAQ.SNMP file: snmp_client.h
 /***************************************************************************
- *   Copyright (C) 2006-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2006-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,7 +18,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
- 
+
 #ifndef SNMP_DAQ_H
 #define SNMP_DAQ_H
 
@@ -42,14 +42,14 @@ namespace SNMP_DAQ
 //* TMdPrm                                        *
 //*************************************************
 class TMdContr;
-    
+
 class TMdPrm : public TParamContr
 {
     public:
 	//Methods
-    	TMdPrm( string name, TTipParam *tp_prm );
-	~TMdPrm( );	
-	
+	TMdPrm( string name, TTipParam *tp_prm );
+	~TMdPrm( );
+
 	string &OIDList( )	{ return m_oid; }
 	vector<string> &lsOID( ){ return ls_oid; }
 	TElem &elem( )		{ return p_el; }
@@ -65,17 +65,17 @@ class TMdPrm : public TParamContr
 
     private:
 	//Methods
-        void postEnable( int flag );
+	void postEnable( int flag );
 	void cntrCmdProc( XMLNode *opt );
 	void vlArchMake( TVal &val );
-	
+
 	void parseOIDList( const string &ioid );
-	
-        //Attributes
-	string	&m_oid;	//Gaher OID list. Single text
-	vector<string>	ls_oid;	//Parsed IOD list.
-	
-        TElem   p_el;           //Work atribute elements
+
+	//Attributes
+	string	&m_oid;			//Gaher OID list. Single text
+	vector<string>	ls_oid;		//Parsed IOD list.
+
+	TElem	p_el;			//Work atribute elements
 };
 
 //*************************************************
@@ -86,44 +86,44 @@ class TMdContr: public TController
     friend class TMdPrm;
     public:
 	//Methods
-    	TMdContr( string name_c, const string &daq_db, ::TElem *cfgelem );
-	~TMdContr( );   
+	TMdContr( string name_c, const string &daq_db, ::TElem *cfgelem );
+	~TMdContr( );
 
-	double period( )	{ return vmax(m_per,0.1); }
-        int    prior( )		{ return m_prior; }
-	int    pAttrLimit( )	{ return m_pattr_lim; }
+	double	period( )	{ return vmax(m_per,0.1); }
+	int	prior( )	{ return m_prior; }
+	int	pAttrLimit( )	{ return m_pattr_lim; }
 
 	AutoHD<TMdPrm> at( const string &nm )	{ return TController::at(nm); }
-	
+
     protected:
 	//Methods
 	void prmEn( const string &id, bool val );
 	void start_( );
-	void stop_( );	
-    	void cntrCmdProc( XMLNode *opt );       //Control interface command process
-	
+	void stop_( );
+	void cntrCmdProc( XMLNode *opt );	//Control interface command process
+
     private:
 	//Methods
 	TParamContr *ParamAttach( const string &name, int type );
 	static void *Task( void *icntr );
-	
+
 	string oid2str( oid *ioid, size_t isz );
-	
+
 	//Attributes
-	Res	en_res;         //Resource for enable params
-	int	&m_per,     	// s
+	Res	en_res;		//Resource for enable params
+	int	&m_per,		// s
 		&m_prior,	// Process task priority
 		&m_pattr_lim;	// Parameter's attributes limit
 	string	&m_addr,	// Host address
-		&m_comm;	// Server community		
-		
-	bool    prc_st,		// Process task active
+		&m_comm;	// Server community
+
+	bool	prc_st,		// Process task active
 		endrun_req;	// Request to stop of the Process task
-        vector< AutoHD<TMdPrm> >  p_hd;
-	
-	pthread_t procPthr;     // Process task thread
-	
-	double 	tm_gath;	// Gathering time
+	vector< AutoHD<TMdPrm> >  p_hd;
+
+	pthread_t procPthr;	// Process task thread
+
+	double	tm_gath;	// Gathering time
 };
 
 //*************************************************
@@ -133,11 +133,11 @@ class TTpContr: public TTipDAQ
 {
     public:
 	//Methods
-    	TTpContr( string name );
+	TTpContr( string name );
 	~TTpContr( );
 
     protected:
-	//Methods	
+	//Methods
 	void postEnable( int flag );
 	void load_( );
 
@@ -149,6 +149,6 @@ class TTpContr: public TTipDAQ
 
 extern TTpContr *mod;
 
-} //End namespace 
+} //End namespace
 
 #endif //SNMP_DAQ_H
