@@ -1,7 +1,7 @@
 
 //OpenSCADA system module Protocol.HTTP file: http.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,8 +18,9 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
- 
+
 #include <getopt.h>
+#include <string.h>
 
 #include <config.h>
 #include <tsys.h>
@@ -30,14 +31,14 @@
 
 //*************************************************
 //* Modul info!                                   *
-#define MOD_ID      "HTTP"
-#define MOD_NAME    "HTTP-realisation"
-#define MOD_TYPE    "Protocol"
-#define VER_TYPE    VER_PROT
-#define M_VERSION   "1.3.1"
-#define AUTORS      "Roman Savochenko"
-#define DESCRIPTION "Allow support HTTP for WWW based UIs."
-#define LICENSE     "GPL"
+#define MOD_ID		"HTTP"
+#define MOD_NAME	"HTTP-realisation"
+#define MOD_TYPE	"Protocol"
+#define VER_TYPE	VER_PROT
+#define M_VERSION	"1.3.2"
+#define AUTORS		"Roman Savochenko"
+#define DESCRIPTION	"Allow support HTTP for WWW based UIs."
+#define LICENSE		"GPL"
 //*************************************************
 
 PrHTTP::TProt *PrHTTP::mod;
@@ -53,7 +54,7 @@ extern "C"
     TModule *attach( const TModule::SAt &AtMod, const string &source )
     {
 	if( AtMod == TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE) )
-    	    return new PrHTTP::TProt( source );
+	    return new PrHTTP::TProt( source );
 	return NULL;
     }
 }
@@ -65,15 +66,15 @@ using namespace PrHTTP;
 //*************************************************
 TProt::TProt( string name )
 {
-    mId 	= MOD_ID;
-    mType  	= MOD_TYPE;
-    mName       = MOD_NAME;
-    mVers      	= M_VERSION;
-    mAutor    	= AUTORS;
-    mDescr  	= DESCRIPTION;
-    mLicense   	= LICENSE;
-    mSource    	= name;
-    
+    mId		= MOD_ID;
+    mType	= MOD_TYPE;
+    mName	= MOD_NAME;
+    mVers	= M_VERSION;
+    mAutor	= AUTORS;
+    mDescr	= DESCRIPTION;
+    mLicense	= LICENSE;
+    mSource	= name;
+
     mod		= this;
 }
 
@@ -86,8 +87,8 @@ string TProt::optDescr( )
 {
     char buf[STR_BUF_LEN];
     snprintf(buf,sizeof(buf),_(
-        "======================= The module <%s:%s> options =======================\n"
-        "---------- Parameters of the module section <%s> in config file ----------\n\n"),
+	"======================= The module <%s:%s> options =======================\n"
+	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
 	MOD_TYPE,MOD_ID,nodePath().c_str());
 
     return(buf);
@@ -97,7 +98,7 @@ void TProt::load_( )
 {
     //- Load parameters from command line -
     int next_opt;
-    char *short_opt="h";
+    const char *short_opt="h";
     struct option long_opt[] =
     {
 	{"help"    ,0,NULL,'h'},
@@ -150,32 +151,32 @@ string TProtIn::http_head( const string &rcode, int cln, const string &addattr )
 bool TProtIn::mess( const string &reqst, string &answer, const string &sender )
 {
     string req;
-    vector<string> vars;    
-    
+    vector<string> vars;
+
     //- Continue for full reqst -
-    if( m_nofull ) 
-    {    
+    if( m_nofull )
+    {
 	m_buf = m_buf+reqst;
 	m_nofull = false;
     }
-    else m_buf=reqst;  //Save request to bufer    
+    else m_buf=reqst;  //Save request to bufer
 
     string request = m_buf;
-    
+
     answer = "";
     if( request.size() > 0 )
     {
 	int    pos = 0;
 	//mess_debug("DEBUG","Content: <%s>!",request.c_str());
-	
-	//- Parse first record -	
+
+	//- Parse first record -
 	req = TSYS::strSepParse(request,0,'\n',&pos);
 	if( !req.empty() ) req.resize(req.size()-1);
 	string method   = TSYS::strSepParse(req,0,' ');
 	string url      = TSYS::strSepParse(req,1,' ');
 	string protocol = TSYS::strSepParse(req,2,' ');
 
-	//- Parse parameters -	
+	//- Parse parameters -
 	int c_lng=-1;
 	while( true )
 	{
@@ -187,11 +188,11 @@ bool TProtIn::mess( const string &reqst, string &answer, const string &sender )
 	    else if( strcasecmp(var.c_str(),"content-length") == 0 )	c_lng = atoi(TSYS::strSepParse(req,1,':').c_str());
 	    vars.push_back( req );
 	}
-	
+
 	//- Check content length -
 	if( (c_lng >= 0 && c_lng > (request.size()-pos)) || (c_lng < 0 && method == "POST") ) m_nofull = true;
 	if( m_nofull ) return m_nofull;
-	
+
 	//- Check protocol version -
 	if( protocol != "HTTP/1.0" && protocol != "HTTP/1.1" )
 	{
@@ -201,38 +202,38 @@ bool TProtIn::mess( const string &reqst, string &answer, const string &sender )
 		     "  <p>This server did not undersand your request.</p>\n"
 		     " </body>\n"
 		     "</html>\n";
-	    answer = http_head("400 Bad Request",answer.size())+answer;				    
+	    answer = http_head("400 Bad Request",answer.size())+answer;
 	    return m_nofull;
 	}
-	
+
 	//- Send request to module -
 	int url_pos = 0;
 	string name_mod = TSYS::pathLev(url,0,false,&url_pos);
 	while( url_pos < url.size() && url[url_pos] == '/' ) url_pos++;
 	url = "/"+url.substr(url_pos);
-	//if( url.empty() )	url = "/";	
-        try
-	{ 
+	//if( url.empty() )	url = "/";
+	try
+	{
 	    AutoHD<TModule> mod = SYS->ui().at().modAt(name_mod);
 	    if( mod.at().modInfo("SubType") != "WWW" )
 		throw TError(nodePath().c_str(),"Find no WWW subtype module!");
-	    
-    	    //- Check metods -
-    	    if( method == "GET" )
-    	    {
+
+	    //- Check metods -
+	    if( method == "GET" )
+	    {
 		void(TModule::*HttpGet)( const string &url, string &page, const string &sender, vector<string> &vars);
-	 	mod.at().modFunc("void HttpGet(const string&,string&,const string&,vector<string>&);",
+		mod.at().modFunc("void HttpGet(const string&,string&,const string&,vector<string>&);",
 		    (void (TModule::**)()) &HttpGet);
-		
-		((&mod.at())->*HttpGet)(url,answer,sender,vars);		
+
+		((&mod.at())->*HttpGet)(url,answer,sender,vars);
 		//mess_debug("DEBUG","Get Content: <%s>!",request.c_str());
 	    }
-	    else if( method == "POST" ) 
+	    else if( method == "POST" )
 	    {
 		void(TModule::*HttpPost)( const string &url, string &page, const string &sender, vector<string> &vars, const string &contain );
-	 	mod.at().modFunc("void HttpPost(const string&,string&,const string&,vector<string>&,const string&);",
-		    (void (TModule::**)()) &HttpPost);		
-		    
+		mod.at().modFunc("void HttpPost(const string&,string&,const string&,vector<string>&,const string&);",
+		    (void (TModule::**)()) &HttpPost);
+
 		((&mod.at())->*HttpPost)(url,answer,sender,vars,request.substr(pos));
 		//mess_debug(nodePath().c_str(),"Post Content: <%s>!",request.c_str());
 	    }
@@ -244,10 +245,10 @@ bool TProtIn::mess( const string &reqst, string &answer, const string &sender )
 			 "  <p>The method "+method+" is not implemented by this server.</p>\n"
 			 " </body>\n"
 			 "</html>\n";
-		answer = http_head("501 Method Not Implemented",answer.size())+answer;				
+		answer = http_head("501 Method Not Implemented",answer.size())+answer;
 	    }
 	}
-	catch(TError err){ index(answer); }	
+	catch(TError err){ index(answer); }
     }
 
     return m_nofull;
@@ -260,12 +261,12 @@ string TProtIn::w_head( )
 	"<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Transitional//EN'\n"
 	"'DTD/xhtml1-transitional.dtd'>\n"
 	"<html xmlns='http://www.w3.org/1999/xhtml'>\n<head>\n"
-    	"<meta http-equiv='Content-Type' content='text/html; charset="+Mess->charset()+"'/>\n"
-    	"<title>"+PACKAGE_NAME+"!"+"</title>\n"
+	"<meta http-equiv='Content-Type' content='text/html; charset="+Mess->charset()+"'/>\n"
+	"<title>"+PACKAGE_NAME+"!"+"</title>\n"
 	"</head>\n"
-        "<body bgcolor='#818181' text='#000000' link='#3366ff' vlink='#339999' alink='#33ccff'>\n"
-    	"<h1 align='center'><font color='#ffff00'>"+PACKAGE_NAME+"!</font></h1>\n"
-    	"<hr width='100%' size='3'/><br/>\n<br/><br/>\n";
+	"<body bgcolor='#818181' text='#000000' link='#3366ff' vlink='#339999' alink='#33ccff'>\n"
+	"<h1 align='center'><font color='#ffff00'>"+PACKAGE_NAME+"!</font></h1>\n"
+	"<hr width='100%' size='3'/><br/>\n<br/><br/>\n";
 }
 
 string TProtIn::w_tail()
@@ -274,10 +275,10 @@ string TProtIn::w_tail()
 	"<hr width='100%' size='2'/>\n"
 	"</body>\n"
 	"</html>\n";
-}             
+}
 
 void TProtIn::index( string &answer )
-{ 
+{
     answer = w_head()+
 	    "<table border='2' align='center' width='40%' bgcolor='#A9A9A9'>\n"
 	    "<tr bgcolor='#9999ff'><td><b>"+_("Present web modules")+"</b></td></tr>\n"
@@ -289,7 +290,7 @@ void TProtIn::index( string &answer )
 	AutoHD<TModule> mod = owner().owner().owner().ui().at().modAt(list[i_l]);
 	if( mod.at().modInfo("SubType") == "WWW" )
 	    answer = answer+"<li><a href='"+list[i_l]+"'>"+mod.at().modInfo("Name")+"</a></li>\n";
-    }     
+    }
     answer = answer+"</ul></td></tr></table>\n"+w_tail();
     answer = http_head("200 OK",answer.size())+answer;
 }
