@@ -1,7 +1,7 @@
 
 //OpenSCADA system module Transport.Sockets file: socket.h
 /***************************************************************************
- *   Copyright (C) 2003-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -43,99 +43,99 @@ namespace Sockets
 class TSocketIn;
 
 //************************************************
-//* Sockets::SSockIn                             *
+//* Sockets::SSockIn				 *
 //************************************************
-class SSockIn 
+class SSockIn
 {
     public:
 	SSockIn( TSocketIn *is_in, int icl_sock, string isender ) :
 	    s_in(is_in), cl_sock(icl_sock), sender(isender)	{  }
-	    
-	TSocketIn *s_in;
-	int       cl_sock;
-	string    sender;
+
+	TSocketIn	*s_in;
+	int		cl_sock;
+	string		sender;
 };
 
 //************************************************
-//* Sockets::SSockCl                             *
+//* Sockets::SSockCl				 *
 //************************************************
 struct SSockCl
 {
-    pid_t  cl_id;    // Client's pids
-    int    cl_sock;
+    pid_t	cl_id;    // Client's pids
+    int		cl_sock;
 };
 
 //************************************************
-//* Sockets::TSocketIn                           *
+//* Sockets::TSocketIn				 *
 //************************************************
 class TSocketIn: public TTransportIn
 {
     public:
 	/* Open input socket <name> for locale <address>
 	 * address : <type:<specific>>
-	 * type: 
+	 * type:
 	 *   TCP  - TCP socket with  "UDP:<host>:<port>"
 	 *   UDP  - UDP socket with  "TCP:<host>:<port>"
 	 *   UNIX - UNIX socket with "UNIX:<path>"
 	 */
-	TSocketIn(string name,const string &idb,TElem *el);
+	TSocketIn( string name, const string &idb, TElem *el );
 	~TSocketIn( );
-	
+
 	int maxQueue( )		{ return max_queue; }
 	int maxFork( )		{ return max_fork; }
 	int bufLen( )		{ return buf_len; }
-	
+
 	void setMaxQueue( int vl )	{ max_queue = vl; modif(); }
 	void setMaxFork( int vl )	{ max_fork = vl; modif(); }
 	void setBufLen( int vl )	{ buf_len = vl; modif(); }
-	
+
 	void start( );
 	void stop( );
-	    
+
     private:
 	//Methods
-	static void *Task(void *);
-	static void *ClTask(void *);
-	
+	static void *Task( void * );
+	static void *ClTask( void * );
+
 	void ClSock( SSockIn &s_in );
 	void PutMess( int sock, string &request, string &answer, string sender, AutoHD<TProtocolIn> &prot_in );
 
-	void RegClient(pthread_t thrid, int i_sock);
-	void UnregClient(pthread_t thrid);
-	    
-	void cntrCmdProc( XMLNode *opt );       //Control interface command process
-	    
+	void RegClient( pthread_t thrid, int i_sock );
+	void UnregClient( pthread_t thrid );
+
+	void cntrCmdProc( XMLNode *opt );	//Control interface command process
+
 	//Attributes
-	pthread_t pthr_tsk;
-	int       sock_fd;
-	Res       sock_res;
-	
-	bool      endrun;      // Command for stop task	    
-	bool      endrun_cl;   // Command for stop client tasks
+	pthread_t	pthr_tsk;
+	int		sock_fd;
+	Res		sock_res;
 
-	int       &max_queue;   // max queue for TCP, UNIX sockets
-	int       &max_fork;    // maximum forking (opened sockets)
-	int       &buf_len;     // input buffer length	
+	bool		endrun;			// Command for stop task
+	bool		endrun_cl;		// Command for stop client tasks
 
-	int       type;        // socket's types 
-	string    path;        // path to file socket for UNIX socket
-	string    host;        // host for TCP/UDP sockets
-	string    port;        // port for TCP/UDP sockets
-	int       mode;        // mode for TCP/UNIX sockets (0 - no hand; 1 - hand connect)
+	int		&max_queue;		// max queue for TCP, UNIX sockets
+	int		&max_fork;		// maximum forking (opened sockets)
+	int		&buf_len;		// input buffer length
 
-	bool            cl_free;  // Clients stoped
-	vector<SSockCl> cl_id;    // Client's pids
+	int		type;			// socket's types
+	string		path;			// path to file socket for UNIX socket
+	string		host;			// host for TCP/UDP sockets
+	string		port;			// port for TCP/UDP sockets
+	int		mode;			// mode for TCP/UNIX sockets (0 - no hand; 1 - hand connect)
+
+	bool		cl_free;		// Clients stoped
+	vector<SSockCl>	cl_id;			// Client's pids
 };
 
 //************************************************
-//* Sockets::TSocketOut                          *
+//* Sockets::TSocketOut				 *
 //************************************************
 class TSocketOut: public TTransportOut
 {
     public:
 	/* Open output socket <name> for locale <address>
 	 * address : <type:<specific>>
-	 * type: 
+	 * type:
 	 *   TCP  - TCP socket with  "UDP:<host>:<port>"
 	 *   UDP  - UDP socket with  "TCP:<host>:<port>"
 	 *   UNIX - UNIX socket with "UNIX:<path>"
@@ -149,42 +149,45 @@ class TSocketOut: public TTransportOut
 	int messIO( const char *obuf, int len_ob, char *ibuf = NULL, int len_ib = 0, int time = 0 );
 
     private:
-	int	sock_fd;
-    
-	int	type;        // socket's types 
-	struct sockaddr_in  name_in;
-	struct sockaddr_un  name_un;
-	Res  	wres;
+	//Methods
+	void cntrCmdProc( XMLNode *opt );	//Control interface command process
+
+	//Attributes
+	int			sock_fd;
+
+	int			type;		// socket's types
+	struct sockaddr_in	name_in;
+	struct sockaddr_un	name_un;
+	Res			wres;
 };
 
 //************************************************
-//* Sockets::TTransSock                          *
+//* Sockets::TTransSock				 *
 //************************************************
 class TTransSock: public TTipTransport
 {
     public:
 	TTransSock( string name );
 	~TTransSock( );
-	    
+
 	TTransportIn  *In( const string &name, const string &idb );
 	TTransportOut *Out( const string &name, const string &idb );
-	
+
     protected:
 	void load_( );
 
-    private:	
+    private:
 	//Methods
 	string optDescr( );
 	void cntrCmdProc( XMLNode *opt );       //Control interface command process
-	    
+
 	void postEnable( int flag );
-	    
+
 	//Attributes
-	static const char *i_cntr; 
+	static const char	*i_cntr;
 };
-    
-extern TTransSock *mod;    
+
+extern TTransSock *mod;
 }
 
 #endif //SOCKET_H
-
