@@ -1,7 +1,7 @@
 
 //OpenSCADA system module UI.QTCfg file: selfwidg.h
 /***************************************************************************
- *   Copyright (C) 2004-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2004-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -38,7 +38,7 @@ using std::vector;
 
 class QPaintEvent;
 class QComboBox;
-class QLineEdit;    
+class QLineEdit;
 class QHBoxLayout;
 class QTextEdit;
 class QDialogButtonBox;
@@ -47,83 +47,47 @@ class QGridLayout;
 namespace QTCFG
 {
 
-//*************************************************
-//* LineEdit: Line edit widget                    *
-//*************************************************
+//*********************************************************************************************
+//* Universal edit line widget. Contain support of: QLineEdit, QSpinBox, QDoubleSpinBox,      *
+//* QTimeEdit, QDateEdit and QDateTimeEdit.                                                   *
+//*********************************************************************************************
 class LineEdit : public QWidget
 {
     Q_OBJECT
-	
+
     public:
-	//Methods
-	LineEdit( QWidget *parent, bool prev_dis = false );
-	
-	QString text( ) const;	
-	bool hasFocus( ) const;
-	bool isChanged( );
+	//- Data -
+	enum LType { Text, Integer, Real, Time, Date, DateTime, Combo };
 
-	void setText( const QString & );
+	//- Methods -
+	LineEdit( QWidget *parent, LType tp = Text, bool prev_dis = false );
 
-	QLineEdit *edit( )	{ return ed_fld; }
+	LType type( )		{ return m_tp; }
+	QString value( );
+	bool isEdited( );
+
+	void setType( LType tp );
+	void setValue( const QString& );
+	void setCfg( const QString& );
+
+	QWidget *workWdg( )	{ return ed_fld; }
 
     signals:
-	//Signals
 	void apply( );
 	void cancel( );
-	void textChanged(const QString&);
-	    
+	void valChanged( const QString& );
+
     protected:
-	//Methods
 	bool event( QEvent * e );
 
     private slots:
-	//Private slots
-	void changed( const QString& );
+	void changed( );
 	void applySlot( );
-	    
+
     private:
-	//Attributes
-	QLineEdit	*ed_fld;
-	QPushButton	*bt_fld;
-}; 
-    
-//*************************************************
-//* DateTimeEdit: Date and time edit widget       *
-//*************************************************
-class DateTimeEdit : public QWidget
-{
-    Q_OBJECT
-    
-    public:
-	//Methods
-	DateTimeEdit( QWidget *parent, bool prev_dis = false );
-	    
-	QDateTime dateTime( ) const;
-	bool hasFocus( ) const;
-	bool isChanged( );
-
-	void setDateTime( const QDateTime & dt );
-
-	QDateTimeEdit *dtEdit( ) { return ed_fld; }
-
-    signals:
-	//Signals
-	void apply( );
-	void cancel( );
-	void valueChanged( const QDateTime& );
-	    
-    protected:
-	//Methods
-	bool event( QEvent * e );	    
-		
-    private slots:
-	//Private slots
-	void changed( const QDateTime & );
-	void applySlot( );
-	    
-    private:
-	//Attributes
-	QDateTimeEdit 	*ed_fld;
+	LType		m_tp;
+	QString		m_val;
+	QWidget		*ed_fld;
 	QPushButton	*bt_fld;
 };
 
@@ -133,7 +97,7 @@ class DateTimeEdit : public QWidget
 class TextEdit : public QWidget
 {
     Q_OBJECT
-    
+
     public:
 	//Methods
 	TextEdit( QWidget *parent, const char * name = 0, bool prev_dis = false );
@@ -151,20 +115,20 @@ class TextEdit : public QWidget
 	void apply( );
 	void cancel( );
 	void textChanged( const QString& );
-	    
+
     private slots:
 	//Private slots
 	void changed( );
 	void btApply( );
 	void btCancel( );
-	    
+
     private:
 	//Private attributes
 	bool	isInit;
 	QTextEdit	*ed_fld;
 	QDialogButtonBox *but_box;
 };
-    
+
 //************************************************
 //* ImgView: Image view widget                   *
 //************************************************
@@ -175,15 +139,15 @@ class ImgView : public QWidget
 	//Methods
 	ImgView( QWidget * parent = 0, Qt::WindowFlags f = 0, int ih_sz = 0, int iv_sz = 0 );
 	~ImgView( );
-	    
+
 	QImage &image( )	{ return m_img; }
-	
+
 	bool setImage( const string &imgdata );
 
     protected:
 	//Methods
 	void paintEvent( QPaintEvent * );
-	
+
     private:
 	//Attributes
 	QImage m_img;
@@ -197,9 +161,9 @@ class InputDlg : public QDialog
 {
     public:
 	//Methods
-	InputDlg( QWidget *parent, const QIcon &icon, const QString &mess, 
+	InputDlg( QWidget *parent, const QIcon &icon, const QString &mess,
 		const QString &ndlg = "QTCfg dialog", bool with_id = false, bool with_nm = true );
-	    
+
 	QString id( );
 	QString name( );
 	QString mess( );
@@ -209,13 +173,13 @@ class InputDlg : public QDialog
 	void setMess( const QString &val );
 
 	//Attributes
-	QGridLayout 	*ed_lay;
+	QGridLayout	*ed_lay;
 
     private:
 	//Attributes
-	QLabel      	*inp_lab;
+	QLabel		*inp_lab;
 	QLineEdit	*m_id, *m_name;
-};    
+};
 
 //************************************************
 //* DlgUser: User select dialog                  *
@@ -227,23 +191,23 @@ class DlgUser : public QDialog
     public:
 	//Data
 	enum Results { SelCancel, SelOK, SelErr };
-	
+
 	//Methods
 	DlgUser( QWidget * parent = 0 );
-            
+
 	QString user( );
 	QString password( );
-        
+
     private slots:
 	//Private slots
 	void finish( int result );
-        
+
     private:
 	//Attributes
-	QComboBox   *users;
-	QLineEdit   *passwd;
+	QComboBox	*users;
+	QLineEdit	*passwd;
 };
-    
+
 //************************************************
 //* UserStBar: Status bar user widget            *
 //************************************************
@@ -254,20 +218,20 @@ class UserStBar : public QLabel
     public:
 	//Methods
 	UserStBar( const QString &iuser, QWidget * parent = 0 );
-			    
+
 	QString user( );
-	bool userSel( );	
-	
+	bool userSel( );
+
 	void setUser( const QString &val );
-	
+
     signals:
 	//Signals
 	void userChanged( );
-	
+
     protected:
 	//Methods
 	bool event( QEvent *event );
-					    
+
     private:
 	//Attributes
 	QString     user_txt;
@@ -281,20 +245,20 @@ class UserStBar : public QLabel
 class TableDelegate : public QItemDelegate
 {
     Q_OBJECT
-	
+
     public:
 	//- Public attributes -
 	TableDelegate( QObject *parent = 0 );
-	    
+
 	void paint( QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index ) const;
-	
+
 	QWidget *createEditor( QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index ) const;
-					  
-    	void setEditorData( QWidget *editor, const QModelIndex &index ) const;
-        void setModelData( QWidget *editor, QAbstractItemModel *model, const QModelIndex &index ) const;
-									
+
+	void setEditorData( QWidget *editor, const QModelIndex &index ) const;
+	void setModelData( QWidget *editor, QAbstractItemModel *model, const QModelIndex &index ) const;
+
 	void updateEditorGeometry( QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index ) const;
-    
+
     private:
 	//- Private attributes -
 	bool eventFilter( QObject *object, QEvent *event );
