@@ -1,7 +1,7 @@
 
 //OpenSCADA system module UI.VCAEngine file: origwidg.cpp
 /***************************************************************************
- *   Copyright (C) 2006-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2006-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -23,14 +23,14 @@
 
 #include "origwidg.h"
 #include "vcaengine.h"
-		  
+
 using namespace VCA;
 
 //*************************************************
 //* PrWidget: Primitive widget template           *
 //*************************************************
 PrWidget::PrWidget( const string &iid ) : LWidget(iid)
-{ 
+{
 
 }
 
@@ -38,36 +38,36 @@ void PrWidget::preDisable(int flag)
 {
     if( flag )
 	throw TError(mod->nodePath().c_str(),_("Deleting of main primitive widgets is error."));
-    
+
     Widget::preDisable(flag);
 }
-    
+
 string PrWidget::ico( )
 {
     if( LWidget::ico().size() )
-        return LWidget::ico();
+	return LWidget::ico();
     if( TUIS::icoPresent("VCA.wdg_"+id()) )
-        return TSYS::strEncode(TUIS::icoGet("VCA.wdg_"+id()),TSYS::base64);
+	return TSYS::strEncode(TUIS::icoGet("VCA.wdg_"+id()),TSYS::base64);
     return "";
 }
 
 void PrWidget::setEnable( bool val )
 {
     if( enable() == val ) return;
-    
+
     LWidget::setEnable( val );
-    
+
     //- Init active attributes -
     if( val )
     {
 	vector<string> ls;
 	attrList(ls);
 	for(int i_l = 0; i_l < ls.size(); i_l++)
-            if( attrAt(ls[i_l]).at().flgGlob()&Attr::Active )
+	    if( attrAt(ls[i_l]).at().flgGlob()&Attr::Active )
 	    {
 		attrAt(ls[i_l]).at().setS(attrAt(ls[i_l]).at().getS(),true);
 		attrList(ls);
-	    }    
+	    }
     }
 }
 
@@ -76,16 +76,16 @@ bool PrWidget::cntrCmdGeneric( XMLNode *opt )
     //- Get page info -
     if( opt->name() == "info" )
     {
-        Widget::cntrCmdGeneric(opt);
-        ctrMkNode("oscada_cntr",opt,-1,"/",_("Base widget: ")+id());
-	ctrMkNode("fld",opt,-1,"/wdg/st/parent",_("Parent"),R_R_R_,user().c_str(),grp().c_str(),
-		2,"tp","str","dest","");		
-        return true;
+	Widget::cntrCmdGeneric(opt);
+	ctrMkNode("oscada_cntr",opt,-1,"/",_("Base widget: ")+id());
+	ctrMkNode("fld",opt,-1,"/wdg/st/parent",_("Parent"),R_R_R_,owner().c_str(),grp().c_str(),
+		2,"tp","str","dest","");
+	return true;
     }
-    
+
     //- Process command to page -
     string a_path = opt->attr("path");
-    if( a_path == "/wdg/st/parent" && ctrChkNode(opt,"get",R_R_R_,user().c_str(),grp().c_str(),SEQ_RD) )
+    if( a_path == "/wdg/st/parent" && ctrChkNode(opt,"get",R_R_R_,owner().c_str(),grp().c_str(),SEQ_RD) )
 	opt->setText(parentNm());
     else if( Widget::cntrCmdGeneric(opt) ) return true;
     else return false;
@@ -94,15 +94,15 @@ bool PrWidget::cntrCmdGeneric( XMLNode *opt )
 void PrWidget::cntrCmdProc( XMLNode *opt )
 {
     if( cntrCmdServ(opt) ) return;
-    
+
     //- Get page info -
     if( opt->name() == "info" )
     {
-        cntrCmdGeneric(opt);
-        cntrCmdAttributes(opt );
-        return;
+	cntrCmdGeneric(opt);
+	cntrCmdAttributes(opt );
+	return;
     }
-    
+
     //- Process command to page -
     if( !(cntrCmdGeneric(opt) || cntrCmdAttributes(opt)) )
 	TCntrNode::cntrCmdProc(opt);
@@ -122,30 +122,31 @@ OrigElFigure::OrigElFigure( ) : PrWidget("ElFigure")
 
 }
 
-string OrigElFigure::name( )      
-{ 
-    return _("Elementary figures"); 
+string OrigElFigure::name( )
+{
+    return _("Elementary figures");
 }
 
-string OrigElFigure::descr( )     
-{ 
-    return _("Elementary figures widget of the end visualisation."); 
+string OrigElFigure::descr( )
+{
+    return _("Elementary figures widget of the end visualisation.");
 }
 
 void OrigElFigure::postEnable( int flag )
 {
     LWidget::postEnable(flag);
-    
+
     if( flag&TCntrNode::NodeConnect )
     {
 	attrAdd( new TFld("lineWdth",_("Line:width"),TFld::Integer,TFld::NoFlag,"2","1","0;99","","20") );
 	attrAdd( new TFld("lineClr",_("Line:color"),TFld::String,Attr::Color,"20","#000000","","","21") );
 	attrAdd( new TFld("lineDecor",_("Line:decorate"),TFld::Integer,TFld::Selected,"1","0","0;1",_("No decor;Pipe"),"22") );
+	attrAdd( new TFld("lineStyle",_("Line:style"),TFld::String,TFld::Selected,"10","solid","solid;dashed;dotted",_("Solid;Dashed;Dotted"),"29") );
 	attrAdd( new TFld("bordWdth",_("Border:width"),TFld::Integer,TFld::NoFlag,"2","0","0;99","","23") );
 	attrAdd( new TFld("bordClr",_("Border:color"),TFld::String,Attr::Color,"20","#000000","","","24") );
 	attrAdd( new TFld("fillColor",_("Fill:color"),TFld::String,Attr::Color,"20","","","","25") );
 	attrAdd( new TFld("fillImg",_("Fill:image"),TFld::String,Attr::Image,"20","","","","26") );
-        attrAdd( new TFld("orient",_("Orientation angle"),TFld::Integer,TFld::NoFlag,"3","0","-180;180","","28") );
+	attrAdd( new TFld("orient",_("Orientation angle"),TFld::Integer,TFld::NoFlag,"3","0","-180;180","","28") );
 	attrAdd( new TFld("elLst",_("Element's list"),TFld::String,TFld::FullText|Attr::Active,"300","","","","27") );
     }
 }
@@ -156,28 +157,28 @@ bool OrigElFigure::attrChange( Attr &cfg, void *prev )
     {
 	string sel;
 	string ls_prev = *(string*)prev;
-	vector<int> pntls, pntls_prev;	
+	vector<int> pntls, pntls_prev;
 	int p[5];
-	//- Parse last attributes list and make point list -	
+	//- Parse last attributes list and make point list -
 	for( int off = 0; (sel=TSYS::strSepParse(ls_prev,0,'\n',&off)).size(); )
 	{
 	    for( int i_p = 0; i_p < sizeof(p)/sizeof(int); i_p++ ) p[i_p]=-1;
-	    sscanf(sel.c_str(),"line:%d:%d",&p[0],&p[1]) || 
+	    sscanf(sel.c_str(),"line:%d:%d",&p[0],&p[1]) ||
 	    sscanf(sel.c_str(),"arc:%d:%d:%d:%d:%d",&p[0],&p[1],&p[2],&p[3],&p[4]) ||
 	    sscanf(sel.c_str(),"bezier:%d:%d:%d:%d",&p[0],&p[1],&p[2],&p[3]);
-	    for( int i_p = 0; i_p < sizeof(p)/sizeof(int); i_p++ ) 
+	    for( int i_p = 0; i_p < sizeof(p)/sizeof(int); i_p++ )
 		if( p[i_p]>=0 ) pntls_prev.push_back(p[i_p]);
 	}
 	//- Parse new attributes list and update point's attributes -
 	for( int off = 0; (sel=TSYS::strSepParse(cfg.getS(),0,'\n',&off)).size(); )
 	{
-	    for( int i_p = 0; i_p < sizeof(p)/sizeof(int); i_p++ ) p[i_p]=-1;	
-	    sscanf(sel.c_str(),"line:%d:%d",&p[0],&p[1]) || 
+	    for( int i_p = 0; i_p < sizeof(p)/sizeof(int); i_p++ ) p[i_p]=-1;
+	    sscanf(sel.c_str(),"line:%d:%d",&p[0],&p[1]) ||
 	    sscanf(sel.c_str(),"arc:%d:%d:%d:%d:%d",&p[0],&p[1],&p[2],&p[3],&p[4]) ||
 	    sscanf(sel.c_str(),"bezier:%d:%d:%d:%d",&p[0],&p[1],&p[2],&p[3]);
 	    for( int i_p = 0; i_p < sizeof(p)/sizeof(int); i_p++ )
 		if( p[i_p] >= 0 )
-		{		    
+		{
 		    pntls.push_back(p[i_p]);
 		    if( cfg.owner()->attrPresent("p"+TSYS::int2str(p[i_p])+"x") )	continue;
 		    cfg.owner()->attrAdd( new TFld(("p"+TSYS::int2str(p[i_p])+"x").c_str(),(_("Point ")+TSYS::int2str(p[i_p])+":x").c_str(),
@@ -198,7 +199,7 @@ bool OrigElFigure::attrChange( Attr &cfg, void *prev )
 		cfg.owner()->attrDel("p"+TSYS::int2str(pntls_prev[i_p])+"y");
 	    }
 	}
-    }    
+    }
     return Widget::attrChange(cfg,prev);
 }
 
@@ -209,26 +210,26 @@ OrigFormEl::OrigFormEl( ) : PrWidget("FormEl")
 {
 
 }
-							  
-string OrigFormEl::name( )      
-{ 
-    return _("Form's elements"); 
+
+string OrigFormEl::name( )
+{
+    return _("Form's elements");
 }
 
-string OrigFormEl::descr( )     
-{ 
-    return _("Form's elements widget of the end visualisation."); 
+string OrigFormEl::descr( )
+{
+    return _("Form's elements widget of the end visualisation.");
 }
 
 void OrigFormEl::postEnable( int flag )
 {
     LWidget::postEnable(flag);
-    
+
     if( flag&TCntrNode::NodeConnect )
 	attrAdd( new TFld("elType",_("Element type"),TFld::Integer,TFld::Selected|Attr::Active,"2","0","0;1;2;3;4;5;6;7",
-                    	       _("Line edit;Text edit;Chek box;Button;Combo box;List;Slider;Scroll Bar"),"20") );
+				    _("Line edit;Text edit;Chek box;Button;Combo box;List;Slider;Scroll Bar"),"20") );
 }
-								  
+
 bool OrigFormEl::attrChange( Attr &cfg, void *prev )
 {
     if( cfg.flgGlob()&Attr::Active && cfg.id() == "elType" )
@@ -240,7 +241,7 @@ bool OrigFormEl::attrChange( Attr &cfg, void *prev )
 		cfg.owner()->attrDel("value");
 		cfg.owner()->attrDel("view");
 		cfg.owner()->attrDel("cfg");
-		cfg.owner()->attrDel("font");		
+		cfg.owner()->attrDel("font");
 		break;
 	    case 1:	//Text edit
 		cfg.owner()->attrDel("value");
@@ -250,25 +251,27 @@ bool OrigFormEl::attrChange( Attr &cfg, void *prev )
 	    case 2:	//Check box
 		cfg.owner()->attrDel("value");
 		cfg.owner()->attrDel("font");
+		cfg.owner()->attrAt("name").at().fld().setReserve("");
 		break;
 	    case 3:	//Button
 		cfg.owner()->attrDel("value");
-	        cfg.owner()->attrDel("img");
+		cfg.owner()->attrDel("img");
 		cfg.owner()->attrDel("color");
 		cfg.owner()->attrDel("checkable");
-		cfg.owner()->attrDel("font");		
+		cfg.owner()->attrDel("font");
+		cfg.owner()->attrAt("name").at().fld().setReserve("");
 		break;
 	    case 4: case 5:	//Combo box and list
 		cfg.owner()->attrDel("value");
 		cfg.owner()->attrDel("items");
-		cfg.owner()->attrDel("font");		
+		cfg.owner()->attrDel("font");
 		break;
 	    case 6: case 7:	//Slider and scroll bar
 		cfg.owner()->attrDel("value");
 		cfg.owner()->attrDel("cfg");
 		break;
 	}
-	
+
 	//- Create specific attributes -
 	switch(cfg.getI())
 	{
@@ -277,7 +280,7 @@ bool OrigFormEl::attrChange( Attr &cfg, void *prev )
 		cfg.owner()->attrAdd( new TFld("view",_("View"),TFld::Integer,TFld::Selected|Attr::Mutable|Attr::Active,
 		    "1","0","0;1;2;3;4;5;6",_("Text;Combo;Integer;Real;Time;Data;Data and time"),"22") );
 		cfg.owner()->attrAdd( new TFld("cfg",_("Config"),TFld::String,TFld::FullText|Attr::Mutable,"","","","","23") );
-		cfg.owner()->attrAdd( new TFld("font",_("Font"),TFld::String,Attr::Font,"50","Arial 11","","","25") );		
+		cfg.owner()->attrAdd( new TFld("font",_("Font"),TFld::String,Attr::Font,"50","Arial 11","","","25") );
 		break;
 	    case 1:	//Text edit
 		cfg.owner()->attrAdd( new TFld("value",_("Value"),TFld::String,TFld::FullText|Attr::Mutable,"","","","","21") );
@@ -287,6 +290,7 @@ bool OrigFormEl::attrChange( Attr &cfg, void *prev )
 	    case 2:	//Check box
 		cfg.owner()->attrAdd( new TFld("value",_("Value"),TFld::Boolean,Attr::Mutable,"","","","","21") );
 		cfg.owner()->attrAdd( new TFld("font",_("Font"),TFld::String,Attr::Font,"50","Arial 11","","","25") );
+		cfg.owner()->attrAt("name").at().fld().setReserve("26");
 		break;
 	    case 3:	//Button
 		cfg.owner()->attrAdd( new TFld("value",_("Value"),TFld::Boolean,Attr::Mutable,"","","","","21") );
@@ -294,6 +298,7 @@ bool OrigFormEl::attrChange( Attr &cfg, void *prev )
 		cfg.owner()->attrAdd( new TFld("color",_("Color"),TFld::String,Attr::Color|Attr::Mutable,"20","","","","23") );
 		cfg.owner()->attrAdd( new TFld("checkable",_("Checkable"),TFld::Boolean,Attr::Mutable,"","","","","24") );
 		cfg.owner()->attrAdd( new TFld("font",_("Font"),TFld::String,Attr::Font,"50","Arial 11","","","25") );
+		cfg.owner()->attrAt("name").at().fld().setReserve("26");
 		break;
 	    case 4: case 5:	//Combo box and list
 		cfg.owner()->attrAdd( new TFld("value",_("Value"),TFld::String,Attr::Mutable,"200","","","","21") );
@@ -309,9 +314,9 @@ bool OrigFormEl::attrChange( Attr &cfg, void *prev )
     else if( cfg.flgGlob()&Attr::Active && cfg.id() == "view" )
     {
 	TFld::Type	ntp = TFld::String;
-	int 		flg = Attr::Mutable;
+	int		flg = Attr::Mutable;
 	VCA::Attr::SelfAttrFlgs	sflg = cfg.owner()->attrAt("value").at().flgSelf();
-	string 		val = cfg.owner()->attrAt("value").at().getS();
+	string		val = cfg.owner()->attrAt("value").at().getS();
 	string		cfgTmpl = cfg.owner()->attrAt("value").at().cfgTempl( );
 	string		cfgVal = cfg.owner()->attrAt("value").at().cfgVal( );
 	switch(cfg.getI())
@@ -328,65 +333,65 @@ bool OrigFormEl::attrChange( Attr &cfg, void *prev )
 	cfg.owner()->attrAt("value").at().setCfgTempl(cfgTmpl);
 	cfg.owner()->attrAt("value").at().setCfgVal(cfgVal);
     }
-    
+
     return Widget::attrChange(cfg,prev);
 }
 
 //************************************************
 //* OrigText: Text element original widget       *
 //************************************************
-OrigText::OrigText( ) : PrWidget("Text")	
-{ 
-
-}
-    
-string OrigText::name( )      
-{ 
-    return _("Text fields"); 
-}
-
-string OrigText::descr( )	
+OrigText::OrigText( ) : PrWidget("Text")
 {
-    return _("Text fields widget of the end visualisation."); 
+
+}
+
+string OrigText::name( )
+{
+    return _("Text fields");
+}
+
+string OrigText::descr( )
+{
+    return _("Text fields widget of the end visualisation.");
 }
 
 void OrigText::postEnable( int flag )
 {
     LWidget::postEnable(flag);
-    
+
     if( flag&TCntrNode::NodeConnect )
-    { 
-        attrAdd( new TFld("backColor",_("Background:color"),TFld::String,Attr::Color,"","","","","20") );
-        attrAdd( new TFld("backImg",_("Background:image"),TFld::String,Attr::Image,"","","","","21") );
-        attrAdd( new TFld("bordWidth",_("Border:width"),TFld::Integer,TFld::NoFlag,"","0","","","22") );
-        attrAdd( new TFld("bordColor",_("Border:color"),TFld::String,Attr::Color,"","#000000","","","23") );
-        attrAdd( new TFld("bordStyle",_("Border:style"),TFld::Integer,TFld::Selected,"","3","0;1;2;3;4;5;6;7;8",
-						_("None;Dotted;Dashed;Solid;Double;Groove;Ridge;Inset;Outset"),"19") );	
+    {
+	attrAdd( new TFld("backColor",_("Background:color"),TFld::String,Attr::Color,"","","","","20") );
+	attrAdd( new TFld("backImg",_("Background:image"),TFld::String,Attr::Image,"","","","","21") );
+	attrAdd( new TFld("bordWidth",_("Border:width"),TFld::Integer,TFld::NoFlag,"","0","","","22") );
+	attrAdd( new TFld("bordColor",_("Border:color"),TFld::String,Attr::Color,"","#000000","","","23") );
+	attrAdd( new TFld("bordStyle",_("Border:style"),TFld::Integer,TFld::Selected,"","3","0;1;2;3;4;5;6;7;8",
+					_("None;Dotted;Dashed;Solid;Double;Groove;Ridge;Inset;Outset"),"19") );
 	attrAdd( new TFld("font",_("Font"),TFld::String,Attr::Font,"50","Arial 11","","","24") );
-        attrAdd( new TFld("color",_("Color"),TFld::String,Attr::Color,"20","#000000","","","25") );
-        attrAdd( new TFld("orient",_("Orientation angle"),TFld::Integer,TFld::NoFlag,"3","0","-180;180","","26") );
-        attrAdd( new TFld("wordWrap",_("Word wrap"),TFld::Boolean,TFld::NoFlag,"1","1","","","27") );
-        attrAdd( new TFld("alignment",_("Alignment"),TFld::Integer,TFld::Selected,"1","0","0;1;2;3;4;5;6;7;8;9;10;11",
-                          _("Top left;Top right;Top center;Top justify;"
+	attrAdd( new TFld("color",_("Color"),TFld::String,Attr::Color,"20","#000000","","","25") );
+	attrAdd( new TFld("orient",_("Orientation angle"),TFld::Integer,TFld::NoFlag,"3","0","-180;180","","26") );
+	attrAdd( new TFld("wordWrap",_("Word wrap"),TFld::Boolean,TFld::NoFlag,"1","1","","","27") );
+	attrAdd( new TFld("alignment",_("Alignment"),TFld::Integer,TFld::Selected,"1","0","0;1;2;3;4;5;6;7;8;9;10;11",
+			    _("Top left;Top right;Top center;Top justify;"
 			    "Bottom left;Bottom right;Bottom center;Bottom justify;"
 			    "V center left; V center right; Center; V center justify"),"28") );
-    	attrAdd( new TFld("text",_("Text"),TFld::String,TFld::FullText,"0","Text","","","29") );
-    	attrAdd( new TFld("numbArg",_("Arguments number"),TFld::Integer,Attr::Active,"","0","0;10","","30") );
+	attrAdd( new TFld("text",_("Text"),TFld::String,TFld::FullText,"0","Text","","","29") );
+	attrAdd( new TFld("numbArg",_("Arguments number"),TFld::Integer,Attr::Active,"","0","0;10","","30") );
     }
-} 
+}
 
 bool OrigText::attrChange( Attr &cfg, void *prev )
 {
     if( cfg.flgGlob()&Attr::Active )
     {
 	int aid = atoi(cfg.fld().reserve().c_str());
-     	if( cfg.id() == "numbArg" )
+	if( cfg.id() == "numbArg" )
 	{
 	    string fid("arg"), fnm(_("Argument ")), fidp, fnmp;
 	    //- Delete specific unnecessary attributes of parameters -
 	    for( int i_p = 0; true; i_p++ )
 	    {
-		fidp = fid+TSYS::int2str(i_p); 
+		fidp = fid+TSYS::int2str(i_p);
 		if( !cfg.owner()->attrPresent( fidp+"val" ) )	break;
 		else if( i_p >= cfg.getI() )
 		{
@@ -433,33 +438,33 @@ OrigMedia::OrigMedia( ) : PrWidget("Media")
 {
 
 }
-    
+
 string OrigMedia::name( )
-{ 
-    return _("Media"); 
+{
+    return _("Media");
 }
 
-string OrigMedia::descr( )	
-{ 
-    return _("Media widget of the end visualisation."); 
+string OrigMedia::descr( )
+{
+    return _("Media widget of the end visualisation.");
 }
- 
+
 void OrigMedia::postEnable( int flag )
 {
     LWidget::postEnable(flag);
-    
+
     if( flag&TCntrNode::NodeConnect )
-    { 	
-        attrAdd( new TFld("backColor",_("Background:color"),TFld::String,Attr::Color,"","#FFFFFF","","","20") );
-        attrAdd( new TFld("backImg",_("Background:image"),TFld::String,Attr::Image,"","","","","21") );
-        attrAdd( new TFld("bordWidth",_("Border:width"),TFld::Integer,TFld::NoFlag,"","0","","","22") );
-        attrAdd( new TFld("bordColor",_("Border:color"),TFld::String,Attr::Color,"","#000000","","","23") );
-        attrAdd( new TFld("bordStyle",_("Border:style"),TFld::Integer,TFld::Selected,"","3","0;1;2;3;4;5;6;7;8",
+    {
+	attrAdd( new TFld("backColor",_("Background:color"),TFld::String,Attr::Color,"","#FFFFFF","","","20") );
+	attrAdd( new TFld("backImg",_("Background:image"),TFld::String,Attr::Image,"","","","","21") );
+	attrAdd( new TFld("bordWidth",_("Border:width"),TFld::Integer,TFld::NoFlag,"","0","","","22") );
+	attrAdd( new TFld("bordColor",_("Border:color"),TFld::String,Attr::Color,"","#000000","","","23") );
+	attrAdd( new TFld("bordStyle",_("Border:style"),TFld::Integer,TFld::Selected,"","3","0;1;2;3;4;5;6;7;8",
 						_("None;Dotted;Dashed;Solid;Double;Groove;Ridge;Inset;Outset"),"19") );
-        attrAdd( new TFld("src",_("Source"),TFld::String,TFld::NoFlag,"50","","","","24") );
-    	attrAdd( new TFld("fit",_("Fit to widget size"),TFld::Boolean,TFld::NoFlag,"","","","","26") );
-        attrAdd( new TFld("type",_("Type"),TFld::Integer,TFld::Selected|Attr::Active,"1","0","0;1",_("Image;Movie"),"25") );
-        attrAdd( new TFld("areas",_("Map areas"),TFld::Integer,Attr::Active,"2","0","0;10","","28") );
+	attrAdd( new TFld("src",_("Source"),TFld::String,TFld::NoFlag,"50","","","","24") );
+	attrAdd( new TFld("fit",_("Fit to widget size"),TFld::Boolean,TFld::NoFlag,"","","","","26") );
+	attrAdd( new TFld("type",_("Type"),TFld::Integer,TFld::Selected|Attr::Active,"1","0","0;1",_("Image;Movie"),"25") );
+	attrAdd( new TFld("areas",_("Map areas"),TFld::Integer,Attr::Active,"2","0","0;10","","28") );
     }
 }
 
@@ -467,11 +472,11 @@ bool OrigMedia::attrChange( Attr &cfg, void *prev )
 {
     if( cfg.flgGlob()&Attr::Active )
     {
-        if( cfg.id() == "type" )
+	if( cfg.id() == "type" )
 	{
 	    //- Delete specific attributes -
 	    if( *(int*)prev == 1 ) cfg.owner()->attrDel("speed");
-	
+
 	    //- Create specific attributes -
 	    if(cfg.getI() == 1 )
 		cfg.owner()->attrAdd( new TFld("speed",_("Play speed"),TFld::Integer,Attr::Mutable,"3","100","1;900","","27") );
@@ -497,9 +502,9 @@ bool OrigMedia::attrChange( Attr &cfg, void *prev )
 		fidp = fid+TSYS::int2str(i_p);
 		fnmp = fnm+TSYS::int2str(i_p);
 		if( cfg.owner()->attrPresent( fidp+"shp" ) ) continue;
-    		cfg.owner()->attrAdd( new TFld((fidp+"shp").c_str(),(fnmp+_(":shape")).c_str(),
+		cfg.owner()->attrAdd( new TFld((fidp+"shp").c_str(),(fnmp+_(":shape")).c_str(),
 					       TFld::Integer,TFld::Selected|Attr::Mutable,"1","0","0;1;2","Rect;Poly;Circle",TSYS::int2str(40+3*i_p).c_str()) );
-    		cfg.owner()->attrAdd( new TFld((fidp+"coord").c_str(),(fnmp+_(":coordinates")).c_str(),
+		cfg.owner()->attrAdd( new TFld((fidp+"coord").c_str(),(fnmp+_(":coordinates")).c_str(),
 					       TFld::String,Attr::Mutable,"","","","",TSYS::int2str(41+3*i_p).c_str()) );
 		cfg.owner()->attrAdd( new TFld((fidp+"title").c_str(),(fnmp+_(":title")).c_str(),
 					       TFld::String,Attr::Mutable,"","","","",TSYS::int2str(42+3*i_p).c_str()) );
@@ -516,52 +521,52 @@ OrigDiagram::OrigDiagram( ) : PrWidget("Diagram")
 {
 
 }
-    
-string OrigDiagram::name( )      
-{ 
+
+string OrigDiagram::name( )
+{
     return _("Diagram");
 }
 
-string OrigDiagram::descr( )     
-{ 
+string OrigDiagram::descr( )
+{
     return _("Diagram widget of the end visualisation.");
 }
- 
+
 void OrigDiagram::postEnable( int flag )
 {
     LWidget::postEnable(flag);
-    
+
     if( flag&TCntrNode::NodeConnect ) 
-    { 
-        attrAdd( new TFld("backColor",_("Background:color"),TFld::String,Attr::Color,"","black","","","20") );
-        attrAdd( new TFld("backImg",_("Background:image"),TFld::String,Attr::Image,"","","","","21") );
-        attrAdd( new TFld("bordWidth",_("Border:width"),TFld::Integer,TFld::NoFlag,"","0","","","22") );
-        attrAdd( new TFld("bordColor",_("Border:color"),TFld::String,Attr::Color,"","#000000","","","23") );
-        attrAdd( new TFld("bordStyle",_("Border:style"),TFld::Integer,TFld::Selected,"","3","0;1;2;3;4;5;6;7;8",
+    {
+	attrAdd( new TFld("backColor",_("Background:color"),TFld::String,Attr::Color,"","black","","","20") );
+	attrAdd( new TFld("backImg",_("Background:image"),TFld::String,Attr::Image,"","","","","21") );
+	attrAdd( new TFld("bordWidth",_("Border:width"),TFld::Integer,TFld::NoFlag,"","0","","","22") );
+	attrAdd( new TFld("bordColor",_("Border:color"),TFld::String,Attr::Color,"","#000000","","","23") );
+	attrAdd( new TFld("bordStyle",_("Border:style"),TFld::Integer,TFld::Selected,"","3","0;1;2;3;4;5;6;7;8",
 						_("None;Dotted;Dashed;Solid;Double;Groove;Ridge;Inset;Outset"),"19") );
-        attrAdd( new TFld("trcPer",_("Tracing period (s)"),TFld::Integer,TFld::NoFlag,"","0","0;360","","24") );
-        attrAdd( new TFld("type",_("Type"),TFld::Integer,TFld::Selected|Attr::Active,"1","0","0",_("Trend"),"25") );
+	attrAdd( new TFld("trcPer",_("Tracing period (s)"),TFld::Integer,TFld::NoFlag,"","0","0;360","","24") );
+	attrAdd( new TFld("type",_("Type"),TFld::Integer,TFld::Selected|Attr::Active,"1","0","0",_("Trend"),"25") );
     }
-} 
+}
 
 bool OrigDiagram::attrChange( Attr &cfg, void *prev )
 {
     if( cfg.flgGlob()&Attr::Active )
     {
-	if( cfg.id() == "active" && cfg.getB() != *(bool*)prev && cfg.owner()->attrPresent("type") && 
+	if( cfg.id() == "active" && cfg.getB() != *(bool*)prev && cfg.owner()->attrPresent("type") &&
 		cfg.owner()->attrAt("type").at().getI() == 0 )
 	{
 	    if( !cfg.getB() )
 	    {
-	        cfg.owner()->attrDel("curSek");
-	        cfg.owner()->attrDel("curUSek");
-	        cfg.owner()->attrDel("curColor");
+		cfg.owner()->attrDel("curSek");
+		cfg.owner()->attrDel("curUSek");
+		cfg.owner()->attrDel("curColor");
 	    }
 	    else
 	    {
-    	        cfg.owner()->attrAdd( new TFld("curSek",_("Cursor:sek"),TFld::Integer,Attr::DataTime|Attr::Mutable,"","","","","29") );
-    	        cfg.owner()->attrAdd( new TFld("curUSek",_("Cursor:usek"),TFld::Integer,Attr::Mutable,"","","","","30") );
-    		cfg.owner()->attrAdd( new TFld("curColor",_("Cursor:color"),TFld::String,Attr::Color|Attr::Mutable,"","white","","","36") );
+		cfg.owner()->attrAdd( new TFld("curSek",_("Cursor:sek"),TFld::Integer,Attr::DataTime|Attr::Mutable,"","","","","29") );
+		cfg.owner()->attrAdd( new TFld("curUSek",_("Cursor:usek"),TFld::Integer,Attr::Mutable,"","","","","30") );
+		cfg.owner()->attrAdd( new TFld("curColor",_("Cursor:color"),TFld::String,Attr::Color|Attr::Mutable,"","white","","","36") );
 	    }
 	}
 	else if( cfg.id() == "type" )
@@ -584,30 +589,30 @@ bool OrigDiagram::attrChange( Attr &cfg, void *prev )
 		    cfg.owner()->attrDel("valArch");
 		    cfg.owner()->attrDel("parNum");
 		    break;
-	    }	    
-	
+	    }
+
 	    //- Create specific attributes -
 	    switch(cfg.getI())
 	    {
 		case 0:
-    		    cfg.owner()->attrAdd( new TFld("tSek",_("Time:sek"),TFld::Integer,Attr::DataTime|Attr::Mutable,"","","","","26") );
-    		    cfg.owner()->attrAdd( new TFld("tUSek",_("Time:usek"),TFld::Integer,Attr::Mutable,"","","","","27") );
-    		    cfg.owner()->attrAdd( new TFld("tSize",_("Size, sek"),TFld::Real,Attr::Mutable,"","60","","","28") );
+		    cfg.owner()->attrAdd( new TFld("tSek",_("Time:sek"),TFld::Integer,Attr::DataTime|Attr::Mutable,"","","","","26") );
+		    cfg.owner()->attrAdd( new TFld("tUSek",_("Time:usek"),TFld::Integer,Attr::Mutable,"","","","","27") );
+		    cfg.owner()->attrAdd( new TFld("tSize",_("Size, sek"),TFld::Real,Attr::Mutable,"","60","","","28") );
 		    if( cfg.owner()->attrAt("active").at().getB() )
 		    {
-    			cfg.owner()->attrAdd( new TFld("curSek",_("Cursor:sek"),TFld::Integer,Attr::DataTime|Attr::Mutable,"","","","","29") );
-    			cfg.owner()->attrAdd( new TFld("curUSek",_("Cursor:usek"),TFld::Integer,Attr::Mutable,"","","","","30") );
-    			cfg.owner()->attrAdd( new TFld("curColor",_("Cursor:color"),TFld::String,Attr::Color|Attr::Mutable,"","white","","","36") );
+			cfg.owner()->attrAdd( new TFld("curSek",_("Cursor:sek"),TFld::Integer,Attr::DataTime|Attr::Mutable,"","","","","29") );
+			cfg.owner()->attrAdd( new TFld("curUSek",_("Cursor:usek"),TFld::Integer,Attr::Mutable,"","","","","30") );
+			cfg.owner()->attrAdd( new TFld("curColor",_("Cursor:color"),TFld::String,Attr::Color|Attr::Mutable,"","white","","","36") );
 		    }
-    		    cfg.owner()->attrAdd( new TFld("sclColor",_("Scale:color"),TFld::String,Attr::Color|Attr::Mutable,"","grey","","","31") );
-    		    cfg.owner()->attrAdd( new TFld("sclHor",_("Scale:horizontal"),TFld::Integer,Attr::Mutable|TFld::Selected,
+		    cfg.owner()->attrAdd( new TFld("sclColor",_("Scale:color"),TFld::String,Attr::Color|Attr::Mutable,"","grey","","","31") );
+		    cfg.owner()->attrAdd( new TFld("sclHor",_("Scale:horizontal"),TFld::Integer,Attr::Mutable|TFld::Selected,
 			"1","0","0;1;2;3",_("No draw;Grid;Markers;Grid and markers"),"32") );
-    		    cfg.owner()->attrAdd( new TFld("sclVer",_("Scale:vertical"),TFld::Integer,Attr::Mutable|TFld::Selected,
+		    cfg.owner()->attrAdd( new TFld("sclVer",_("Scale:vertical"),TFld::Integer,Attr::Mutable|TFld::Selected,
 			"1","0","0;1;2;3;5;6;7",_("No draw;Grid;Markers;Grid and markers;Grid (log);Marker (log);Grid and markers (log)"),"33") );
-    		    cfg.owner()->attrAdd( new TFld("sclMarkColor",_("Scale:Markers:color"),TFld::String,Attr::Color|Attr::Mutable,"","white","","","37") );
-    		    cfg.owner()->attrAdd( new TFld("sclMarkFont",_("Scale:Markers:font"),TFld::String,Attr::Font|Attr::Mutable,"","Arial 10","","","38") );
-    		    cfg.owner()->attrAdd( new TFld("valArch",_("Value archivator"),TFld::String,Attr::Mutable,"","","","","34") );
-    		    cfg.owner()->attrAdd( new TFld("parNum",_("Parameters number"),TFld::Integer,Attr::Mutable|Attr::Active,"1","1","0;10","","35") );
+		    cfg.owner()->attrAdd( new TFld("sclMarkColor",_("Scale:Markers:color"),TFld::String,Attr::Color|Attr::Mutable,"","white","","","37") );
+		    cfg.owner()->attrAdd( new TFld("sclMarkFont",_("Scale:Markers:font"),TFld::String,Attr::Font|Attr::Mutable,"","Arial 10","","","38") );
+		    cfg.owner()->attrAdd( new TFld("valArch",_("Value archivator"),TFld::String,Attr::Mutable,"","","","","34") );
+		    cfg.owner()->attrAdd( new TFld("parNum",_("Parameters number"),TFld::Integer,Attr::Mutable|Attr::Active,"1","1","0;10","","35") );
 		    break;
 	    }
 	}
@@ -634,22 +639,22 @@ bool OrigDiagram::attrChange( Attr &cfg, void *prev )
 		fidp = fid+TSYS::int2str(i_p);
 		fnmp = fnm+TSYS::int2str(i_p);
 		if( cfg.owner()->attrPresent( fidp+"addr" ) ) continue;
-    		cfg.owner()->attrAdd( new TFld((fidp+"addr").c_str(),(fnmp+_(":address")).c_str(),
+		cfg.owner()->attrAdd( new TFld((fidp+"addr").c_str(),(fnmp+_(":address")).c_str(),
 					       TFld::String,Attr::Address|Attr::Mutable,"","","","",TSYS::int2str(50+10*i_p).c_str()) );
-    		cfg.owner()->attrAdd( new TFld((fidp+"bordL").c_str(),(fnmp+_(":view border:lower")).c_str(),
+		cfg.owner()->attrAdd( new TFld((fidp+"bordL").c_str(),(fnmp+_(":view border:lower")).c_str(),
 					       TFld::Real,Attr::Mutable,"","","","",TSYS::int2str(51+10*i_p).c_str()) );
-    		cfg.owner()->attrAdd( new TFld((fidp+"bordU").c_str(),(fnmp+_(":view border:upper")).c_str(),
+		cfg.owner()->attrAdd( new TFld((fidp+"bordU").c_str(),(fnmp+_(":view border:upper")).c_str(),
 					       TFld::Real,Attr::Mutable,"","","","",TSYS::int2str(52+10*i_p).c_str()) );
-    		cfg.owner()->attrAdd( new TFld((fidp+"aScale").c_str(),(fnmp+_(":view border:autoscale")).c_str(),
-					       TFld::Integer,Attr::Mutable,"","0","","",TSYS::int2str(55+10*i_p).c_str()) );		
-    		cfg.owner()->attrAdd( new TFld((fidp+"color").c_str(),(fnmp+_(":color")).c_str(),
+		cfg.owner()->attrAdd( new TFld((fidp+"aScale").c_str(),(fnmp+_(":view border:autoscale")).c_str(),
+					       TFld::Integer,Attr::Mutable,"","0","","",TSYS::int2str(55+10*i_p).c_str()) );
+		cfg.owner()->attrAdd( new TFld((fidp+"color").c_str(),(fnmp+_(":color")).c_str(),
 					       TFld::String,Attr::Color|Attr::Mutable,"","","","",TSYS::int2str(53+10*i_p).c_str()) );
-    		cfg.owner()->attrAdd( new TFld((fidp+"val").c_str(),(fnmp+_(":value")).c_str(),
+		cfg.owner()->attrAdd( new TFld((fidp+"val").c_str(),(fnmp+_(":value")).c_str(),
 					       TFld::Real,Attr::Mutable,"","","","",TSYS::int2str(54+10*i_p).c_str()) );
 	    }
 	}
     }
-    
+
     return Widget::attrChange(cfg,prev);
 }
 
@@ -657,40 +662,40 @@ bool OrigDiagram::attrChange( Attr &cfg, void *prev )
 //* OrigProtocol: Protocol original widget       *
 //************************************************
 OrigProtocol::OrigProtocol( ) : PrWidget("Protocol")
-{ 
-    
-}
-    
-string OrigProtocol::name( )
-{ 
-    return _("Protocol"); 
+{
+
 }
 
-string OrigProtocol::descr( )	
-{ 
+string OrigProtocol::name( )
+{
+    return _("Protocol");
+}
+
+string OrigProtocol::descr( )
+{
     return _("Protocol widget of the end visualisation.");
 }
 
 void OrigProtocol::postEnable( int flag )
 {
     LWidget::postEnable(flag);
-    
-    if( flag&TCntrNode::NodeConnect ) 
-    { 
-        attrAdd( new TFld("backColor",_("Background:color"),TFld::String,Attr::Color,"","","","","20") );
-        attrAdd( new TFld("backImg",_("Background:image"),TFld::String,Attr::Image,"","","","","21") );
-    	attrAdd( new TFld("time",_("Time, sek"),TFld::Integer,Attr::DataTime,"","","","","24") );
-        attrAdd( new TFld("tSize",_("Size, sek"),TFld::Integer,TFld::NoFlag,"","60","","","25") );
-        attrAdd( new TFld("trcPer",_("Tracing period (s)"),TFld::Integer,TFld::NoFlag,"","0","0;360","","26") );
-        attrAdd( new TFld("arch",_("Archivator"),TFld::String,TFld::NoFlag,"","","","","27") );
-        attrAdd( new TFld("tmpl",_("Template"),TFld::String,TFld::NoFlag,"","","","","28") );
-        attrAdd( new TFld("lev",_("Level"),TFld::Integer,TFld::NoFlag,"","0","","","29") );
-        attrAdd( new TFld("viewOrd",_("View order"),TFld::Integer,TFld::Selected,"","0",
+
+    if( flag&TCntrNode::NodeConnect )
+    {
+	attrAdd( new TFld("backColor",_("Background:color"),TFld::String,Attr::Color,"","","","","20") );
+	attrAdd( new TFld("backImg",_("Background:image"),TFld::String,Attr::Image,"","","","","21") );
+	attrAdd( new TFld("time",_("Time, sek"),TFld::Integer,Attr::DataTime,"","","","","24") );
+	attrAdd( new TFld("tSize",_("Size, sek"),TFld::Integer,TFld::NoFlag,"","60","","","25") );
+	attrAdd( new TFld("trcPer",_("Tracing period (s)"),TFld::Integer,TFld::NoFlag,"","0","0;360","","26") );
+	attrAdd( new TFld("arch",_("Archivator"),TFld::String,TFld::NoFlag,"","","","","27") );
+	attrAdd( new TFld("tmpl",_("Template"),TFld::String,TFld::NoFlag,"","","","","28") );
+	attrAdd( new TFld("lev",_("Level"),TFld::Integer,TFld::NoFlag,"","0","","","29") );
+	attrAdd( new TFld("viewOrd",_("View order"),TFld::Integer,TFld::Selected,"","0",
 	    "0;1;2",_("On time;On level;On level and trigered"),"30") );
-        attrAdd( new TFld("col",_("View columns"),TFld::String,TFld::NoFlag,"","pos;tm;lev;cat;mess","","","31") );
-        attrAdd( new TFld("itProp",_("Items properties"),TFld::Integer,Attr::Active,"","0","0;10","","32") );
+	attrAdd( new TFld("col",_("View columns"),TFld::String,TFld::NoFlag,"","pos;tm;lev;cat;mess","","","31") );
+	attrAdd( new TFld("itProp",_("Items properties"),TFld::Integer,Attr::Active,"","0","0;10","","32") );
     }
-} 
+}
 
 bool OrigProtocol::attrChange( Attr &cfg, void *prev )
 {
@@ -702,7 +707,7 @@ bool OrigProtocol::attrChange( Attr &cfg, void *prev )
 	    //- Delete specific unnecessary items -
 	    for( int i_p = 0; true; i_p++ )
 	    {
-		fidp = fid+TSYS::int2str(i_p); 
+		fidp = fid+TSYS::int2str(i_p);
 		if( !cfg.owner()->attrPresent( fidp+"lev" ) )	break;
 		else if( i_p >= cfg.getI() )
 		{
@@ -720,56 +725,56 @@ bool OrigProtocol::attrChange( Attr &cfg, void *prev )
 		fidp = fid+TSYS::int2str(i_p);
 		fnmp = fnm+TSYS::int2str(i_p);
 		if( cfg.owner()->attrPresent( fidp+"lev" ) ) continue;
-    		cfg.owner()->attrAdd( new TFld((fidp+"lev").c_str(),(fnmp+_(":levels")).c_str(),
+		cfg.owner()->attrAdd( new TFld((fidp+"lev").c_str(),(fnmp+_(":levels")).c_str(),
 					       TFld::String,Attr::Mutable,"","","","",TSYS::int2str(50+10*i_p).c_str()) );
-    		cfg.owner()->attrAdd( new TFld((fidp+"tmpl").c_str(),(fnmp+_(":template")).c_str(),
+		cfg.owner()->attrAdd( new TFld((fidp+"tmpl").c_str(),(fnmp+_(":template")).c_str(),
 					       TFld::String,Attr::Mutable,"","","","",TSYS::int2str(51+10*i_p).c_str()) );
-    		cfg.owner()->attrAdd( new TFld((fidp+"fnt").c_str(),(fnmp+_(":font")).c_str(),
+		cfg.owner()->attrAdd( new TFld((fidp+"fnt").c_str(),(fnmp+_(":font")).c_str(),
 					       TFld::String,Attr::Font|Attr::Mutable,"","","","",TSYS::int2str(52+10*i_p).c_str()) );
-    		cfg.owner()->attrAdd( new TFld((fidp+"color").c_str(),(fnmp+_(":color")).c_str(),
+		cfg.owner()->attrAdd( new TFld((fidp+"color").c_str(),(fnmp+_(":color")).c_str(),
 					       TFld::String,Attr::Color|Attr::Mutable,"","","","",TSYS::int2str(53+10*i_p).c_str()) );
-    		cfg.owner()->attrAdd( new TFld((fidp+"blink").c_str(),(fnmp+_(":blink")).c_str(),
+		cfg.owner()->attrAdd( new TFld((fidp+"blink").c_str(),(fnmp+_(":blink")).c_str(),
 					       TFld::Boolean,Attr::Mutable,"","0","","",TSYS::int2str(54+10*i_p).c_str()) );
 	    }
 	}
     }
-    return Widget::attrChange(cfg,prev);    
+    return Widget::attrChange(cfg,prev);
 }
 
 //************************************************
 //* OrigDocument: Document original widget       *
 //************************************************
 OrigDocument::OrigDocument( ) : PrWidget("Document")
-{ 
-    
-}
-    
-string OrigDocument::name( )      
-{ 
-    return _("Document"); 
+{
+
 }
 
-string OrigDocument::descr( )     
-{ 
-    return _("Document widget of the end visualisation."); 
+string OrigDocument::name( )
+{
+    return _("Document");
+}
+
+string OrigDocument::descr( )
+{
+    return _("Document widget of the end visualisation.");
 }
 
 //************************************************
 //* OrigFunction: User function original widget  *
 //************************************************
 OrigFunction::OrigFunction( ) : PrWidget("Function")
-{ 
-    
-}
-    
-string OrigFunction::name( )      
-{ 
-    return _("Buildin function"); 
+{
+
 }
 
-string OrigFunction::descr( )     
-{ 
-    return _("Buildin function widget."); 
+string OrigFunction::name( )
+{
+    return _("Buildin function");
+}
+
+string OrigFunction::descr( )
+{
+    return _("Buildin function widget.");
 }
 
 //************************************************
@@ -781,34 +786,31 @@ OrigBox::OrigBox( ) : PrWidget("Box")
 }
 
 string OrigBox::name( )
-{ 
+{
     return _("Elements box");
 }
 
 string OrigBox::descr( )
-{ 
+{
     return _("Elements box widget of the end visualisation.");
 }
 
 void OrigBox::postEnable( int flag )
 {
     LWidget::postEnable(flag);
-    
-    if( flag&TCntrNode::NodeConnect )  
-    { 
-        attrAdd( new TFld("backColor",_("Background:color"),TFld::String,Attr::Color,"","#FFFFFF","","","20") );
-        attrAdd( new TFld("backImg",_("Background:image"),TFld::String,Attr::Image,"","","","","21") );
-        attrAdd( new TFld("bordWidth",_("Border:width"),TFld::Integer,TFld::NoFlag,"","0","","","22") );
-        attrAdd( new TFld("bordColor",_("Border:color"),TFld::String,Attr::Color,"","#000000","","","23") );
-        attrAdd( new TFld("bordStyle",_("Border:style"),TFld::Integer,TFld::Selected,"","3","0;1;2;3;4;5;6;7;8",
+
+    if( flag&TCntrNode::NodeConnect )
+    {
+	attrAdd( new TFld("backColor",_("Background:color"),TFld::String,Attr::Color,"","#FFFFFF","","","20") );
+	attrAdd( new TFld("backImg",_("Background:image"),TFld::String,Attr::Image,"","","","","21") );
+	attrAdd( new TFld("bordWidth",_("Border:width"),TFld::Integer,TFld::NoFlag,"","0","","","22") );
+	attrAdd( new TFld("bordColor",_("Border:color"),TFld::String,Attr::Color,"","#000000","","","23") );
+	attrAdd( new TFld("bordStyle",_("Border:style"),TFld::Integer,TFld::Selected,"","3","0;1;2;3;4;5;6;7;8",
 						_("None;Dotted;Dashed;Solid;Double;Groove;Ridge;Inset;Outset"),"19") );
-	attrAdd( new TFld("pgOpen",_("Page:open state"),TFld::Boolean,TFld::NoFlag,"","","","","24") );
-	attrAdd( new TFld("pgNoOpenProc",_("Page:no open process"),TFld::Boolean,TFld::NoFlag,"","","","","25") );
-        attrAdd( new TFld("pgOpenSrc",_("Page:open source"),TFld::String,TFld::NoFlag,"","","","","26") );
-        attrAdd( new TFld("pgGrp",_("Page:group"),TFld::String,TFld::NoFlag,"","","","","27") );
-	attrAdd( new TFld("pgFullScr",_("Page:full screen"),TFld::Boolean,TFld::NoFlag,"","","","","28") );
+	attrAdd( new TFld("pgOpenSrc",_("Page:open source"),TFld::String,TFld::NoFlag,"","","","","3") );
+	attrAdd( new TFld("pgGrp",_("Page:group"),TFld::String,TFld::NoFlag,"","","","","4") );
     }
-} 
+}
 
 //************************************************
 //* OrigLink: Link original widget               *
@@ -817,28 +819,28 @@ OrigLink::OrigLink( ) : PrWidget("Link")
 {
 
 }
- 
-string OrigLink::name( )      
-{ 
-    return _("Interwidget link"); 
+
+string OrigLink::name( )
+{
+    return _("Interwidget link");
 }
 
-string OrigLink::descr( )     
-{ 
-    return _("Interwidget link of the end visualisation."); 
+string OrigLink::descr( )
+{
+    return _("Interwidget link of the end visualisation.");
 }
- 
+
 void OrigLink::postEnable( int flag )
 {
     LWidget::postEnable(flag);
-    
-    if( flag&TCntrNode::NodeConnect )  
-    { 
-    	attrAdd( new TFld("out",_("Output"),TFld::String,TFld::NoFlag,"50","","","","20") );
-    	attrAdd( new TFld("in",_("Input"),TFld::String,TFld::NoFlag,"50","","","","21") );
-    	attrAdd( new TFld("lineWdth",_("Line:width"),TFld::Integer,TFld::NoFlag,"2","1","0;99","","22") );
-    	attrAdd( new TFld("lineClr",_("Line:color"),TFld::String,Attr::Color,"20","#000000","","","23") );
-    	attrAdd( new TFld("lineSquare",_("Line:square angle"),TFld::Boolean,TFld::NoFlag,"1","0","","","24") );
+
+    if( flag&TCntrNode::NodeConnect )
+    {
+	attrAdd( new TFld("out",_("Output"),TFld::String,TFld::NoFlag,"50","","","","20") );
+	attrAdd( new TFld("in",_("Input"),TFld::String,TFld::NoFlag,"50","","","","21") );
+	attrAdd( new TFld("lineWdth",_("Line:width"),TFld::Integer,TFld::NoFlag,"2","1","0;99","","22") );
+	attrAdd( new TFld("lineClr",_("Line:color"),TFld::String,Attr::Color,"20","#000000","","","23") );
+	attrAdd( new TFld("lineSquare",_("Line:square angle"),TFld::Boolean,TFld::NoFlag,"1","0","","","24") );
 	//Next is dynamic created internal points
     }
-} 
+}
