@@ -1,7 +1,7 @@
 
 //OpenSCADA system module UI.VCAEngine file: vcaengine.h
 /***************************************************************************
- *   Copyright (C) 2006-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2006-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -40,39 +40,50 @@ namespace VCA
 class Engine : public TUI
 {
     public:
+	//Data
+	enum Alarm	{ Light = 0x01, Alarm = 0x02, Sound = 0x04 };
+
 	//Methods
 	Engine( string name );
-	~Engine( );	
+	~Engine( );
 
-        string modInfo( const string &name );
-        void   modInfo( vector<string> &list );
+	string modInfo( const string &name );
+	void   modInfo( vector<string> &list );
 
-        void modStart( );
+	void modStart( );
 	void modStop( );
-	
+
+	string synthCom( )	{ return mSynthCom; }
+	string synthCode( )	{ return mSynthCode; }
+
 	string wlbTable( )	{ return "VCALibs"; }
 	string prjTable( )	{ return "VCAPrjs"; }
-	
+
+	void setSynthCom( const string &vl )	{ mSynthCom = vl; modif(); }
+	void setSynthCode( const string &vl )	{ mSynthCode = vl; modif(); }
+
+	string callSynth( const string &txt );
+
 	//- Widget's libraries -
-        void wlbList( vector<string> &ls )			{ chldList(id_wlb,ls); }
-        bool wlbPresent( const string &id )			{ return chldPresent(id_wlb,id); }
+	void wlbList( vector<string> &ls )			{ chldList( id_wlb, ls ); }
+	bool wlbPresent( const string &id )			{ return chldPresent( id_wlb, id ); }
 	void wlbAdd( const string &iid, const string &inm = "", const string &idb = "*.*" );
-        void wlbDel( const string &iid, bool full = false )	{ chldDel(id_wlb,iid,-1,full); }	
-        AutoHD<WidgetLib> wlbAt( const string &id );
-	
+	void wlbDel( const string &iid, bool full = false )	{ chldDel( id_wlb, iid, -1, full ); }
+	AutoHD<WidgetLib> wlbAt( const string &id );
+
 	//- Projects -
-        void prjList( vector<string> &ls )			{ chldList(id_prj,ls); }
-        bool prjPresent( const string &id )			{ return chldPresent(id_prj,id); }
+	void prjList( vector<string> &ls )			{ chldList( id_prj, ls ); }
+	bool prjPresent( const string &id )			{ return chldPresent( id_prj, id ); }
 	void prjAdd( const string &iid, const string &inm = "", const string &idb = "*.*" );
-        void prjDel( const string &iid, bool full = false )	{ chldDel(id_prj,iid,-1,full); }	
-        AutoHD<Project> prjAt( const string &id );
-	
+	void prjDel( const string &iid, bool full = false )	{ chldDel( id_prj, iid, -1, full ); }
+	AutoHD<Project> prjAt( const string &id );
+
 	//- Sessions -
-        void sesList( vector<string> &ls )			{ chldList(id_ses,ls); }
-        bool sesPresent( const string &id )			{ return chldPresent(id_ses,id); }
+	void sesList( vector<string> &ls )			{ chldList( id_ses, ls ); }
+	bool sesPresent( const string &id )			{ return chldPresent( id_ses, id ); }
 	void sesAdd( const string &id, const string &proj = "" );
-        void sesDel( const string &iid, bool full = false )	{ chldDel(id_ses,iid,-1,full); }	
-        AutoHD<Session> sesAt( const string &id );
+	void sesDel( const string &iid, bool full = false )	{ chldDel( id_ses, iid, -1, full ); }
+	AutoHD<Session> sesAt( const string &id );
 
 	//- DB structures -
 	TElem &elWdgLib( )	{ return lbwdg_el; }
@@ -86,27 +97,30 @@ class Engine : public TUI
 
     protected:
 	//Methods
-        void load_( );
+	void load_( );
+	void save_( );
 	void postEnable( int flag );
 	void preDisable( int flag );
-	void cntrCmdProc( XMLNode *opt );       //Control interface command process	
-	
+	void cntrCmdProc( XMLNode *opt );       //Control interface command process
+
 	string optDescr( );
-	
+
     private:
 	//Attributes
 	int	id_wlb, id_prj, id_ses;
-	TElem	lbwdg_el, 	//The generic table structure of libraries
+	TElem	lbwdg_el,	//The generic table structure of libraries
 		wdgdata_el,	//Media and other data what use by widgets and stored into DB
-		wdg_el, 	//The table structure of library widgets
-		wdgio_el, 	//The table structure of library widget's atributes
+		wdg_el,		//The table structure of library widgets
+		wdgio_el,	//The table structure of library widget's atributes
 		wdguio_el,	//The table structure of library widget's user atributes
 		inclwdg_el,	//The table structure of container including widgets
 		prj_el,		//The generic table structure of project
 		page_el;	//The table structure of project's pages
-	string	cp_sel, cp_del;	//Copy source and destination elements
+	string	mSynthCom, mSynthCode;	//Synth parameters
+
+	Res	mSynthRes;	//Synth resource
 };
-    
+
 extern  Engine *mod;
 }
 
