@@ -1,7 +1,7 @@
 
 //OpenSCADA system module UI.Vision file: vis_devel_widgs.h
 /***************************************************************************
- *   Copyright (C) 2006-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2006-2008 by Roman Savochenko                           *
  *   rom_as@diyaorg.dp.ua                                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -25,6 +25,7 @@
 
 #include <string>
 #include <vector>
+#include <map>
 
 #include <QAbstractTableModel>
 #include <QDockWidget>
@@ -36,6 +37,7 @@
 
 using std::string;
 using std::vector;
+using std::map;
 
 class QTreeWidgetItem;
 class QTreeWidget;
@@ -46,7 +48,7 @@ namespace VISION
 
 //****************************************
 //* Inspector of attributes model        *
-//**************************************** 
+//****************************************
 class VisDevelop;
 
 class ModInspAttr: public QAbstractTableModel
@@ -54,89 +56,89 @@ class ModInspAttr: public QAbstractTableModel
     Q_OBJECT
 
     public:
-        //Public data
- 	//* Item of the inspector of attributes model   *
+	//Public data
+	//* Item of the inspector of attributes model   *
 	class Item
 	{
 	    public:
 		//Public data
 		enum Type { WdgGrp, Wdg, AttrGrp, Attr };
-		enum Flag 
-		{ 
-		    Select   = 0x01, 
-		    FullText = 0x08,
-		    Active   = 0x0100,
-		    Image    = 0x0200,
-		    Color    = 0x0400,
-		    Font     = 0x0800,
-		    DataTime = 0x0200
+		enum Flag
+		{
+		    Select	= 0x01,
+		    FullText	= 0x08,
+		    Active	= 0x0100,
+		    Image	= 0x0200,
+		    Color	= 0x0400,
+		    Font	= 0x0800,
+		    DataTime	= 0x0200
 		};
 
 		//Public attributes
-                Item( const string &iid, Type tp, Item *parent = NULL );
-		~Item();
+		Item( const string &iid, Type tp, Item *parent = NULL );
+		~Item( );
 
-                string id( )        	{ return idItem; }
-                string name( );
-                Type   type( )		{ return typeItem; }
-		bool   edited()		{ return edit_access; }
-		int    flag()		{ return flag_item; }
-                QVariant data( );
-                QVariant dataEdit( );
+		string	id( )		{ return idItem; }
+		string	name( );
+		Type	type( )		{ return typeItem; }
+		bool	edited()	{ return edit_access; }
+		int	flag()		{ return flag_item; }
+		QVariant data( );
+		QVariant dataEdit( );
 
-                void setName( const string &nit )       { nameItem = nit; }
+		void setName( const string &nit )	{ nameItem = nit; }
 		void setEdited( bool ied )		{ edit_access = ied; }
 		void setFlag( int iflg )		{ flag_item = iflg; }
-                void setData( const QVariant &idt )     { dataItem = idt; }
-                void setDataEdit( const QVariant &idt ) { dataEditItem = idt; }		
+		void setData( const QVariant &idt )	{ dataItem = idt; }
+		void setDataEdit( const QVariant &idt )	{ dataEditItem = idt; }
 
-                void clean( );
+		void clean( );
 
-                Item *child(int row) const;
-                int  childGet( const string &id ) const;
+		Item *child(int row) const;
+		int  childGet( const string &id ) const;
 		int  childCount() const;
-                int  childInsert( const string &id, int row, Type tp );
-                void childDel( int row );
-		
-		Item *parent()	{ return parentItem; }
-		
+		int  childInsert( const string &id, int row, Type tp );
+		void childDel( int row );
+
+		Item *parent()		{ return parentItem; }
+
 	    private:
-                string  idItem, nameItem;
-                Type    typeItem;
-                QVariant dataItem, dataEditItem;
+		string	idItem, nameItem;
+		Type	typeItem;
+		QVariant	dataItem, dataEditItem;
 		bool	edit_access;
 		int	flag_item;
 
-		QList<Item*> childItems;
-		Item *parentItem;
-	};                
+		QList<Item*>	childItems;
+		Item	*parentItem;
+	};
 
 	//Public methods
 	ModInspAttr( const string &wdg, VisDevelop *mainWind );
 	~ModInspAttr( );
-	
+
 	string user( );
-	
+
 	void setWdg( const string &iwdg );
-	
+
 	Qt::ItemFlags flags( const QModelIndex &index ) const;
 	QVariant headerData( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;
-        QModelIndex index( int row, int column, const QModelIndex &parent = QModelIndex() ) const;
-        QModelIndex parent(const QModelIndex &index) const;
-	
+	QModelIndex index( int row, int column, const QModelIndex &parent = QModelIndex() ) const;
+	QModelIndex parent(const QModelIndex &index) const;
+
 	int rowCount( const QModelIndex &parent = QModelIndex() ) const;
 	int columnCount( const QModelIndex &parent = QModelIndex() ) const;
 	QVariant data( const QModelIndex &index, int role = Qt::DisplayRole ) const;
-        bool setData ( const QModelIndex &index, const QVariant &value, int role = Qt::EditRole );
-	
-	VisDevelop *mainWin( )     { return main_win; }
+	bool setData ( const QModelIndex &index, const QVariant &value, int role = Qt::EditRole );
+
+	VisDevelop *mainWin( )		{ return main_win; }
 
     signals:
 	void modified( const string &idwdg );
 
     private:
-        //Private methods
-        void wdgAttrUpdate( const QModelIndex &mod_it );// Item *it);
+	//Private methods
+	void wdgAttrUpdate( const QModelIndex &mod_it );// Item *it);
 
 	//Private attributes
 	string cur_wdg;
@@ -150,19 +152,19 @@ class ModInspAttr: public QAbstractTableModel
 class InspAttr: public QTreeView
 {
     Q_OBJECT
-    
+
     public:
 	//Public methods
 	InspAttr( QWidget * parent, VisDevelop *mainWind );
 	~InspAttr( );
-		
+
 	bool hasFocus( );
-		
+
 	void setWdg( const string &iwdg );
 
-    signals:	
+    signals:
 	void modified( const string &idwdg );
-	
+
     private:
 	//Private data
 	//* Attributes item delegate    *
@@ -171,29 +173,29 @@ class InspAttr: public QTreeView
 	{
 	    public:
 		//Public methods
-	        ItemDelegate(QObject *parent = 0);
+		ItemDelegate(QObject *parent = 0);
 
-	        QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;
-	        void setEditorData(QWidget *editor, const QModelIndex &index) const;
-    		void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;
+		QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;
+		void setEditorData(QWidget *editor, const QModelIndex &index) const;
+		void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;
 		QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const;
 
-	    private:		
-		//Private methods			
-	        bool eventFilter(QObject *object, QEvent *event);
+	    private:
+		//Private methods
+		bool eventFilter(QObject *object, QEvent *event);
 	};
-	
+
 	//Private methods
 	bool event( QEvent *event );
-	
+
 	//Private attributes
 	ModInspAttr modelData;
 };
 
 //****************************************
 //* Inspector of attributes dock widget  *
-//**************************************** 
-class VisDevelop; 
+//****************************************
+class VisDevelop;
 
 class InspAttrDock: public QDockWidget
 {
@@ -203,20 +205,20 @@ class InspAttrDock: public QDockWidget
 	//Public methods
 	InspAttrDock( VisDevelop * parent );
 	~InspAttrDock( );
-	
+
 	VisDevelop *owner( );
-	
+
 	bool hasFocus( );
 
     signals:
 	void modified( const string &idwdg );
-	
+
     public slots:
-	void setWdg( const string &iwdg );	
+	void setWdg( const string &iwdg );
 
     private:
-        //Private attributes
-        InspAttr *ainsp_w;
+	//Private attributes
+	InspAttr *ainsp_w;
 };
 
 //****************************************
@@ -225,48 +227,48 @@ class InspAttrDock: public QDockWidget
 class InspLnk: public QTreeWidget
 {
     Q_OBJECT
-    
+
     public:
 	//Public methods
 	InspLnk( QWidget * parent, VisDevelop *mainWind );
 	~InspLnk( );
-	
+
 	string user( );
-	
+
 	void setWdg( const string &iwdg );
 
 	VisDevelop *mainWin( )     { return main_win; }
-	
+
     public slots:
 	void changeLnk( QTreeWidgetItem*, int );
-	
+
     private:
 	//Private methods
-        bool event( QEvent *event );
-    
+	bool event( QEvent *event );
+
 	//Private attributes
 	bool show_init;
 	string it_wdg;
 	VisDevelop *main_win;
 };
- 
+
 //*************************
 //* Link item delegate    *
 //*************************
 class LinkItemDelegate: public QItemDelegate
 {
     Q_OBJECT
-	
+
     public:
 	//Public methods
-        LinkItemDelegate(InspLnk *parent = 0);		
+	LinkItemDelegate(InspLnk *parent = 0);
 
-        QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;
-        void setEditorData(QWidget *editor, const QModelIndex &index) const;
+	QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;
+	void setEditorData(QWidget *editor, const QModelIndex &index) const;
 	void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;
 
 	InspLnk *owner( ) const;
-		
+
     //public slots:
     //	void selItem( int val );
 
@@ -278,24 +280,24 @@ class LinkItemDelegate: public QItemDelegate
 //* Inspector of links dock widget       *
 //**************************************** 
 class InspLnkDock: public QDockWidget
-{    
+{
     Q_OBJECT
 
     public:
 	//Public methods
 	InspLnkDock( VisDevelop * parent );
 	~InspLnkDock( );
-	
+
 	VisDevelop *owner( );
 
     public slots:
-	void setWdg( const string &iwdg );	
+	void setWdg( const string &iwdg );
 
     private:
 	//Private attributes
 	InspLnk *ainsp_w;
 };
- 
+
 //****************************************
 //* Widget's libraries tree              *
 //****************************************
@@ -307,17 +309,17 @@ class WdgTree: public QDockWidget
 	//Public methods
 	WdgTree( VisDevelop *parent = 0 );
 	~WdgTree();
-	
+
 	VisDevelop *owner( );
-	
+
 	bool hasFocus( );
 
     signals:
         void selectItem( const string &vca_it, bool force = false );
 
-    public slots:	
+    public slots:
 	void updateTree( const string &vca_it = "" );
-	
+
     protected:
 	//Protecten methods
 	bool eventFilter( QObject *target, QEvent *event );
@@ -327,13 +329,13 @@ class WdgTree: public QDockWidget
 	void ctrTreePopup( );
 	void dblClick( );
 	void selectItem( bool force = false );
-	
+
     private:
 	//Private attributes
-	QTreeWidget *treeW;
-	QPoint      dragStartPos;
+	QTreeWidget	*treeW;
+	QPoint		dragStartPos;
 };
- 
+
 //****************************************
 //* Project's tree                       *
 //**************************************** 
@@ -345,17 +347,17 @@ class ProjTree: public QDockWidget
 	//Public methods
 	ProjTree( VisDevelop * parent = 0 );
 	~ProjTree();
-	
+
 	bool hasFocus( );
-	
-	VisDevelop *owner( );	
+
+	VisDevelop *owner( );
 
     signals:
         void selectItem( const string &idwdg, bool force = false );
 
-    public slots:	
+    public slots:
 	void updateTree( const string &vca_it = "", QTreeWidgetItem *it = NULL );
-	
+
     protected:
 	//Protecten methods
 	bool eventFilter( QObject *target, QEvent *event );
@@ -365,10 +367,10 @@ class ProjTree: public QDockWidget
 	void ctrTreePopup( );
 	void dblClick( );
 	void selectItem( bool force = false );
-	
+
     private:
 	//Private attributes
-	QTreeWidget *treeW;	
+	QTreeWidget *treeW;
 };
 
 //**********************************************************************************************
@@ -379,24 +381,24 @@ class LineEditProp : public QWidget
     Q_OBJECT
 
     public:
-        //- Data -
-        enum DType { Font, Color };
+	//- Data -
+	enum DType { Font, Color };
 
-        //- Methods -
-        LineEditProp( QWidget *parent, DType tp = Font );
+	//- Methods -
+	LineEditProp( QWidget *parent, DType tp = Font );
 
-        DType type( )       { return m_tp; }
-        QString value( );
+	DType type( )       { return m_tp; }
+	QString value( );
 
-        void setType( DType tp )	{ m_tp = tp; }
-        void setValue( const QString& );
+	void setType( DType tp )	{ m_tp = tp; }
+	void setValue( const QString& );
 
     private slots:
 	void callDlg( );
 
     private:
-        DType       m_tp;
-	QLineEdit   *ed_fld;
+	DType		m_tp;
+	QLineEdit	*ed_fld;
 };
 
 //*********************************************
@@ -409,14 +411,14 @@ class WScaleStBar : public QLabel
     public:
 	WScaleStBar( QWidget *parent = 0 );
 
-        bool scale()			{ return isScale; }
-        void setScale( bool val );
-						
+	bool scale()			{ return isScale; }
+	void setScale( bool val );
+
     protected:
 	void mousePressEvent( QMouseEvent * event );
-	
+
     private:
-        bool isScale;
+	bool isScale;
 };
 
 //*************************************************
@@ -427,26 +429,26 @@ class SizePntWdg : public QWidget
     Q_OBJECT
 
     public:
-        //Data
-        enum WView { SizeDots, EditBorder, SelectBorder };
-        //Methods
-        SizePntWdg( QWidget* parent = 0 );
+	//Data
+	enum WView { Hide, SizeDots, EditBorder, SelectBorder };
+	//Methods
+	SizePntWdg( QWidget* parent = 0 );
+
+	QPointF posF( )             { return w_pos; }
+	QSizeF  sizeF( )            { return w_size; }
+	QRectF geometryF( )         { return QRectF(w_pos,w_size); }
 
-        QPointF posF( )             { return w_pos; }
-        QSizeF  sizeF( )            { return w_size; }
-        QRectF geometryF( )         { return QRectF(w_pos,w_size); }
-        
 	void setSelArea( const QRectF &geom, WView view = SizeDots );
-        bool event( QEvent *event );
+	bool event( QEvent *event );
 
     public slots:
 	void apply( );
-    
+
     private:
-        //Attributes
-        WView   view;
-        QPointF w_pos;          //Widget position into real;
-        QSizeF  w_size;         //Widget size into real;
+	//Attributes
+	WView	view;
+	QPointF	w_pos;		//Widget position into real;
+	QSizeF	w_size;		//Widget size into real;
 };
 
 //*************************************************
@@ -455,68 +457,75 @@ class SizePntWdg : public QWidget
 class DevelWdgView: public WdgView
 {
     Q_OBJECT
-		
+
     public:
-	//Data
-	enum DevelFlgs
-	{
-	    makeScale 	= 0x01,	//Make visual item scaling
-	    wdgEdit   	= 0x02,	//Widget under edition
-	    wdgSelect 	= 0x04,	//Widget is selected
-	    moveHold  	= 0x08,	//Mouse move hold state
-	    holdChild 	= 0x10,	//Hold child widget in time of moving and resizing
-	    leftTop   	= 0x20,	//Left top anchors
-	    holdSelRect = 0x40,	//Hold for select rect
-	    moveHoldMove= 0x80	//Mouse move on hold
-	};
-    
 	//Public methods
-        DevelWdgView( const string &iwid, int ilevel, VisDevelop *mainWind, QWidget* parent = 0 );
+	DevelWdgView( const string &iwid, int ilevel, VisDevelop *mainWind, QWidget* parent = 0 );
 	~DevelWdgView( );
-	
+
 	string user( );
 	VisDevelop *mainWin( );
-	
-        bool select( )		{ return m_flgs&wdgSelect; }	//Select widget state
-        string selectChilds( int *cnt = NULL, vector<DevelWdgView*> *wdgs = NULL );     	//Get selected include widgets list
-        bool edit( )		{ return m_flgs&wdgEdit; }     	//Edit mode state
-	
-        void setSelect( bool vl, bool childs = true );
-        void setEdit( bool vl );
+
+	bool select( )		{ return fWdgSelect; }		//Select widget state
+	string selectChilds( int *cnt = NULL, vector<DevelWdgView*> *wdgs = NULL );	//Get selected include widgets list
+	bool edit( )		{ return fWdgEdit; }		//Edit mode state
+
+        void setSelect( bool vl, bool childs = true, bool onlyFlag = false );
+	void setEdit( bool vl );
 	void wdgsMoveResize( const QPointF &dP );
-	
+
 	WdgView *newWdgItem( const string &iwid );
 
 	void load( const string& item, bool load = true, bool init = true );
-						
+
+	DevelWdgView *levelWidget( int lev );
+
+	//- Resource and cache operations -
+	string resGet( const string &res );
+	string cacheResGet( const string &res );
+	void cacheResSet( const string &res, const string &val );
+
     signals:
-        void selected( const string& item );        //Change selection signal
+	void selected( const string& item );		//Change selection signal
 	void apply( const string & );
-    
+
     public slots:
-        void wdgViewTool( QAction* );               //View order and align of included widgets operated
+	void wdgViewTool( QAction* );			//View order and align of included widgets operated
 	void saveGeom( const string& item );
 	void wdgPopup( );
 	void makeIcon( );
+	void makeImage( );
 	void editEnter( );
 	void editExit( );
-	
+
     protected:
-        //Protected methods
+	//Protected methods
+	void childsUpdate( bool newLoad = true );
 	bool event( QEvent * event );
 	int cntrIfCmd( XMLNode &node, bool glob = false );
-    
+
     private:
 	//Private methods
-        bool grepAnchor( const QPointF &apnt, const QPoint &cpnt );
-        void upMouseCursors( const QPoint &pnt );
-	
-        //Private attributes
-	char		m_flgs;		//Developmen flags
-	QPoint 		holdPnt;	//Hold move point
-        SizePntWdg 	*pntView;	//Point view
+	bool grepAnchor( const QPointF &apnt, const QPoint &cpnt );
+	void upMouseCursors( const QPoint &pnt );
+
+	//Private attributes
+	short int	fMakeScale	:1;	//Make visual item scaling
+	short int	fWdgEdit	:1;	//Widget under edition
+	short int	fWdgSelect	:1;	//Widget is selected
+	short int	fMoveHold	:1;	//Mouse move hold state
+	short int	fHoldChild	:1;	//Hold child widget in time of moving and resizing
+	short int	fLeftTop	:1;	//Left top anchors
+	short int	fHoldSelRect	:1;	//Hold for select rect
+	short int	fMoveHoldMove	:1;	//Mouse move on hold
+	short int	fHideChilds	:1;	//Hide childs on move
+	short int	fSelChange	:1;	//Changed select map
+
+	QPoint		holdPnt;		//Hold move point
+	SizePntWdg	*pntView;		//Point view
 	DevelWdgView	*editWdg;
-        QPoint 		dragStartPos;
+	QPoint		dragStartPos;
+	map<string,string>	mCacheRes;	//Resources cache
 };
 
 }
