@@ -1,23 +1,23 @@
 
 //OpenSCADA system module UI.VISION file: vis_shape_elfig.h
 /***************************************************************************
- *   Copyright (C) 2007 by Lysenko Maxim (mlisenko@ukr.net)
- *   			by Yashina Kseniya (sobacurka@ukr.net)
- *                                                                         
- *   This program is free software; you can redistribute it and/or modify  
- *   it under the terms of the GNU General Public License as published by  
- *   the Free Software Foundation; either version 2 of the License, or     
- *   (at your option) any later version.                                   
- *                                                                         
- *   This program is distributed in the hope that it will be useful,       
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         
- *   GNU General Public License for more details.                          
- *                                                                         
- *   You should have received a copy of the GNU General Public License     
- *   along with this program; if not, write to the                         
- *   Free Software Foundation, Inc.,                                       
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             
+ *   Copyright (C) 2007-2008 by Lysenko Maxim (mlisenko@ukr.net)
+ *   			     by Yashina Kseniya (sobacurka@ukr.net)
+ *
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the
+ *   Free Software Foundation, Inc.,
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ***************************************************************************/
 
 #ifndef VIS_SHAPE_ELFIG_H
@@ -69,15 +69,18 @@ class inundationItem
     public:
         //Methods
         inundationItem( )	{ }
-        inundationItem( const QPainterPath &ipath, const QBrush &ibrush, QBrush &ibrush_img, const QVector<int> &inumber_shape ) : 
-                        brush(ibrush), brush_img(ibrush_img), path(ipath), number_shape(inumber_shape)
+        inundationItem( const QPainterPath &ipath, const QBrush &ibrush, QBrush &ibrush_img,
+                        const QVector<int> &inumber_shape, const QVector<int> &inumber_point ) : 
+                        brush(ibrush), brush_img(ibrush_img), path(ipath), number_shape(inumber_shape),
+                       number_point(inumber_point)       
         { };
 
         //Attributes
         QPainterPath   	path; 
         QBrush         	brush, brush_img;
         QVector<int>  	number_shape;
-        QString        	img_nm;
+        QVector<int>    number_point;
+        //QString        	img_nm;
 };
 
 
@@ -131,7 +134,9 @@ class ShapeElFigure : public WdgShape
 			rectPath,		//Path fo RectItem creation
         		ellipse_startPath, ellipse_endPath,
         		ellipse_draw_startPath, ellipse_draw_endPath,
-                        inundationPath;
+                        inundationPath,
+                        dashedRectPath;
+        QRect           dashedRect;
     
 	bool status; 				//Check fo any primitive paint key pressed
 	int shapeType; 				//Selected figure type
@@ -141,6 +146,31 @@ class ShapeElFigure : public WdgShape
         void toolAct( QAction* );
     
     private:
+        //Data
+        class ElFigDt
+        {
+            public:
+		//Methods
+                ElFigDt( ) : en(true), active(true), geomMargin(0), lineWdth(1), lineDecor(0),
+                        bordWdth(0), orient(0)  { }
+		//Attributes
+                short	en          :1;
+                short	active      :1;
+                short   geomMargin  :8;
+                short   lineDecor   :4;
+                short   bordWdth    :8;
+                float   lineWdth;
+                QColor  lineClr;
+                QColor  bordClr;
+                QColor  fillClr;
+                QBrush  fillImg;
+                double  orient;
+                string  elLst;
+                string  lineStyle;
+                QVector<ShapeItem> shapeItems;
+                QVector<inundationItem> inundationItems;
+                PntMap  shapePnts;
+        };
     	//Methods
         int itemAt( const QPointF &pos, const QVector<ShapeItem> &shapeItems, WdgView *w );			//Check for figure type under cursor
         void moveItemTo( const QPointF &pos, QVector<ShapeItem> &shapeItems, PntMap *pnts, WdgView *w );	//Move figure procedure
@@ -158,6 +188,7 @@ class ShapeElFigure : public WdgShape
         void rectNum0_1( const QVector<ShapeItem> &shapeItems, int rect_num_temp, PntMap *pnts, WdgView *w );
         void rectNum3_4( const QVector<ShapeItem> &shapeItems);
         void moveAll( const QPointF &pos, QVector<ShapeItem> &shapeItems, PntMap *pnts, QVector<inundationItem> &inundationItems, WdgView *w );
+        void checkPoint_checkInundation( QVector<ShapeItem> &shapeItems, PntMap *pnts, QVector<inundationItem> &inundationItems );
         QPainterPath painterPath( float el_width, float el_border_width, int el_type, double el_ang,
 		QPointF el_p1 = QPointF(0,0), QPointF el_p2 = QPointF(0,0), QPointF el_p3 = QPointF(0,0), 
 		QPointF el_p4 = QPointF(0,0), QPointF el_p5 = QPointF(0,0), QPointF el_p6 = QPointF(0,0) );
@@ -170,8 +201,8 @@ class ShapeElFigure : public WdgShape
         bool inundation( const QPointF &point, const QVector<ShapeItem> &shapeItems, PntMap *pnts, const QVector<int> &vect, int N, WdgView *w );
         bool inundation1_2( const QPointF &point, const QVector<ShapeItem> &shapeItems, QVector<inundationItem> &inundationItems, PntMap *pnts, WdgView *w, int number );
         int  buildMatrix( const QVector<ShapeItem> &shapeItems );
-        QPointF scaleRotate( const QPointF &point, WdgView *w, bool flag_scale );
-        QPointF unScaleRotate( const QPointF &point, WdgView *w, bool flag_scale );
+        QPointF scaleRotate( const QPointF &point, WdgView *w, bool flag_scale, bool flag_rotate );
+        QPointF unScaleRotate( const QPointF &point, WdgView *w, bool flag_scale, bool flag_rotate );
         QPainterPath createInundationPath( const QVector<int> &in_fig_num, const QVector<ShapeItem> &shapeItems, PntMap *pnts, WdgView *w );
         QVector<int> inundationSort( const QPainterPath &inundationPath, QVector<int> &inundation_fig_num, const QVector<ShapeItem> &shapeItems, PntMap *pnts, WdgView *w );
       	
@@ -186,22 +217,24 @@ class ShapeElFigure : public WdgShape
     
         QVector<int> index_array;
         QVector<int> rect_array;
+        QVector<int> copy_index, index_array_copy, index_array_copy_flag_A;
             
 	int count_Shapes, count_moveItemTo,
 	    index, index_temp, index_del,index_inund,
 	    rect_num;
 	bool flag_cursor, flag_key, flag_up, flag_down, flag_left, flag_right, 
-	     flag_ctrl, flag_ctrl_move, flag_m, flag_hold_arc, flag_A, flag_hold_checked;
+             flag_ctrl, flag_ctrl_move, flag_m, flag_hold_arc, flag_A, flag_copy, flag_check_pnt_inund;
 					    	
-        bool flag_rect, flag_arc_rect_3_4, flag_arc_release, flag_first_move, Flag;
+        bool flag_rect, flag_arc_rect_3_4, flag_arc_release, flag_first_move, flag_move, flag_release;
         QPointF Start_offset, End_offset, CtrlPos1_offset, CtrlPos2_offset, CtrlPos3_offset, CtrlPos4_offset;
         int count_rects, rect_num_arc, arc_rect;
         bool flag_holds;			
         bool flag_hold_move;			
         bool flag_inund_break;
-        bool flag_scaleRotate;
+        bool flag_scale, flag_rotate;
         double t_start, t_end;			
-	QPointF Mouse_pos, offset;		
+	QPointF Mouse_pos, offset;	
+        QPoint stPointDashedRect;	
         int current_ss, current_se, current_ee, current_es;
         int count_holds;			
         QVector<int> arc_rect_array, fig_rect_array, vect;
