@@ -1,7 +1,7 @@
 
 //OpenSCADA system module UI.Vision file: vis_shapes.h
 /***************************************************************************
- *   Copyright (C) 2007 by Roman Savochenko
+ *   Copyright (C) 2007-2008 by Roman Savochenko
  *   rom_as@diyaorg.dp.ua
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -24,15 +24,22 @@
 #define VIS_SHAPES_H
 
 #include <string>
-#include <deque> 
+#include <deque>
+#include <vector>
 
 #include <QObject>
 #include <QMap>
 #include <QVariant>
 #include <QMenu>
+#include <QPen>
+#include <QLabel>
+#include <QPicture>
+#include <QTableWidget>
+#include <QScrollArea>
 
 using std::string;
-using std::deque; 
+using std::deque;
+using std::vector;
 
 class QEvent;
 class QAction;
@@ -53,26 +60,26 @@ class WdgShape : public QObject
     public:
 	//Methods
 	WdgShape( const string &iid );
-	
+
 	string id( )									{ return m_id; }
-    
-        virtual bool isEditable( ) 							{ return false; }
-	
+
+	virtual bool isEditable( )							{ return false; }
+
 	virtual void init( WdgView *view )						{ }
 	virtual void destroy( WdgView *view )						{ }
-	
+
 	virtual void editEnter( WdgView *view )						{ }
 	virtual void editExit( WdgView *view )						{ }
 	virtual void wdgPopup( WdgView *view, QMenu &menu )				{ }
-	
-	virtual bool attrSet( WdgView *view, int uiPrmPos, const string &val )		{ return false; }
-	
+
+	virtual bool attrSet( WdgView *view, int uiPrmPos, const string &val );
+
 	virtual bool event( WdgView *view, QEvent *event );
 	virtual bool eventFilter( WdgView *view, QObject *object, QEvent *event )	{ }
 
     protected:
 	//Methods
-	void borderDraw( QPainter &pnt, QRect dA, QPen bpen, int bordStyle );    
+	void borderDraw( QPainter &pnt, QRect dA, QPen bpen, int bordStyle );
 
     private:
 	//Attributes
@@ -90,24 +97,24 @@ class ShapeFormEl : public WdgShape
 
     public:
 	ShapeFormEl( );
-	
+
 	void init( WdgView *view );
-	void destroy( WdgView *view );	
+	void destroy( WdgView *view );
 	bool attrSet( WdgView *view, int uiPrmPos, const string &val);
-	bool event( WdgView *view, QEvent *event );	
+	bool event( WdgView *view, QEvent *event );
 	bool eventFilter( WdgView *view, QObject *object, QEvent *event );
-    
+
     public slots:
 	//-- Edit line events --
 	void lineAccept( );
-	//-- Edit text events --	
+	//-- Edit text events --
 	void textAccept( );
 	//-- Check box events --
 	void checkChange(int);
 	//-- Combo box and list events --
 	void comboChange(const QString&);
 	//-- List events --
-	void listChange(int);	
+	void listChange(int);
 	//-- Button's events --
 	void buttonPressed( );
 	void buttonReleased( );
@@ -116,8 +123,33 @@ class ShapeFormEl : public WdgShape
 	void sliderMoved( int val );
 
     private:
-	//- Private methods -
-	//Recursively widgets process for disable focusable and events filter set    
+	//Data
+	//- Shape node date -
+	class ShpDt
+	{
+	    public:
+		//Methods
+		ShpDt( ) : en(true), active(true), evLock(false), addrWdg(NULL), elType(-1), welType(-1) { }
+		//Attributes
+		short	en	:1;
+		short	active	:1;
+		short	evLock	:1;
+		short	elType	:4;
+		short	welType	:4;
+		short	view	:4;
+		short	wordWrap:1;
+		short	checkable:1;
+		QWidget	*addrWdg;
+		QFont	font;
+		string	name;
+		string	value;
+		string	img;
+		string	items;
+		string	cfg;
+		string	color;
+	};
+	//Private methods
+	//Recursively widgets process for disable focusable and events filter set
 	void eventFilterSet( WdgView *view, QWidget *wdg, bool en );
 	void setFocus(WdgView *view, QWidget *wdg, bool en = false, bool devel = false );
 };
@@ -136,26 +168,48 @@ class ShapeText : public WdgShape
 	bool event( WdgView *view, QEvent *event );
 
     private:
- 	//Data
+	//Data
 	//- Argument object's class -
 	class ArgObj
 	{
 	    public:
 		//Methods
 		ArgObj( )				{ };
-		
+
 		string &cfg( )				{ return m_cfg; }
 		QVariant val( )				{ return m_val; }
 
 		void setCfg( const string &vl )		{ m_cfg = vl; }
 		void setVal( const QVariant &vl )	{ m_val = vl; }
-		
-	    private:		
+
+	    private:
 		//Attributes
 		QVariant	m_val;
 		string		m_cfg;
-	};        
-}; 
+	};
+	//- Shape node date -
+	class ShpDt
+	{
+	    public:
+		//Methods
+		ShpDt( ) : en(true), active(true), geomMargin(0), text_flg(0)	{ }
+		//Attributes
+		short	en		:1;
+		short	active		:1;
+		short	geomMargin	:8;
+		short	bordStyle	:5;
+		short int orient;
+		short int text_flg;
+		string	text_tmpl,
+			text;
+		QColor	color,
+			backColor;
+		QBrush	backImg;
+		QFont	font;
+		QPen	border;
+		vector<ArgObj>	args;
+	};
+};
 
 //************************************************
 //* Media view shape widget                      *
@@ -165,28 +219,48 @@ class ShapeMedia : public WdgShape
     public:
 	//Methods
 	ShapeMedia( );
-	
+
 	void init( WdgView *view );
-	void destroy( WdgView *view );	
+	void destroy( WdgView *view );
 	bool attrSet( WdgView *view, int uiPrmPos, const string &val);
 	bool event( WdgView *view, QEvent *event );
 
     private:
 	//Data
-        //- Map areas structure -
+	//- Map areas structure -
 	class MapArea
 	{
 	    public:
 		//Methods
 		MapArea( ) : shp(-1)	{ }
-		
+
 		bool containsPoint( const QPoint & point );
-		
+
 		//Attributes
-		int 	shp;		//Area shape
+		int	shp;		//Area shape
 		string	title;		//Area title
 		QVector<QPoint>	pnts;	//Area points
-	};  
+	};
+	//- Shape node date -
+	class ShpDt
+	{
+	    public:
+		//Methods
+		ShpDt( ) : en(true), mediaType(-1), geomMargin(0)	{ }
+		//Attributes
+		short	en		:1;
+		short	geomMargin	:8;
+		short	bordStyle	:5;
+		short	mediaType	:3;
+		short	mediaFit	:1;
+		short	mediaSpeed	:8;
+		QColor	backColor;
+		QBrush	backImg;
+		QLabel	*labWdg;
+		QPen	border;
+		string	mediaSrc;
+		vector<MapArea>	maps;
+	};
 };
 
 //************************************************
@@ -199,12 +273,12 @@ class ShapeDiagram : public WdgShape
     public:
 	//Methods
 	ShapeDiagram( );
-	
+
 	void init( WdgView *view );
 	void destroy( WdgView *view );
 	bool attrSet( WdgView *view, int uiPrmPos, const string &val);
 	bool event( WdgView *view, QEvent *event );
-	
+
     private slots:
 	void tracing( );	//Trends tracing
 
@@ -215,25 +289,25 @@ class ShapeDiagram : public WdgShape
 	{
 	    public:
 		//Data
-		class SHg  
-		{ 
+		class SHg
+		{
 		    public:
 			SHg( long long itm, double ival ) : tm(itm), val(ival) { }
-			long long tm; 
+			long long tm;
 			double val;
 		};
 		//Methods
 		TrendObj( WdgView *view );
-		
-		string addr( )		{ return m_addr; }
-		double bordL( )		{ return m_bord_low; }
-		double bordU( )		{ return m_bord_up; }
-		string color( )		{ return m_color; }
-		double curVal( )	{ return m_curvl; }
-		int    valTp( )		{ return val_tp; }
+
+		string	addr( )		{ return m_addr; }
+		double	bordL( )	{ return m_bord_low; }
+		double	bordU( )	{ return m_bord_up; }
+		string	color( )	{ return m_color; }
+		double	curVal( )	{ return m_curvl; }
+		int	valTp( )	{ return val_tp; }
 		long long valBeg( );
 		long long valEnd( );
-		int val( long long tm );
+		int	val( long long tm );
 		deque<SHg> &val( )	{ return vals; }
 
 		void setAddr( const string &vl );
@@ -243,35 +317,62 @@ class ShapeDiagram : public WdgShape
 		void setCurVal( double vl )	{ m_curvl = vl; }
 
 		void loadData( bool full = false );
-		
-	    private:		
+
+	    private:
 		//Attributes
-		string 		m_addr;		//A parameter or an archive item address
+		string		m_addr;		//A parameter or an archive item address
 		double m_bord_low, m_bord_up;	//Borders
 		double		m_curvl;	//Curent value
-		string 		m_color;	//Values line color
-                //- Archive -		
-        	int		arh_per;	//Archive period
-		long long       arh_beg;        //Archive begin time
-		long long       arh_end;        //Archive end time
+		string		m_color;	//Values line color
+		//- Archive -
+		int		arh_per;	//Archive period
+		long long	arh_beg;	//Archive begin time
+		long long	arh_end;	//Archive end time
 		//- Values -
-		int    		val_tp;		//Values type
+		int		val_tp;		//Values type
 		deque<SHg>	vals;		//Values buffer
-		
+
 		WdgView 	*view;
 	};
-	
+	//- Shape node date -
+	class ShpDt
+	{
+	    public:
+		//Methods
+		ShpDt( ) : en(true), active(true), geomMargin(0), tTime(0), tPict(0), curTime(0), tTimeCurent(false), trcPer(0)	{ }
+		//Attributes
+		short	en		:1;
+		short	active		:1;
+		short	geomMargin	:8;
+		short	bordStyle	:5;
+		short	tTimeCurent	:1;
+		short	trcPer		:10;
+		short	sclHor		:4;
+		short	sclVer		:4;
+		QColor	backColor, curColor, sclColor, sclMarkColor;
+		QBrush	backImg;
+		QPen		border;
+		QPicture	pictObj;
+		QRect		pictRect;
+		QFont		sclMarkFont;
+		QTimer 		*trcTimer;
+		vector<TrendObj>	prms;
+		long long	tTime, tPict, curTime;
+		float		tSize;
+		string		valArch;
+	};
+
 	//Methods
 	void makeTrendsPicture( WdgView *view );
 	void loadTrendsData( WdgView *view, bool full = false );
 	void setCursor( WdgView *view, long long itm );
-};    
+};
 
 //************************************************
 //* Protocol view shape widget                   *
 //************************************************
 class ShapeProtocol : public WdgShape
-{    
+{
     Q_OBJECT
 
     public:
@@ -287,11 +388,32 @@ class ShapeProtocol : public WdgShape
 	void tracing( );	//Trends tracing
 
     private:
- 	//Private methods
+	//Data
+	//- Shape node date -
+	class ShpDt
+	{
+	    public:
+		//Methods
+		ShpDt( ) : active(true), time(0), tSize(60), timeCurent(false), arhBeg(0), arhEnd(0)	{ }
+		//Attributes
+		short	active		:1;
+		short	timeCurent	:1;
+		short	trcPer		:10;
+		short	lev		:4;
+		short	viewOrd		:3;
+		short	itProp		:3;
+		QTimer 		*trcTimer;
+		QTableWidget	*addrWdg;
+		QMap<QString,int> indMap;
+		unsigned int	time, tSize;
+		unsigned int	arhBeg, arhEnd;
+		string		arch, tmpl, col;
+	};
+	//Private methods
 	void loadData( WdgView *view, bool full = false );
 	//- Recursively widgets process for disable focusable and events filter set -
-	void eventFilterSet( WdgView *view, QWidget *wdg, bool en );    
-	void setFocus(WdgView *view, QWidget *wdg, bool en = false, bool devel = false );	
+	void eventFilterSet( WdgView *view, QWidget *wdg, bool en );
+	void setFocus(WdgView *view, QWidget *wdg, bool en = false, bool devel = false );
 };
 
 //************************************************
@@ -317,9 +439,12 @@ class ShapeFunction : public WdgShape
 //************************************************
 //* User element shape widget                    *
 //************************************************
+class RunPageView;
+
 class ShapeBox : public WdgShape
 {
     public:
+	//Methods
 	ShapeBox( );
 
 	void init( WdgView *view );
@@ -327,6 +452,25 @@ class ShapeBox : public WdgShape
 
 	bool attrSet( WdgView *view, int uiPrmPos, const string &val);
 	bool event( WdgView *view, QEvent *event );
+
+    private:
+	//Attributes
+	//- Shape node date -
+	class ShpDt
+	{
+	    public:
+		//Methods
+		ShpDt( ) : en(true), geomMargin(0), bordStyle(0), inclWidget(NULL), inclScrl(NULL)	{ }
+		//Attributes
+		short	en		:1;
+		short	geomMargin	:8;
+		short	bordStyle	:5;
+		QPen		border;
+		QColor		backColor;
+		QBrush		backImg;
+		RunPageView	*inclWidget;
+		QScrollArea 	*inclScrl;
+	};
 };
 
 //************************************************
