@@ -1,7 +1,7 @@
 
 //OpenSCADA system module UI.Vision file: vis_widgs.cpp
 /***************************************************************************
- *   Copyright (C) 2007 by Roman Savochenko                                *
+ *   Copyright (C) 2007-2008 by Roman Savochenko                           *
  *   rom_as@diyaorg.dp.ua                                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -42,6 +42,7 @@
 #include "vis_shapes.h"
 #include "tvision.h"
 #include "vis_widgs.h"
+#include "vis_run_widgs.h"
 
 using namespace VISION;
 
@@ -56,46 +57,46 @@ InputDlg::InputDlg( QWidget *parent, const QIcon &icon, const QString &mess,
     setMinimumSize( QSize( 120, 150 ) );
     setWindowIcon(icon);
     setSizeGripEnabled(true);
-    
+
     QVBoxLayout *dlg_lay = new QVBoxLayout(this);
     dlg_lay->setMargin(10);
     dlg_lay->setSpacing(6);
-    
+
     //Icon label and text message
     QHBoxLayout *intr_lay = new QHBoxLayout;
-    intr_lay->setSpacing(6);    
-    
+    intr_lay->setSpacing(6);
+
     QLabel *icon_lab = new QLabel(this);
     icon_lab->setSizePolicy( QSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum) );
     icon_lab->setPixmap(icon.pixmap(48));
     intr_lay->addWidget(icon_lab);
-    
+
     inp_lab = new QLabel(mess,this);
     //inp_lab->setAlignment(Qt::AlignHCenter);
     inp_lab->setWordWrap(true);
     intr_lay->addWidget(inp_lab);
     dlg_lay->addItem(intr_lay);
-    
+
     //Id and name fields
     if( with_nm || with_id )
     {
 	QGridLayout *ed_lay = new QGridLayout;
 	ed_lay->setSpacing(6);
-	if( with_id ) 
+	if( with_id )
 	{
 	    ed_lay->addWidget( new QLabel(_("ID:"),this), 0, 0 );
 	    m_id = new QLineEdit(this);
 	    ed_lay->addWidget( m_id, 0, 1 );
-	}	    
+	}
 	if( with_nm )
 	{
 	    ed_lay->addWidget( new QLabel(_("Name:"),this), 1, 0 );
 	    m_name = new QLineEdit(this);
 	    ed_lay->addWidget( m_name, 1, 1 );
 	}
-	dlg_lay->addItem(ed_lay);	
+	dlg_lay->addItem(ed_lay);
     }
-    
+
     //Qk and Cancel buttons
     dlg_lay->addItem( new QSpacerItem( 10, 0, QSizePolicy::Minimum, QSizePolicy::Expanding ) );
 
@@ -103,9 +104,9 @@ InputDlg::InputDlg( QWidget *parent, const QIcon &icon, const QString &mess,
     sep->setFrameShape( QFrame::HLine );
     sep->setFrameShadow( QFrame::Raised );
     dlg_lay->addWidget( sep );
-    
-    QDialogButtonBox *but_box = new QDialogButtonBox(QDialogButtonBox::Ok|
-                                    		     QDialogButtonBox::Cancel,Qt::Horizontal,this);
+
+    QDialogButtonBox *but_box = new QDialogButtonBox( QDialogButtonBox::Ok|
+						      QDialogButtonBox::Cancel, Qt::Horizontal, this );
     QImage ico_t;
     but_box->button(QDialogButtonBox::Ok)->setText(_("Ok"));
     if(!ico_t.load(TUIS::icoPath("button_ok").c_str())) ico_t.load(":/images/button_ok.png");
@@ -116,7 +117,7 @@ InputDlg::InputDlg( QWidget *parent, const QIcon &icon, const QString &mess,
     but_box->button(QDialogButtonBox::Cancel)->setIcon(QPixmap::fromImage(ico_t));
     connect(but_box, SIGNAL(rejected()), this, SLOT(reject()));
     dlg_lay->addWidget( but_box );
-    
+
     resize(400,120+(40*with_nm)+(40*with_id));
 }
 
@@ -155,24 +156,26 @@ void InputDlg::setMess( const QString &val )
 //*************************************************
 //* User select dialog                            *
 //*************************************************
-DlgUser::DlgUser( QWidget *parent ) : QDialog(parent)
+DlgUser::DlgUser( const QString &iuser, const QString &ipass, const QString &iVCAstat, QWidget *parent ) : 
+    QDialog(parent), VCAstat(iVCAstat)
 {
     setWindowTitle(_("Select user"));
-    
+
     QVBoxLayout *dlg_lay = new QVBoxLayout(this);
     dlg_lay->setMargin(10);
     dlg_lay->setSpacing(6);
 
     QGridLayout *ed_lay = new QGridLayout;
-    ed_lay->setSpacing(6);    
-    ed_lay->addWidget( new QLabel(_("User:"),this), 0, 0 );    
+    ed_lay->setSpacing(6);
+    ed_lay->addWidget( new QLabel(_("User:"),this), 0, 0 );
     users = new QComboBox(this);
+    users->setEditable(true);
     ed_lay->addWidget( users, 0, 1 );
-    ed_lay->addWidget( new QLabel(_("Password:"),this), 1, 0 );    
+    ed_lay->addWidget( new QLabel(_("Password:"),this), 1, 0 );
     passwd = new QLineEdit(this);
     passwd->setEchoMode( QLineEdit::Password );
     ed_lay->addWidget( passwd, 1, 1 );
-    dlg_lay->addItem(ed_lay);            
+    dlg_lay->addItem(ed_lay);
 
     dlg_lay->addItem( new QSpacerItem( 20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding ) );
 
@@ -181,9 +184,9 @@ DlgUser::DlgUser( QWidget *parent ) : QDialog(parent)
     sep->setFrameShadow( QFrame::Raised );
     dlg_lay->addWidget( sep );
 
-    QDialogButtonBox *but_box = new QDialogButtonBox(QDialogButtonBox::Ok|
-                                    		     QDialogButtonBox::Cancel,Qt::Horizontal,this);
-    QImage ico_t;    
+    QDialogButtonBox *but_box = new QDialogButtonBox( QDialogButtonBox::Ok|
+						      QDialogButtonBox::Cancel, Qt::Horizontal, this );
+    QImage ico_t;
     but_box->button(QDialogButtonBox::Ok)->setText(_("Ok"));
     if(!ico_t.load(TUIS::icoPath("button_ok").c_str())) ico_t.load(":/images/button_ok.png");
     but_box->button(QDialogButtonBox::Ok)->setIcon(QPixmap::fromImage(ico_t));
@@ -193,20 +196,17 @@ DlgUser::DlgUser( QWidget *parent ) : QDialog(parent)
     but_box->button(QDialogButtonBox::Cancel)->setIcon(QPixmap::fromImage(ico_t));
     connect(but_box, SIGNAL(rejected()), this, SLOT(reject()));
     dlg_lay->addWidget( but_box );
-    
+
     connect(this, SIGNAL(finished(int)), this, SLOT(finish(int)));
-    
-    //Fill users list
-    vector<string> u_list;	
-    SYS->security().at().usrList(u_list);
-    for(int i_l = 0; i_l < u_list.size(); i_l++ )
-    {
-        string simg = TSYS::strDecode(SYS->security().at().usrAt(u_list[i_l]).at().picture(),TSYS::base64);
-        QImage img;
-        if( img.loadFromData((const uchar*)simg.c_str(),simg.size()) )
-	    users->addItem(QPixmap::fromImage(img),u_list[i_l].c_str());
-	else users->addItem(u_list[i_l].c_str());
-    }
+
+    //- Fill users list -
+    XMLNode req("get");
+    req.setAttr("path","/Security/%2fusgr%2fusers");
+    if( !mod->cntrIfCmd(req,iuser.toAscii().data(),ipass.toAscii().data(),iVCAstat.toAscii().data(),true) )
+	for( int i_u = 0; i_u < req.childSize(); i_u++ )
+	    users->addItem(req.childGet(i_u)->text().c_str());
+
+    users->setEditText(iuser);
 }
 
 QString DlgUser::user()
@@ -223,9 +223,10 @@ void DlgUser::finish( int result )
 {
     if( result )
     {
-	//Check user
-	if( SYS->security().at().usrPresent(user().toAscii().data()) && 
-		SYS->security().at().usrAt(user().toAscii().data()).at().auth(password().toAscii().data()) )
+	//- Check user auth -
+	XMLNode req("get");
+	req.setAttr("path",string("/Security/")+user().toAscii().data()+"/%2fauth")->setAttr("password",password().toAscii().data());
+	if( !mod->cntrIfCmd(req,user().toAscii().data(),password().toAscii().data(),VCAstat.toAscii().data(),true) && atoi(req.text().c_str()) )
 	    setResult(SelOK);
 	else setResult(SelErr);
     }
@@ -238,18 +239,18 @@ void DlgUser::finish( int result )
 FontDlg::FontDlg( QWidget *parent, const QString &ifnt )
 {
     setWindowTitle(_("Font select"));
-    
+
     QGridLayout *dlg_lay = new QGridLayout(this);
     dlg_lay->setMargin(10);
-    dlg_lay->setSpacing(6);    
-    
+    dlg_lay->setSpacing(6);
+
     QLabel *lab = new QLabel(_("Font:"),this);
     lab->setSizePolicy( QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed) );
-    dlg_lay->addWidget(lab,0,0,1,2);    
+    dlg_lay->addWidget(lab,0,0,1,2);
     fntSel = new QFontComboBox(this);
     connect(fntSel, SIGNAL(currentFontChanged(const QFont&)), this, SLOT(cfgChange()));
     dlg_lay->addWidget(fntSel,1,0,1,2);
-    
+
     QVBoxLayout *sz_lay = new QVBoxLayout;
     sz_lay->setSpacing(6);
     lab = new QLabel(_("Size:"),this);
@@ -273,7 +274,7 @@ FontDlg::FontDlg( QWidget *parent, const QString &ifnt )
     grpBox->setLayout(grpLay);
     sz_lay->addWidget(grpBox);
     dlg_lay->addItem(sz_lay,2,0);
-    
+
     grpBox = new QGroupBox(_("Sample:"),this);
     grpLay = new QVBoxLayout;
     grpLay->setMargin(5);
@@ -283,7 +284,7 @@ FontDlg::FontDlg( QWidget *parent, const QString &ifnt )
     grpLay->addWidget(sampleText);
     grpBox->setLayout(grpLay);
     dlg_lay->addWidget(grpBox,2,1);
-    
+
     dlg_lay->addItem( new QSpacerItem( 20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding ), 3, 0, 1, 2 );
 
     QFrame *sep = new QFrame(this);
@@ -291,9 +292,9 @@ FontDlg::FontDlg( QWidget *parent, const QString &ifnt )
     sep->setFrameShadow( QFrame::Raised );
     dlg_lay->addWidget( sep, 4, 0, 1, 2 );
 
-    QDialogButtonBox *but_box = new QDialogButtonBox(QDialogButtonBox::Ok|
-                                    		     QDialogButtonBox::Cancel,Qt::Horizontal,this);
-    QImage ico_t;    
+    QDialogButtonBox *but_box = new QDialogButtonBox( QDialogButtonBox::Ok|
+						      QDialogButtonBox::Cancel, Qt::Horizontal, this );
+    QImage ico_t;
     but_box->button(QDialogButtonBox::Ok)->setText(_("Ok"));
     if(!ico_t.load(TUIS::icoPath("button_ok").c_str())) ico_t.load(":/images/button_ok.png");
     but_box->button(QDialogButtonBox::Ok)->setIcon(QPixmap::fromImage(ico_t));
@@ -303,7 +304,7 @@ FontDlg::FontDlg( QWidget *parent, const QString &ifnt )
     but_box->button(QDialogButtonBox::Cancel)->setIcon(QPixmap::fromImage(ico_t));
     connect(but_box, SIGNAL(rejected()), this, SLOT(reject()));
     dlg_lay->addWidget( but_box, 5, 0, 1, 2 );
-    
+
     setFont(ifnt);
 }
 
@@ -311,12 +312,12 @@ QString	FontDlg::font( )
 {
     return QString("%1 %2 %3 %4 %5 %6").arg(fntSel->currentFont().family().replace(QRegExp(" "),"_")).
 					arg(spBox->value()).
-                			arg(chBold->checkState()?"1":"0").
+					arg(chBold->checkState()?"1":"0").
 					arg(chItalic->checkState()?"1":"0").
 					arg(chUnder->checkState()?"1":"0").
 					arg(chStrike->checkState()?"1":"0");
 }
-					
+
 void FontDlg::setFont( const QString &fnt )
 {
     char family[101]; strcpy(family,"Arial");
@@ -345,14 +346,11 @@ void FontDlg::cfgChange()
 //*********************************************
 //* Status bar user widget                    *
 //*********************************************
-UserStBar::UserStBar( const QString &iuser, QWidget *parent ) : QLabel(parent)
+UserStBar::UserStBar( const QString &iuser, const QString &ipass, const QString &iVCAstat, QWidget *parent ) : QLabel(parent)
 {
     setUser(iuser);
-}
-
-QString UserStBar::user()
-{
-    return user_txt;
+    setPass(ipass);
+    setVCAStation(iVCAstat);
 }
 
 void UserStBar::setUser( const QString &val )
@@ -369,17 +367,20 @@ bool UserStBar::event( QEvent *event )
 
 bool UserStBar::userSel()
 {
-    DlgUser d_usr(parentWidget());
+    DlgUser d_usr(user(),pass(),VCAStation(),parentWidget());
     int rez = d_usr.exec();
     if( rez == DlgUser::SelOK && d_usr.user() != user() )
     {
-        setUser( d_usr.user() );
-	emit userChanged();
+	QString old_user = user(),
+		old_pass = pass();
+	setUser( d_usr.user() );
+	setPass( d_usr.password() );
+	emit userChanged( old_user, old_pass );
 	return true;
     }
     else if( rez == DlgUser::SelErr )
 	mod->postMess(mod->nodePath().c_str(),_("Auth wrong!!!"),TVision::Warning,this);
-    
+
     return false;
 }
 
@@ -393,23 +394,28 @@ LineEdit::LineEdit( QWidget *parent, LType tp, bool prev_dis ) :
     QHBoxLayout *box = new QHBoxLayout(this);
     box->setMargin(0);
     box->setSpacing(0);
-		
+
     if( !prev_dis )
     {
-        bt_fld = new QPushButton(this);
-        bt_fld->setIcon(QIcon(":/images/ok.png"));
+	bt_fld = new QPushButton(this);
+	bt_fld->setIcon(QIcon(":/images/ok.png"));
 	bt_fld->setIconSize(QSize(12,12));
 	bt_fld->setSizePolicy( QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed) );
 	bt_fld->setMaximumWidth(15);
 	//bt_fld->setMinimumSize(15,15);
 	//bt_fld->setFlat( true );
-        bt_fld->setEnabled( false );
-        bt_fld->setVisible( false );
-        connect( bt_fld, SIGNAL( pressed() ), this, SLOT( applySlot() ) );
-        box->addWidget(bt_fld);
+	bt_fld->setEnabled( false );
+	bt_fld->setVisible( false );
+	connect( bt_fld, SIGNAL( pressed() ), this, SLOT( applySlot() ) );
+	box->addWidget(bt_fld);
     }
     setType(tp);
-}											
+}
+
+LineEdit::~LineEdit( )
+{
+    //printf("TEST 00: %d\n",d_ptr->inEventHandler);
+}
 
 bool LineEdit::isEdited( )
 {
@@ -420,18 +426,18 @@ bool LineEdit::isEdited( )
 void LineEdit::setType( LType tp )
 {
     if( tp == m_tp ) return;
-    
+
     //- Delete previous -
     if( tp >= 0 && ed_fld ) delete ed_fld;
-    
+
     //- Create new widget -
     switch( tp )
     {
-	case Text:	
+	case Text:
 	    ed_fld = new QLineEdit(this);
 	    connect( (QLineEdit*)ed_fld, SIGNAL( textEdited(const QString&) ), SLOT( changed() ) );
 	    break;
-	case Integer:	
+	case Integer:
 	    ed_fld = new QSpinBox(this);
 	    connect( (QSpinBox*)ed_fld, SIGNAL( valueChanged(int) ), SLOT( changed() ) );
 	    break;
@@ -439,15 +445,15 @@ void LineEdit::setType( LType tp )
 	    ed_fld = new QDoubleSpinBox(this);
 	    connect( (QDoubleSpinBox*)ed_fld, SIGNAL( valueChanged(double) ), SLOT( changed() ) );
 	    break;
-	case Time:	
+	case Time:
 	    ed_fld = new QTimeEdit(this);
 	    connect( (QTimeEdit*)ed_fld, SIGNAL( timeChanged(const QTime&) ), SLOT( changed() ) );
 	    break;
-	case Date:	
+	case Date:
 	    ed_fld = new QDateEdit(this);
 	    connect( (QDateEdit*)ed_fld, SIGNAL( dateChanged(const QDate&) ), SLOT( changed() ) );
 	    break;
-	case DateTime:	
+	case DateTime:
 	    ed_fld = new QDateTimeEdit(this);
 	    connect( (QDateTimeEdit*)ed_fld, SIGNAL( dateTimeChanged(const QDateTime&) ), SLOT( changed() ) );
 	    break;
@@ -455,25 +461,24 @@ void LineEdit::setType( LType tp )
 	    ed_fld = new QComboBox(this);
 	    ((QComboBox*)ed_fld)->setEditable(true);
 	    connect( (QComboBox*)ed_fld, SIGNAL( editTextChanged(const QString&) ), SLOT( changed() ) );
-	    break;	    
+	    break;
     }
     ((QBoxLayout*)layout())->insertWidget(0,ed_fld);
     setFocusProxy( ed_fld );
 
-    
     m_tp = tp;
 }
 
 void LineEdit::changed( )
 {
-    //- Enable apply 
+    //- Enable apply
     if( bt_fld && !bt_fld->isEnabled() )
     {
 	bt_fld->setEnabled(true);
-        bt_fld->setVisible(true);
-	//QWidget::setTabOrder( mod->getFocusedWdg(ed_fld), mod->getFocusedWdg(bt_fld) );	
+	bt_fld->setVisible(true);
+	//QWidget::setTabOrder( mod->getFocusedWdg(ed_fld), mod->getFocusedWdg(bt_fld) );
     }
-    
+
     emit valChanged(value());
 }
 
@@ -481,10 +486,10 @@ void LineEdit::setValue(const QString &txt)
 {
     switch(type())
     {
-	case Text: 
+	case Text:
 	    ((QLineEdit*)ed_fld)->setText(txt);
 	    ((QLineEdit*)ed_fld)->setCursorPosition(0);
-	    break;	    
+	    break;
 	case Integer:
 	    ((QSpinBox*)ed_fld)->setValue(txt.toInt());
 	    break;
@@ -493,7 +498,7 @@ void LineEdit::setValue(const QString &txt)
 	    break;
 	case Time:
 	    ((QTimeEdit*)ed_fld)->setTime(QTime().addSecs(txt.toInt()));
-	    break;	 
+	    break;
 	case Date: case DateTime:
 	    ((QDateTimeEdit*)ed_fld)->setDateTime(QDateTime::fromTime_t(txt.toInt()));
 	    break;
@@ -502,25 +507,25 @@ void LineEdit::setValue(const QString &txt)
 	    ((QComboBox*)ed_fld)->setEditText(txt);
 	    break;
     }
-    
+
     m_val = txt;
 
     if( bt_fld && bt_fld->isEnabled() )
     {
         bt_fld->setEnabled(false);
         bt_fld->setVisible(false);
-    }    
+    }
 }
 
 void LineEdit::setCfg(const QString &cfg)
 {
     switch(type())
     {
-	case Text: 	((QLineEdit*)ed_fld)->setInputMask(cfg);	break;
+	case Text:	((QLineEdit*)ed_fld)->setInputMask(cfg);	break;
 	case Integer:
 	{
-	    int	minv = 0, maxv = 100, sstep = 1;
-	    string pref, suff;
+	    int		minv = 0, maxv = 100, sstep = 1;
+	    string	pref, suff;
 	    if( !cfg.isEmpty() )
 	    {
 		minv  = atoi(TSYS::strSepParse(cfg.toAscii().data(),0,':').c_str());
@@ -535,7 +540,7 @@ void LineEdit::setCfg(const QString &cfg)
 	    ((QSpinBox*)ed_fld)->setSuffix(suff.c_str());
 	    break;
 	}
-	case Real:	
+	case Real:
 	{
 	    double minv = 0, maxv = 100, sstep = 1;
 	    string pref, suff;
@@ -563,7 +568,7 @@ void LineEdit::setCfg(const QString &cfg)
 	{
 	    QString ctext = ((QComboBox*)ed_fld)->currentText();
 	    ((QComboBox*)ed_fld)->clear();
-            ((QComboBox*)ed_fld)->addItems(cfg.split("\n"));
+	    ((QComboBox*)ed_fld)->addItems(cfg.split("\n"));
 	    if( !ctext.isEmpty() )
 	    {
 		if( ((QComboBox*)ed_fld)->findText(ctext) < 0 ) ((QComboBox*)ed_fld)->addItem(ctext);
@@ -574,20 +579,20 @@ void LineEdit::setCfg(const QString &cfg)
     }
     if( bt_fld && bt_fld->isEnabled() )
     {
-        bt_fld->setEnabled(false);
-        bt_fld->setVisible(false);
-    }        
+	bt_fld->setEnabled(false);
+	bt_fld->setVisible(false);
+    }
 }
-				    
+
 QString LineEdit::value()
 {
     switch(type())
     {
-	case Text: 	return ((QLineEdit*)ed_fld)->text();
+	case Text:	return ((QLineEdit*)ed_fld)->text();
 	case Integer:	return QString::number(((QSpinBox*)ed_fld)->value());
 	case Real:	return QString::number(((QDoubleSpinBox*)ed_fld)->value());
 	case Time:	return QString::number(QTime().secsTo(((QTimeEdit*)ed_fld)->time()));
-	case Date: case DateTime: 	
+	case Date: case DateTime:
 			return QString::number(((QDateTimeEdit*)ed_fld)->dateTime().toTime_t());
 	case Combo:	return ((QComboBox*)ed_fld)->currentText();
     }
@@ -600,10 +605,10 @@ void LineEdit::applySlot( )
     bt_fld->setVisible(false);
 
     m_val = value();
-	
+
     emit apply();
 }
-	    
+
 bool LineEdit::event( QEvent * e )
 {
     if(e->type() == QEvent::KeyRelease && bt_fld && bt_fld->isEnabled())
@@ -611,14 +616,14 @@ bool LineEdit::event( QEvent * e )
         QKeyEvent *keyEvent = (QKeyEvent *)e;
         if(keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return)
         {
-            bt_fld->animateClick( );
-            return true;
+	    bt_fld->animateClick( );
+	    return true;
         }
         else if(keyEvent->key() == Qt::Key_Escape )
         {
-            emit cancel();
+	    emit cancel();
 	    setValue(m_val);
-            return true;
+	    return true;
         }
     }
     return QWidget::event(e);
@@ -633,29 +638,29 @@ TextEdit::TextEdit( QWidget *parent, bool prev_dis ) :
     QVBoxLayout *box = new QVBoxLayout(this);
     box->setMargin(0);
     box->setSpacing(0);
-		    
+
     ed_fld = new QTextEdit(this);
     setFocusProxy( ed_fld );
     connect( ed_fld, SIGNAL( textChanged() ), this, SLOT( changed() ) );
     box->addWidget(ed_fld);
-				
+
     if( !prev_dis )
     {
-        but_box = new QDialogButtonBox(QDialogButtonBox::Apply|QDialogButtonBox::Cancel,Qt::Horizontal,this);
-        QImage ico_t;
-        but_box->button(QDialogButtonBox::Apply)->setText("");
-        if(!ico_t.load(TUIS::icoPath("button_ok").c_str())) ico_t.load(":/images/button_ok.png");
-        but_box->button(QDialogButtonBox::Apply)->setIcon(QPixmap::fromImage(ico_t));
+	but_box = new QDialogButtonBox(QDialogButtonBox::Apply|QDialogButtonBox::Cancel,Qt::Horizontal,this);
+	QImage ico_t;
+	but_box->button(QDialogButtonBox::Apply)->setText("");
+	if(!ico_t.load(TUIS::icoPath("button_ok").c_str())) ico_t.load(":/images/button_ok.png");
+	but_box->button(QDialogButtonBox::Apply)->setIcon(QPixmap::fromImage(ico_t));
 	but_box->button(QDialogButtonBox::Apply)->setIconSize(QSize(12,12));
-        connect(but_box->button(QDialogButtonBox::Apply), SIGNAL(pressed()), this, SLOT(applySlot()));
-        but_box->button(QDialogButtonBox::Cancel)->setText("");
+	connect(but_box->button(QDialogButtonBox::Apply), SIGNAL(pressed()), this, SLOT(applySlot()));
+	but_box->button(QDialogButtonBox::Cancel)->setText("");
 	if(!ico_t.load(TUIS::icoPath("button_cancel").c_str())) ico_t.load(":/images/button_cancel.png");
-        but_box->button(QDialogButtonBox::Cancel)->setIcon(QPixmap::fromImage(ico_t));
+	but_box->button(QDialogButtonBox::Cancel)->setIcon(QPixmap::fromImage(ico_t));
 	but_box->button(QDialogButtonBox::Cancel)->setIconSize(QSize(12,12));
-        connect(but_box->button(QDialogButtonBox::Cancel), SIGNAL(pressed()), this, SLOT(cancelSlot()));
-        but_box->setVisible(false);
+	connect(but_box->button(QDialogButtonBox::Cancel), SIGNAL(pressed()), this, SLOT(cancelSlot()));
+	but_box->setVisible(false);
 	but_box->setEnabled(false);
-        box->addWidget(but_box);
+	box->addWidget(but_box);
     }
 }
 
@@ -663,7 +668,7 @@ QString TextEdit::text()
 {
     return ed_fld->toPlainText();
 }
-    
+
 void TextEdit::setText(const QString &text)
 {
     isInit=true;
@@ -672,12 +677,12 @@ void TextEdit::setText(const QString &text)
     {
 	but_box->setVisible(false);
 	but_box->setEnabled(false);
-    }    
+    }
     isInit=false;
-    
+
     m_text = text;
 }
-			    
+
 void TextEdit::changed()
 {
     if( isInit ) return;
@@ -696,16 +701,16 @@ void TextEdit::applySlot( )
 	but_box->setVisible(false);
 	but_box->setEnabled(false);
     }
-    
+
     m_text = text();
-    
+
     emit apply();
 }
 
 void TextEdit::cancelSlot( )
 {
     setText(m_text);
-    
+
     emit cancel();
 }
 
@@ -713,8 +718,8 @@ void TextEdit::cancelSlot( )
 //* Shape widget view                    *
 //****************************************
 WdgView::WdgView( const string &iwid, int ilevel, QMainWindow *mainWind, QWidget *parent, Qt::WindowFlags f ) :
-    QWidget(parent,f), idWidget(iwid), shape(NULL), w_level(ilevel), main_win(mainWind), 
-    x_scale(1.0), y_scale(1.0), z_coord(0), all_attr_load(false)
+    QWidget(parent,f), idWidget(iwid), shape(NULL), w_level(ilevel), main_win(mainWind),
+    x_scale(1.0), y_scale(1.0), z_coord(0), all_attr_load(false), shpData(NULL)
 {
     //setAttribute(Qt::WA_OpaquePaintEvent,true);
 }
@@ -745,7 +750,7 @@ string WdgView::root( )
 void WdgView::moveF( const QPointF &pos )
 {
     w_pos = pos;
-    move( pos.toPoint() );
+    move( QPoint((int)TSYS::realRound(pos.x()),(int)TSYS::realRound(pos.y())) );
 }
 
 void WdgView::resizeF( const QSizeF &size )
@@ -753,7 +758,7 @@ void WdgView::resizeF( const QSizeF &size )
     w_size = size;
     w_size.setWidth(vmax(w_size.width(),3));
     w_size.setHeight(vmax(w_size.height(),3));
-    resize( w_size.toSize() );
+    resize( QSize((int)TSYS::realRound(w_size.width()), (int)TSYS::realRound(w_size.height())) );
 }
 
 WdgView *WdgView::newWdgItem( const string &iwid )
@@ -767,64 +772,88 @@ bool WdgView::attrSet( const string &attr, const string &val, int uiPrmPos )
     if( !attr.empty() )
     {
 	XMLNode req("set");
-	req.setAttr("path",id()+"/%2fserv%2f0");
+	req.setAttr("path",id()+"/%2fserv%2fattr");
 	req.childAdd("el")->setAttr("id",attr)->setText(val);
 	cntrIfCmd(req);
     }
-    switch(uiPrmPos)
+    bool up = false;
+    switch( uiPrmPos )
     {
-	case 0:	 return false;
-	case 7:	 
-	    if( wLevel( ) == 0 )break;
-	    moveF(QPointF(((WdgView*)parentWidget())->xScale(true)*atof(val.c_str()),posF().y()));
+	case -1:
+	    up = true;
 	    break;
-	case 8:	 
-	    if( wLevel( ) == 0 )break;
-	    moveF(QPointF(posF().x(),((WdgView*)parentWidget())->yScale(true)*atof(val.c_str())));
+	case 0:	return false;
+	case 7:
+	    if( wLevel( ) == 0 )	break;
+	    w_pos = QPointF(((WdgView*)parentWidget())->xScale(true)*atof(val.c_str()),posF().y());
+	    up = true;
 	    break;
-	case 9:	resizeF(QSizeF(xScale(true)*atof(val.c_str()),sizeF().height()));	break;
-	case 10:resizeF(QSizeF(sizeF().width(),yScale(true)*atof(val.c_str())));	break;
-	case 11:if(wLevel( )>0) z_coord = atoi(val.c_str());				break;
+	case 8:
+	    if( wLevel( ) == 0 )	break;
+	    w_pos = QPointF(posF().x(),((WdgView*)parentWidget())->yScale(true)*atof(val.c_str()));
+	    up = true;
+	    break;
+	case 9:
+	    w_size = QSizeF(xScale(true)*atof(val.c_str()),sizeF().height());
+	    up = true;
+	    break;
+	case 10:
+	    w_size = QSizeF(sizeF().width(),yScale(true)*atof(val.c_str()));
+	    up = true;
+	    break;
+	case 11: if(wLevel( )>0) z_coord = atoi(val.c_str());	break;
 	case 13:
-	    resizeF(QSizeF((atof(val.c_str())/x_scale)*sizeF().width(),sizeF().height()));
+	    w_size = QSizeF((atof(val.c_str())/x_scale)*sizeF().width(),sizeF().height());
 	    x_scale = atof(val.c_str());
+	    up = true;
 	    break;
-	case 14: 
-	    resizeF(QSizeF(sizeF().width(),(atof(val.c_str())/y_scale)*sizeF().height()));
+	case 14:
+	    w_size = QSizeF(sizeF().width(),(atof(val.c_str())/y_scale)*sizeF().height());
 	    y_scale = atof(val.c_str());
+	    up = true;
 	    break;
     }
-    return shape ? shape->attrSet(this,uiPrmPos,val) : true;
+    if( up && !allAttrLoad( ) )
+    {
+	if( wLevel( ) > 0 )	moveF(posF());
+	resizeF(sizeF());
+    }
+
+    if( shape )	return shape->attrSet(this,uiPrmPos,val);
+
+    return true;
 }
 
 string WdgView::resGet( const string &res )
 {
+    if( res.empty() )	return "";
+
     XMLNode req("get");
     req.setAttr("path",id()+"/%2fwdg%2fres")->setAttr("id",res);
     if( !cntrIfCmd(req) )	return TSYS::strDecode(req.text(),TSYS::base64);
-    
+
     return "";
 }
 
 void WdgView::load( const string& item, bool load, bool init )
-{   
+{
     //printf("TEST 00: Load: %s (%d:%d)\n",id().c_str(),load,init);
-    
-    unsigned long long t_cnt;
-    if( wLevel() == 0 ) t_cnt = SYS->shrtCnt();
-    
-    //- Load from data model - 
+
+    //unsigned long long t_cnt;
+    //if( wLevel() == 0 ) t_cnt = SYS->shrtCnt();
+
+    //- Load from data model -
     if( load )
     {
-        childsUpdate( false );    
-	setAllAttrLoad( true );    
+	childsUpdate( false );
+	setAllAttrLoad( true );
 	if( item.empty() || item == id() )
 	{
 	    shapeUpdate( );
 	
-	    //-- Request to widget for last attributes --
-	    XMLNode req("get");	
-	    req.setAttr("path",id()+"/%2fserv%2f0");
+	    //-- Request the widget for last attributes --
+	    XMLNode req("get");
+	    req.setAttr("path",id()+"/%2fserv%2fattr");
 	    if( !cntrIfCmd(req) )
 		for( int i_el = 0; i_el < req.childSize(); i_el++ )
 		    attrSet("",req.childGet(i_el)->text(),atoi(req.childGet(i_el)->attr("pos").c_str()));
@@ -832,15 +861,15 @@ void WdgView::load( const string& item, bool load, bool init )
 	setAllAttrLoad( false );
     }
 
-    //- Init loaded data -     
+    //- Init loaded data -
     if( init && (item.empty() || item == id()) && wLevel()>0 )	attrSet("","load",-1);
 
     //- Going to children load and/or init -
     for( int i_c = 0; i_c < children().size(); i_c++ )
     {
 	WdgView *wdg = qobject_cast<WdgView*>(children().at(i_c));
-        if( wdg && (item.empty() || item == id() || wdg->id() == item.substr(0,wdg->id().size())) )
-    	    wdg->load((item==id())?"":item,load,(wLevel()>0)?init:false);
+	if( wdg && (item.empty() || item == id() || wdg->id() == item.substr(0,wdg->id().size())) )
+	    wdg->load((item==id())?"":item,load,(wLevel()>0)?init:false);
     }
 
     //- Children widgets order update -
@@ -849,50 +878,52 @@ void WdgView::load( const string& item, bool load, bool init )
     //- Post load init for root widget -
     if( wLevel() == 0 )
     {
-	//printf("TEST 01: Load '%s' time %fms\n",item.c_str(),1.0e3*((double)(SYS->shrtCnt()-t_cnt))/((double)SYS->sysClk()));
+	//printf("TEST 01: Load '%s' time %fms\n",id().c_str(),1.0e3*((double)(SYS->shrtCnt()-t_cnt))/((double)SYS->sysClk()));
 	//t_cnt = SYS->shrtCnt();
-    
+
 	attrSet("","load",-1);
 	for( int i_c = 0; i_c < children().size(); i_c++ )
 	{
 	    WdgView *wdg = qobject_cast<WdgView*>(children().at(i_c));
-    	    if( wdg && (item.empty() || item == id() || wdg->id() == item.substr(0,wdg->id().size())) )
-    		wdg->load((item==id())?"":item,false,true);
+	    if( wdg && (item.empty() || item == id() || wdg->id() == item.substr(0,wdg->id().size())) )
+		wdg->load((item==id())?"":item,false,true);
 	}
 	update();
+        //repaint();
 
-	//printf("TEST 02: Init '%s' time %fms\n",item.c_str(),1.0e3*((double)(SYS->shrtCnt()-t_cnt))/((double)SYS->sysClk()));
+	//printf("TEST 02: Init '%s' time %fms\n",id().c_str(),1.0e3*((double)(SYS->shrtCnt()-t_cnt))/((double)SYS->sysClk()));
     }
 }
 
 void WdgView::childsUpdate( bool newLoad )
 {
     XMLNode req("get");
-    
+
     string b_nm = id();
     req.setAttr("path",id()+"/%2fwdg%2fcfg%2fpath")->setAttr("resLink","1");
     if( !cntrIfCmd(req) ) b_nm = req.text();
-	    
+
     vector<string> lst;
     req.clear()->setAttr("path",b_nm+"/%2finclwdg%2fwdg");
+
     if( !cntrIfCmd(req) )
 	for( int i_el = 0; i_el < req.childSize(); i_el++ )
 	    lst.push_back(req.childGet(i_el)->attr("id"));
-    
+
     //- Delete child widgets -
     for( int i_c = 0; i_c < children().size(); i_c++ )
     {
 	if( !qobject_cast<WdgView*>(children().at(i_c)) ) continue;
 	int i_l;
-	for( i_l = 0; i_l < lst.size(); i_l++ )		
+	for( i_l = 0; i_l < lst.size(); i_l++ )
 	    if( qobject_cast<WdgView*>(children().at(i_c))->id() == (b_nm+"/wdg_"+lst[i_l]) )
 		break;
-	if( i_l >= lst.size() ) delete children().at(i_c);
+	if( i_l >= lst.size() ) delete children().at(i_c--);
     }
 
     //- Create new child widget -
     for( int i_l = 0; i_l < lst.size(); i_l++ )
-    {	
+    {
 	int i_c;
 	for( i_c = 0; i_c < children().size(); i_c++ )
 	    if( qobject_cast<WdgView*>(children().at(i_c)) && 
@@ -901,7 +932,7 @@ void WdgView::childsUpdate( bool newLoad )
 	if( i_c >= children().size() )
 	{
 	    WdgView *nwdg = newWdgItem(b_nm+"/wdg_"+lst[i_l]);
-	    if( nwdg ) 
+	    if( nwdg )
 	    {
 		nwdg->show();
 		if( newLoad ) nwdg->load("");
@@ -958,4 +989,3 @@ bool WdgView::eventFilter( QObject *object, QEvent *event )
     if( shape )	return shape->eventFilter(this,object,event);
     return false;
 }
-
