@@ -1,7 +1,7 @@
 
 //OpenSCADA system module UI.Vision file: vis_widgs.h
 /***************************************************************************
- *   Copyright (C) 2007 by Roman Savochenko                                *
+ *   Copyright (C) 2007-2008 by Roman Savochenko                           *
  *   rom_as@diyaorg.dp.ua                                                  *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -53,7 +53,7 @@ namespace VISION
     class InputDlg : public QDialog
     {
 	public:
-	    InputDlg( QWidget *parent, const QIcon &icon, const QString &mess, 
+	    InputDlg( QWidget *parent, const QIcon &icon, const QString &mess,
 		    const QString &ndlg = "Vision dialog", bool with_id = false, bool with_nm = true );
 
 	    QString id( );
@@ -65,8 +65,8 @@ namespace VISION
 	    void setMess( const QString &val );
 
 	private:
-	    QLabel 	*inp_lab;
-	    QLineEdit 	*m_id, *m_name;
+	    QLabel	*inp_lab;
+	    QLineEdit	*m_id, *m_name;
     };
 
     //*********************************************
@@ -75,13 +75,13 @@ namespace VISION
     class DlgUser : public QDialog
     {
 	Q_OBJECT
-    
+
 	public:
 	    //Data
 	    enum Results { SelCancel, SelOK, SelErr };
 
 	    //Methods
-	    DlgUser( QWidget *parent = 0 );
+	    DlgUser( const QString &iuser, const QString &ipass, const QString &iVCAstat, QWidget *parent = 0 );
 
 	    QString user();
 	    QString password();
@@ -90,8 +90,9 @@ namespace VISION
 	    void finish( int result );
 
 	private:
-	    QComboBox 	*users;
-	    QLineEdit 	*passwd;
+	    QComboBox	*users;
+	    QLineEdit	*passwd;
+	    QString	VCAstat;
     };
 
     //*********************************************
@@ -100,7 +101,7 @@ namespace VISION
     class FontDlg : public QDialog
     {
 	Q_OBJECT
-    
+
 	public:
 	    FontDlg( QWidget *parent, const QString &ifnt );
 
@@ -112,82 +113,88 @@ namespace VISION
 	    void cfgChange();
 
 	private:
-	    QFontComboBox 	*fntSel;
-	    QSpinBox 		*spBox;
-	    QCheckBox 		*chBold,
+	    QFontComboBox	*fntSel;
+	    QSpinBox		*spBox;
+	    QCheckBox		*chBold,
 				*chItalic,
 				*chStrike,
 				*chUnder;
-	    QLineEdit 		*sampleText;
+	    QLineEdit		*sampleText;
     };
-    
+
     //*********************************************
     //* Status bar user widget                    *
     //*********************************************
     class UserStBar : public QLabel
     {
 	Q_OBJECT
-    
-	public:
-	    UserStBar( const QString &iuser, QWidget * parent = 0 );
 
-	    QString user();
+	public:
+	    UserStBar( const QString &iuser, const QString &ipass, const QString &iVCAstat, QWidget *parent = 0 );
+
+	    QString user( )		{ return user_txt; }
+	    QString pass( )		{ return user_pass; }
+	    QString VCAStation( )	{ return VCA_stat; }
+
 	    void setUser( const QString &val );
+	    void setPass( const QString &val )		{ user_pass = val; }
+	    void setVCAStation( const QString &val )	{ VCA_stat = val.isEmpty() ? "." : val; }
 
 	    bool userSel( );
 
 	signals:
-            void userChanged();
+	    void userChanged( const QString &oldUser, const QString &oldPass );
 
 	protected:
 	    bool event( QEvent *event );
 
 	private:
-	    QString	user_txt;
-    };    
-    
+	    QString	user_txt, user_pass, VCA_stat;
+    };
+
     //*********************************************************************************************
     //* Universal edit line widget. Contain support of: QLineEdit, QSpinBox, QDoubleSpinBox,      *
     //* QTimeEdit, QDateEdit and QDateTimeEdit.                                                   *
     //*********************************************************************************************
     class LineEdit : public QWidget
     {
-        Q_OBJECT
-		
-        public:
+	Q_OBJECT
+
+	public:
 	    //- Data -
 	    enum LType { Text, Integer, Real, Time, Date, DateTime, Combo };
-	    
+
 	    //- Methods -
-            LineEdit( QWidget *parent, LType tp = Text, bool prev_dis = false );
-	    
-	    LType type( ) 	{ return m_tp; }
-            QString value( );
-	    bool  isEdited( );
+	    LineEdit( QWidget *parent, LType tp = Text, bool prev_dis = false );
+	    ~LineEdit( );
+
+	    LType type( )	{ return m_tp; }
+	    QString value( );
+	    bool isEdited( );
 
 	    void setType( LType tp );
-            void setValue( const QString& );
+	    void setValue( const QString& );
 	    void setCfg( const QString& );
-	    
+
 	    QWidget *workWdg( )	{ return ed_fld; }
-        
+
 	signals:
-            void apply( );
-            void cancel( );
+	    void apply( );
+	    void cancel( );
 	    void valChanged( const QString& );
-	
+
 	protected:
-            bool event( QEvent * e );
-        
+	    bool event( QEvent * e );
+
 	private slots:
-            void changed( );
-            void applySlot( );
-        
+	    void changed( );
+	    void applySlot( );
+
 	private:
-	    LType       m_tp;
-	    QString     m_val;
-            QWidget	*ed_fld;
-    	    QPushButton *bt_fld;
+	    LType	m_tp;
+	    QString	m_val;
+	    QWidget	*ed_fld;
+	    QPushButton	*bt_fld;
     };
 
     //*********************************************
@@ -195,98 +202,98 @@ namespace VISION
     //*********************************************
     class TextEdit : public QWidget
     {
-        Q_OBJECT
-	    
-        public:
-            TextEdit( QWidget *parent, bool prev_dis = false );
-						
-            QString text();
-            void setText(const QString &);
-						    
-            QTextEdit *workWdg() { return ed_fld; }
-	    
-        signals:
-            void apply( );
-            void cancel( );
-            void textChanged(const QString&);
-        
+	Q_OBJECT
+
+	public:
+	    TextEdit( QWidget *parent, bool prev_dis = false );
+
+	    QString text( );
+	    void setText( const QString & );
+
+	    QTextEdit *workWdg( )	{ return ed_fld; }
+
+	signals:
+	    void apply( );
+	    void cancel( );
+	    void textChanged( const QString& );
+
 	private slots:
-            void changed();
-            void applySlot( );
-            void cancelSlot( );
-        
+	    void changed( );
+	    void applySlot( );
+	    void cancelSlot( );
+
 	private:
-    	    bool isInit;
-	    QString     m_text;
-            QTextEdit   *ed_fld;
-            QDialogButtonBox *but_box;
+	    bool		isInit;
+	    QString		m_text;
+	    QTextEdit		*ed_fld;
+	    QDialogButtonBox	*but_box;
     };
 
     //****************************************
     //* Shape widget view                    *
     //****************************************
     class WdgShape;
-    
+
     class WdgView: public QWidget
     {
 	Q_OBJECT
-    
-        public:
-    	    //- Public methods -
+
+	public:
+	    //- Public methods -
 	    WdgView( const string &iwid, int ilevel, QMainWindow *mainWind, QWidget* parent = 0, Qt::WindowFlags f = 0 );
 	    ~WdgView( );
 
-	    string id( )    		{ return idWidget; }
-	    int    wLevel( )		{ return w_level; }
-	    string root( );
-	    QMainWindow *mainWin( )	{ return main_win; }
-	    QPointF posF( )		{ return w_pos; }
-	    QSizeF  sizeF( )		{ return w_size; }
-	    QRectF  geometryF( )	{ return QRectF(w_pos,w_size); }
-	    virtual float  xScale( bool full = false );
-	    virtual float  yScale( bool full = false );
-	    int	   z( )			{ return z_coord;  }
+	    string	id( )		{ return idWidget; }
+	    int		wLevel( )	{ return w_level; }
+	    string	root( );
+	    QMainWindow	*mainWin( )	{ return main_win; }
+	    QPointF	posF( )		{ return w_pos; }
+	    QSizeF	sizeF( )	{ return w_size; }
+	    QRectF	geometryF( )	{ return QRectF(w_pos,w_size); }
+	    virtual float xScale( bool full = false );
+	    virtual float yScale( bool full = false );
+	    int		z( )		{ return z_coord;  }
 	    virtual string user( )	{ return ""; }
-	    QMap<QString, QVariant> &dc(){ return cache_data; }
-	    bool allAttrLoad( )		{ return all_attr_load; }
+	    bool	allAttrLoad( )	{ return all_attr_load; }
+
+	    void	moveF( const QPointF &pos );
+	    void	resizeF( const QSizeF &size );
+	    void	setAllAttrLoad( bool vl )	{ all_attr_load = vl; }
 
-	    void   moveF( const QPointF &pos );
-	    void   resizeF( const QSizeF &size );
-	    void   setAllAttrLoad( bool vl )	{ all_attr_load = vl; }
-	    
 	    virtual bool attrSet( const string &attr, const string &val, int uiPrmPos = 0 );
-	    string resGet( const string &res );
+	    virtual string resGet( const string &res );
 
 	    virtual int cntrIfCmd( XMLNode &node, bool glob = false )	{ };
 
+            //Attributes
+	    void	*shpData;	//Shape data link
+
 	public slots:
 	    virtual WdgView *newWdgItem( const string &iwid );
 	    virtual void load( const string& item, bool load = true, bool init = true );
-	    
+
 	protected:
 	    //- Protected methods -
-	    void childsUpdate( bool newLoad = true );
+	    virtual void childsUpdate( bool newLoad = true );
 	    void shapeUpdate( );
 	    virtual void orderUpdate( );
-	    
+
 	    bool event( QEvent * event );
-	    bool eventFilter( QObject *object, QEvent *event );	    
-	    
+	    bool eventFilter( QObject *object, QEvent *event );
+
 	    //- Protected attributes -
 	    bool		all_attr_load;	//All attributes load
-	    int 		w_level;	//Widget level
+	    int			w_level;	//Widget level
 	    QPointF		w_pos;		//Widget position into real;
 	    QSizeF		w_size;		//Widget size into real;
 	    float		x_scale, 	//Widget x scale
 				y_scale;	//	 y scale
 	    int			z_coord;	//Z coordinate
-	    string		idWidget, 	//Full widget identifier
+	    string		idWidget,	//Full widget identifier
 				m_root;		//Root widget identifier
 	    WdgShape		*shape;		//Link to root widget shape
-	    QMap<QString, QVariant> cache_data;	//Internal data cache
 	    QMainWindow		*main_win;	//Main window
     };
 }
 
 #endif //VIS_WIDGS_H
-
