@@ -1,7 +1,7 @@
 
 //OpenSCADA system file: tcntrnode.h
 /***************************************************************************
- *   Copyright (C) 2003-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,7 +18,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
- 
+
 #ifndef TCNTRNODE_H
 #define TCNTRNODE_H
 
@@ -50,13 +50,17 @@ namespace __gnu_cxx
 template <> class hash<string>
 {
     public:
-        size_t operator()(const string& h) const
-        {
-            return hash<const char*>()(h.c_str());
-        }
+	size_t operator()(const string& h) const
+	{
+	    return hash<const char*>()(h.c_str());
+	}
 };
 }
 typedef hash_map<string, TCntrNode*, __gnu_cxx::hash<string> > TMap;
+
+//#include <unordered_map>
+//using std::unordered_map;
+//typedef unordered_map<string, TCntrNode* > TMap;
 #else
 #include <map>
 using std::map;
@@ -73,7 +77,7 @@ typedef map<string, TCntrNode* > TMap;
 //* TCntrNode - Controll node					*
 //***************************************************************
 class TCntrNode
-{        
+{
     //* Controll scenaries language section *
     public:
 	//Methods
@@ -83,52 +87,53 @@ class TCntrNode
 	virtual TCntrNode &operator=( TCntrNode &node );
 
 	void cntrCmd( XMLNode *opt, int lev = 0, const string &path = "", int off = 0 );
-	
+
 	//- Static functions -
 	static XMLNode *ctrId( XMLNode *inf, const string &n_id, bool noex = false );      //get node for it full identifier
-	
+
 	//-- Controll Field --
-	static XMLNode *ctrMkNode( const char *n_nd, XMLNode *nd, int pos, const char *req, const string &dscr, 
+	static XMLNode *ctrMkNode( const char *n_nd, XMLNode *nd, int pos, const char *req, const string &dscr,
 	    int perm=0777, const char *user="root", const char *grp="root", int n_attr=0, ... );
-	static bool ctrChkNode( XMLNode *nd, const char *cmd="get", int perm=0444, const char *user="root", 
-	    const char *grp="root", char mode=04, const char *warn = NULL ); 
-	
+	static bool ctrChkNode( XMLNode *nd, const char *cmd="get", int perm=0444, const char *user="root",
+	    const char *grp="root", char mode=04, const char *warn = NULL );
+
     protected:
 	//Methods
 	virtual void cntrCmdProc( XMLNode *req );
-	
+
     //* Resource section *
     public:
 	//Data
 	enum Flag
 	{
 	    //- Modes -
-	    MkDisable  = 0x00,	//Node make disable
-	    Disable    = 0x01,	//Node disabled
-	    MkEnable   = 0x02,	//Node make enable
-	    Enable     = 0x03,	//Node enabled
+	    MkDisable	= 0x00,		//Node make disable
+	    Disable	= 0x01,		//Node disabled
+	    MkEnable	= 0x02,		//Node make enable
+	    Enable	= 0x03,		//Node enabled
 	    //- Flags -
-	    SelfModify = 0x04,	//Self modify
+	    SelfModify	= 0x04,		//Self modify
 	};
 	enum EnFlag
-	{ 
-	    NodeConnect = 0x01,	//Connect node to control tree
-	    NodeRestore = 0x02	//Restore node enabling after broken disabling. 
+	{
+	    NodeConnect	= 0x01,		//Connect node to control tree
+	    NodeRestore	= 0x02,		//Restore node enabling after broken disabling.
+	    NodeShiftDel= 0x04
 	};
 	enum ModifFlag	{ Self = 0x01, Child = 0x02, All = 0x03 };
-	
+
 	//Methods
-       	virtual string nodeName( )	{ return "NO Named!"; }
+	virtual string nodeName( )	{ return "NO Named!"; }
 	string nodePath( char sep = 0, bool from_root = false );
-	
+
 	void nodeList( vector<string> &list, const string& gid = "" );				//Full node list
 	AutoHD<TCntrNode> nodeAt( const string &path, int lev = 0, char sep = 0, int off = 0 );	//Get node for full path
-	void nodeDel( const string &path, char sep = 0, int flag = 0 );				//Delete node at full path
+	void nodeDel( const string &path, char sep = 0, int flag = 0, bool shDel = false );	//Delete node at full path
 	static void nodeCopy( const string &src, const string &dst, const string &user = "root" );
-	
+
 	TCntrNode *nodePrev( bool noex = false );
 	char	 nodeFlg( )		{ return m_flg; }
-        char 	 nodeMode( )		{ return m_flg&0x3; }
+	char	 nodeMode( )		{ return m_flg&0x3; }
 	unsigned nodeUse( );
 
 	//- Modify process methods -
@@ -136,47 +141,48 @@ class TCntrNode
 	void modif( )	{ m_flg |= SelfModify; }		//Set local modify
 	void modifG( );						//Set group modify
 	void modifClr( ){ m_flg &= ~(SelfModify); }		//Clear modify
-	void load( );						//Load node, if modified 
+	void modifGClr( );					//Modify group clear
+	void load( );						//Load node, if modified
 	void save( );						//Save node, if modified
 
 	void AHDConnect( );
 	void AHDDisConnect( );
-    
+
     protected:
 	//Data
 	struct GrpEl
 	{
-	    string 	id;
+	    string	id;
 	    bool	ordered;
-	    TMap 	elem;
+	    TMap	elem;
 	};
 
 	//Methods
 	//- Commands -
 	void nodeEn( int flag = 0 );
 	void nodeDis( long tm = 0, int flag = 0 );
-	
+
 	void nodeDelAll( );	//For hard link objects
-	
+
 	void setNodePrev( TCntrNode *node )	{ prev.node = node; }
 	void setNodeMode( char mode )		{ m_flg = (m_flg&(~0x03))|(mode&0x03); }
 
 	//- Childs -
-        AutoHD<TCntrNode> chldAt( char igr, const string &name, const string &user = "" );
+	AutoHD<TCntrNode> chldAt( char igr, const string &name, const string &user = "" );
 	void chldList( char igr, vector<string> &list );
 	bool chldPresent( char igr, const string &name );
 	void chldAdd( char igr, TCntrNode *node, int pos = -1 );
-	void chldDel( char igr, const string &name, long tm = -1, int flag = 0 );
-	
+	void chldDel( char igr, const string &name, long tm = -1, int flag = 0, bool shDel = false );
+
 	//- Conteiners -
-        char     grpSize( )	{ return chGrp.size(); }
-	char 	 grpId( const string &sid );
+	char	grpSize( )	{ return chGrp.size(); }
+	char	 grpId( const string &sid );
 	GrpEl	&grpAt( char id );
-        unsigned grpAdd( const string &id, bool ordered = false );
+	unsigned grpAdd( const string &id, bool ordered = false );
 
 	virtual void preEnable( int flag )	{ }
 	virtual void postEnable( int flag )	{ }
-	
+
 	virtual void preDisable( int flag )	{ }
 	virtual void postDisable( int flag )	{ }
 
@@ -185,19 +191,19 @@ class TCntrNode
 
     private:
 	//Data
- 	struct
+	struct
 	{
 	    TCntrNode	*node;
 	    char	grp;
 	} prev;
-    
+
 	//Attributes
-	//- Childs -	
-	Res 	hd_res,			//Resource HD
+	//- Childs -
+	Res	hd_res,			//Resource HD
 		conn_res;		//Connect resource
 
 	vector<GrpEl>	chGrp;		//Child groups
-	
+
 	//- Curent node -
 	unsigned char		m_use;	//Use counter
 	unsigned short int	m_oi;	//Order index
@@ -206,4 +212,3 @@ class TCntrNode
 };
 
 #endif //TCNTRNODE_H
-
