@@ -1,7 +1,7 @@
 
 //OpenSCADA system file: telem.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -35,14 +35,14 @@ TElem::TElem( const string &name ) : m_name(name)
 TElem::~TElem()
 {
     while(cont.size())	cont[0]->detElem(this);
-    while(elem.size()) 	fldDel(0);
+    while(elem.size())	fldDel(0);
 }
 
 int TElem::fldAdd( TFld *fld, int id )
 {
     //- Find dublicates -
     for( int i_f = 0; i_f < elem.size(); i_f++)
-	if(elem[i_f]->name() == fld->name()) 
+	if(elem[i_f]->name() == fld->name())
 	{
 	    delete fld;
 	    return i_f;
@@ -50,7 +50,7 @@ int TElem::fldAdd( TFld *fld, int id )
     if( id > elem.size() || id < 0 ) id = elem.size();
     elem.insert(elem.begin()+id,fld);
     //- Add value and set them default -
-    for(unsigned cfg_i=0; cfg_i < cont.size(); cfg_i++) 
+    for(unsigned cfg_i=0; cfg_i < cont.size(); cfg_i++)
 	cont[cfg_i]->addFld(this,id);
 
     return id;
@@ -59,7 +59,7 @@ int TElem::fldAdd( TFld *fld, int id )
 void TElem::fldDel( unsigned int id )
 {
     if( id >= elem.size() ) throw TError("Elem",_("Id error!"));
-    for(unsigned cfg_i=0; cfg_i < cont.size(); cfg_i++) 
+    for(unsigned cfg_i=0; cfg_i < cont.size(); cfg_i++)
 	cont[cfg_i]->delFld(this,id);
     delete elem[id];
     elem.erase(elem.begin()+id);
@@ -71,7 +71,7 @@ void TElem::valAtt( TValElem *cnt )
 	if(cont[i] == cnt) throw TError("Elem",_("The element container already attached!"));
     cont.push_back(cnt);
 }
- 
+
 void TElem::valDet( TValElem *cnt )
 {
     for(unsigned i=0; i < cont.size() ;i++)
@@ -88,14 +88,14 @@ unsigned TElem::fldId( const string &name )
 bool TElem::fldPresent( const string &name )
 {
     for(unsigned i=0; i < elem.size(); i++)
-        if(elem[i]->name() == name) return true;
-    return false;	
+	if(elem[i]->name() == name) return true;
+    return false;
 }
 
 void TElem::fldList( vector<string> &list )
 {
     list.clear();
-    for(unsigned i = 0; i < elem.size(); i++) 
+    for(unsigned i = 0; i < elem.size(); i++)
 	list.push_back(elem[i]->name());
 }
 
@@ -116,52 +116,55 @@ TFld::TFld( ) : m_type(TFld::Integer), m_flg(0)
 
 TFld::TFld( TFld &ifld ) : m_type(TFld::Integer), m_len(0), m_dec(0), m_flg(0)
 {
-    int st_pos, cur_pos;    
-    m_sel   = NULL;
-    m_val.v_s = NULL;
-    
-    m_name  = ifld.name();
-    m_descr = ifld.descr();
-    m_type  = ifld.type();
-    m_flg   = ifld.flg();
-    m_def   = ifld.def();
-    m_res   = ifld.reserve();
-    m_len   = ifld.len();
-    m_dec   = ifld.dec();
+    int st_pos, cur_pos;
+    m_sel	= NULL;
+    m_val.v_s	= NULL;
+
+    m_name	= ifld.name();
+    m_descr	= ifld.descr();
+    m_type	= ifld.type();
+    m_flg	= ifld.flg();
+    m_def	= ifld.def();
+    m_res	= ifld.reserve();
+    m_len	= ifld.len();
+    m_dec	= ifld.dec();
 
     setValues(ifld.values());
     setSelNames(ifld.selNames());
 }
 
 TFld::TFld( const char *name, const char *descr, TFld::Type itype, unsigned iflg,
-            const char *valLen, const char *valDef, const char *val_s, const char *n_Sel, const char *ires ) : 
-    m_type(TFld::Integer), m_len(0), m_dec(0), m_flg(0)
+            const char *valLen, const char *valDef, const char *val_s, const char *n_Sel, const char *ires ) :
+    m_type(TFld::Integer), m_flg(0)
 {
-    int st_pos, cur_pos;    
+    int st_pos, cur_pos;
     m_sel   = NULL;
     m_val.v_s = NULL;
-    
+
     m_name  = name;
-    m_descr = descr;    
-    m_type  = itype; 
+    m_descr = descr;
+    m_type  = itype;
     m_flg   = iflg;
     m_def   = valDef;
     m_res   = ires;
-    
-    sscanf(valLen,"%d.%d",&m_len,&m_dec);
+
+    int ilen = 0, idec = 0;
+    sscanf(valLen,"%d.%d",&ilen,&idec);
+    m_len = ilen;
+    m_dec = idec;
+
     setValues(val_s);
     setSelNames(n_Sel);
 }
-	    
 
 TFld::~TFld( )
 {
-    if( m_sel )	delete m_sel;    
+    if( m_sel )	delete m_sel;
     if( m_val.v_s != NULL )
 	switch(type())
 	{
-	    case TFld::String:	delete m_val.v_s;	break;			
-	    case TFld::Integer:	delete m_val.v_i; 	break;
+	    case TFld::String:	delete m_val.v_s;	break;
+	    case TFld::Integer:	delete m_val.v_i;	break;
 	    case TFld::Real:	delete m_val.v_r;	break;
 	    case TFld::Boolean:	delete m_val.v_b;	break;
 	}
@@ -182,21 +185,21 @@ string TFld::values()
     string rez;
     switch(type())
     {
-        case TFld::String:  
+	case TFld::String:
 	    for(int i_el = 0; i_el < m_val.v_s->size(); i_el++)
-		rez=rez+(*m_val.v_s)[i_el]+";";    
+		rez=rez+(*m_val.v_s)[i_el]+";";
 	    break;
-        case TFld::Integer: 
+	case TFld::Integer:
 	    for(int i_el = 0; i_el < m_val.v_i->size(); i_el++)
-		rez=rez+TSYS::int2str((*m_val.v_i)[i_el])+";";    
-	    break;	
-        case TFld::Real:    
+		rez=rez+TSYS::int2str((*m_val.v_i)[i_el])+";";
+	    break;
+	case TFld::Real:
 	    for(int i_el = 0; i_el < m_val.v_r->size(); i_el++)
 		rez=rez+TSYS::real2str((*m_val.v_r)[i_el],6)+";";
-	    break;	
-        case TFld::Boolean: 
+	    break;
+	case TFld::Boolean:
 	    for(int i_el = 0; i_el < m_val.v_b->size(); i_el++)
-		rez=rez+TSYS::int2str((*m_val.v_b)[i_el])+";";    
+		rez=rez+TSYS::int2str((*m_val.v_b)[i_el])+";";
 	    break;
     }
     return rez.size()?rez.substr(0,rez.size()-1):"";
@@ -205,11 +208,11 @@ string TFld::values()
 string TFld::selNames()
 {
     if( m_sel == NULL )	return "";
-    
+
     string rez;
     for(int i_el = 0; i_el < m_sel->size(); i_el++)
-        rez=rez+(*m_sel)[i_el]+";";
-    
+	rez=rez+(*m_sel)[i_el]+";";
+
     return rez.size()?rez.substr(0,rez.size()-1):"";
 }
 
@@ -220,15 +223,15 @@ void TFld::setValues( const string &vls )
     {
 	//- Count alements amount -
 	int i_lvl = 0, i_off = 0;
-        while( TSYS::strSepParse(vls,0,';',&i_off).size() ) i_lvl++;
-	
+	while( TSYS::strSepParse(vls,0,';',&i_off).size() ) i_lvl++;
+
 	switch(type())
 	{
-	    case TFld::String:	
+	    case TFld::String:
 		if(!m_val.v_s)	m_val.v_s = new vector<string>; 
 		m_val.v_s->resize(i_lvl,"");
-		break;			
-	    case TFld::Integer:	
+		break;
+	    case TFld::Integer:
 		if(!m_val.v_i)	m_val.v_i = new vector<int>;
 		m_val.v_i->resize(i_lvl,0);
 		break;
@@ -236,16 +239,16 @@ void TFld::setValues( const string &vls )
 		if(!m_val.v_r)	m_val.v_r = new vector<double>;
 		m_val.v_r->resize(i_lvl,0);
 		break;
-	    case TFld::Boolean:	
+	    case TFld::Boolean:
 		if(!m_val.v_b)	m_val.v_b = new vector<bool>;
 		m_val.v_b->resize(i_lvl,false);
 		break;
 	}
-        //- Get elements -
+	//- Get elements -
 	for( int i = 0, i_off=0; i < i_lvl; i++ )
 	{
 	    string s_el = TSYS::strSepParse(vls,0,';',&i_off);
-    	    switch(type())
+	    switch(type())
 	    {
 		case TFld::String:	(*m_val.v_s)[i] = s_el; break;
 		case TFld::Integer:	(*m_val.v_i)[i] = strtol(s_el.c_str(),NULL,(flg()&HexDec)?16:((flg()&OctDec)?8:10));	break;
@@ -262,8 +265,8 @@ void TFld::setValues( const string &vls )
 		m_val.v_i->resize(2,0);
 		(*m_val.v_i)[0] = strtol(TSYS::strSepParse(vls,0,';').c_str(),NULL,(flg()&HexDec)?16:((flg()&OctDec)?8:10));
 		(*m_val.v_i)[1] = strtol(TSYS::strSepParse(vls,1,';').c_str(),NULL,(flg()&HexDec)?16:((flg()&OctDec)?8:10));
-		break;		
-	    case TFld::Real:	
+		break;
+	    case TFld::Real:
 		if(!m_val.v_r)	m_val.v_r = new vector<double>;
 		m_val.v_r->resize(2,0);
 		(*m_val.v_r)[0] = atof(TSYS::strSepParse(vls,0,';').c_str());
@@ -276,72 +279,72 @@ void TFld::setSelNames( const string &slnms )
 {
     //- Set value list -
     if( !(flg()&TFld::Selected) ) return;
-    
+
     int i_lvl = 0;
     for( int i_off = 0; TSYS::strSepParse(slnms,0,';',&i_off).size(); i_lvl++ );
-    
+
     if(!m_sel)  m_sel = new vector<string>;
     m_sel->resize(i_lvl,"");
-    
+
     for( int i = 0, i_off = 0; i < i_lvl; i++ )
 	(*m_sel)[i] = TSYS::strSepParse(slnms,0,';',&i_off);
 }
 
 const vector<string> &TFld::selValS()
-{ 
-    if( flg()&TFld::Selected && type() == TFld::String ) 
+{
+    if( flg()&TFld::Selected && type() == TFld::String )
 	return *m_val.v_s;
     throw TError("Field",_("Field is not string."));
 }
 
 const vector<int> &TFld::selValI()
-{ 
-    if( type() == TFld::Integer ) 
+{
+    if( type() == TFld::Integer )
 	return *m_val.v_i;
     throw TError("Field",_("Field is not integer."));
 }
 
 const vector<double> &TFld::selValR()
-{ 
-    if( type() == TFld::Real ) 
+{
+    if( type() == TFld::Real )
 	return *m_val.v_r;
     throw TError("Field",_("Field is not real."));
 }
 
 const vector<bool> &TFld::selValB()
-{ 
-    if( flg()&TFld::Selected && type() == TFld::Boolean ) 
+{
+    if( flg()&TFld::Selected && type() == TFld::Boolean )
 	return *m_val.v_b;
     throw TError("Field",_("Field is not boolean."));
 }
 
 const vector<string> &TFld::selNm()
-{ 
-    if( m_sel && flg()&TFld::Selected )	return *m_sel; 
+{
+    if( m_sel && flg()&TFld::Selected )	return *m_sel;
     throw TError("Field",_("Field is not select type!"));
 }
-	
+
 TFld &TFld::operator=( TFld &fld )
 {
     //- Free old -
-    if( m_sel )	delete m_sel;    
+    if( m_sel )	delete m_sel;
     if( m_val.v_s != NULL )
 	switch(type())
 	{
-	    case TFld::String:	delete m_val.v_s;	break;			
-	    case TFld::Integer:	delete m_val.v_i; 	break;
+	    case TFld::String:	delete m_val.v_s;	break;
+	    case TFld::Integer:	delete m_val.v_i;	break;
 	    case TFld::Real:	delete m_val.v_r;	break;
 	    case TFld::Boolean:	delete m_val.v_b;	break;
 	}
     //- Create new -
-    m_name  = fld.name();
-    m_descr = fld.descr();
-    m_len   = fld.len();
-    m_flg   = fld.flg();
-    m_type  = fld.type();
-    m_def   = fld.def();
+    m_name	= fld.name();
+    m_descr	= fld.descr();
+    m_len	= fld.len();
+    m_flg	= fld.flg();
+    m_type	= fld.type();
+    m_def	= fld.def();
     //m_vals  = fld.vals();
-   
+
     //- Copy select and values border -
     if( flg()&TFld::Selected )
     {
@@ -349,21 +352,21 @@ TFld &TFld::operator=( TFld &fld )
 	*m_sel = fld.selNm();
 	switch(type())
 	{
-	    case TFld::String:	
+	    case TFld::String:
 		m_val.v_s = new vector<string>;
-	        *(m_val.v_s) = fld.selValS();
-		break;			
-	    case TFld::Integer:	
+		*(m_val.v_s) = fld.selValS();
+		break;
+	    case TFld::Integer:
 		m_val.v_i = new vector<int>;
-		*(m_val.v_i) = fld.selValI();		
+		*(m_val.v_i) = fld.selValI();
 		break;
-	    case TFld::Real:	
+	    case TFld::Real:
 		m_val.v_r = new vector<double>;
-		*(m_val.v_r) = fld.selValR();		
+		*(m_val.v_r) = fld.selValR();
 		break;
-	    case TFld::Boolean:	
+	    case TFld::Boolean:
 		m_val.v_b = new vector<bool>;
-		*(m_val.v_b) = fld.selValB();		
+		*(m_val.v_b) = fld.selValB();
 		break;
 	}
     }
@@ -372,11 +375,11 @@ TFld &TFld::operator=( TFld &fld )
 	{
 	    case TFld::Integer:
 		m_val.v_i = new vector<int>;
-		*(m_val.v_i) = fld.selValI();		
+		*(m_val.v_i) = fld.selValI();
 		break;
-	    case TFld::Real:	
+	    case TFld::Real:
 		m_val.v_r = new vector<double>;
-		*(m_val.v_r) = fld.selValR();		
+		*(m_val.v_r) = fld.selValR();
 		break;
 	}
     return *this;
@@ -389,10 +392,10 @@ string TFld::selVl2Nm( const string &val )
 	int sz = vmin(m_sel->size(), m_val.v_s->size());
 	if( !sz ) return("Empty");
 	int i_val = 0;
-	for(i_val = 0; i_val < sz; i_val++) 
+	for(i_val = 0; i_val < sz; i_val++)
 	    if((*m_val.v_s)[i_val] == val) break;
-        if( i_val >= sz ) return val;
-    	return (*m_sel)[i_val];
+	if( i_val >= sz ) return val;
+	return (*m_sel)[i_val];
     }
     throw TError("Field",_("Select error! Val: <%s>."),val.c_str());
 }
@@ -404,8 +407,8 @@ string TFld::selVl2Nm( int val )
 	int sz = vmin(m_sel->size(), m_val.v_i->size());
 	if( !sz ) return("Empty");
 	int i_val = 0;
-       	for(i_val = 0; i_val < sz; i_val++)
-    	    if((*m_val.v_i)[i_val] == val) break;
+	for(i_val = 0; i_val < sz; i_val++)
+	    if((*m_val.v_i)[i_val] == val) break;
 	if( i_val >= sz ) return TSYS::int2str(val);
 	return (*m_sel)[i_val];
     }
@@ -421,7 +424,7 @@ string TFld::selVl2Nm( double val )
 	int i_val = 0;
 	for(i_val = 0; i_val < sz; i_val++)
 	    if((*m_val.v_r)[i_val] == val) break;
-        if( i_val >= sz ) return TSYS::real2str(val);
+	if( i_val >= sz ) return TSYS::real2str(val);
 	return (*m_sel)[i_val];
     }
     throw TError("Field",_("Select error! Val: <%f>."),val);
@@ -437,7 +440,7 @@ string TFld::selVl2Nm( bool val )
 	for(i_val = 0; i_val < sz; i_val++)
 	    if( (*m_val.v_b)[i_val] == val ) break;
 	if( i_val >= sz ) return TSYS::int2str(val);
-    	return (*m_sel)[i_val];
+	return (*m_sel)[i_val];
     }
     throw TError("Field",_("Select error! Val: <%d>."),val);
 }
@@ -446,40 +449,40 @@ string TFld::selNm2VlS( const string &name )
 {
     if( flg()&TFld::Selected && type()==TFld::String )
 	for(int i_val = 0; i_val < vmin(m_sel->size(), m_val.v_s->size()); i_val++)
-	    if( name == (*m_sel)[i_val] ) 
+	    if( name == (*m_sel)[i_val] )
 		return (*m_val.v_s)[i_val];
     return name;
-    //throw TError("Field",_("Select error! Name: <%s>."),name.c_str());    
+    //throw TError("Field",_("Select error! Name: <%s>."),name.c_str());
 }
 
 int TFld::selNm2VlI( const string &name )
 {
     if( flg()&TFld::Selected && type()==TFld::Integer )
 	for(int i_val = 0; i_val < vmin(m_sel->size(), m_val.v_i->size()); i_val++)
-	    if( name == (*m_sel)[i_val] ) 
+	    if( name == (*m_sel)[i_val] )
 		return (*m_val.v_i)[i_val];
     return atoi(name.c_str());
-    //throw TError("Field",_("Select error! Name: <%s>."),name.c_str());    
+    //throw TError("Field",_("Select error! Name: <%s>."),name.c_str());
 }
 
 double TFld::selNm2VlR( const string &name )
 {
     if( flg()&TFld::Selected && type()==TFld::Real )
 	for(int i_val = 0; i_val < vmin(m_sel->size(), m_val.v_r->size()); i_val++)
-	    if( name == (*m_sel)[i_val] ) 
+	    if( name == (*m_sel)[i_val] )
 		return (*m_val.v_r)[i_val];
     return atof(name.c_str());
-    //throw TError("Field",_("Select error! Name: <%s>."),name.c_str());    
+    //throw TError("Field",_("Select error! Name: <%s>."),name.c_str());
 }
 
 bool TFld::selNm2VlB( const string &name )
 {
     if( flg()&TFld::Selected && type()==TFld::Boolean )
 	for(int i_val = 0; i_val < vmin(m_sel->size(), m_val.v_b->size()); i_val++)
-	    if( name == (*m_sel)[i_val] ) 
+	    if( name == (*m_sel)[i_val] )
 		return (*m_val.v_b)[i_val];
     return atoi(name.c_str());
-    //throw TError("Field",_("Select error! Name: <%s>."),name.c_str());    
+    //throw TError("Field",_("Select error! Name: <%s>."),name.c_str());
 }
 
 XMLNode *TFld::cntrCmdMake( XMLNode *opt, const string &path, int pos, const string &user, const string &grp, int perm )
@@ -487,19 +490,20 @@ XMLNode *TFld::cntrCmdMake( XMLNode *opt, const string &path, int pos, const str
     XMLNode *n_e = TCntrNode::ctrMkNode("fld",opt,pos,(path+"/"+name()).c_str(),descr(),
 	    (flg()&TFld::NoWrite)?(perm&~0222):perm,user.c_str(),grp.c_str(),1,"len",TSYS::int2str(len()).c_str());
     if( n_e )
-    {	    
-	if(flg()&TFld::Selected) 
+    {
+	if( flg()&TFld::Selected )
 	    n_e->setAttr_("tp","str")->setAttr_("len","")->setAttr_("dest","select")->
-		setAttr("select",path+"/sel_"+name());
+		setAttr("sel_list",selNames());
+		//setAttr("select",path+"/sel_"+name());
 	else switch(type())
-	{	    
-    	    case TFld::String:	
-		n_e->setAttr_("tp","str");	
+	{
+	    case TFld::String:
+		n_e->setAttr_("tp","str");
 		if( flg()&FullText )	n_e->setAttr_("cols","50")->setAttr_("rows","4");
 		break;
-    	    case TFld::Integer:	n_e->setAttr_("tp",(flg()&HexDec)?"hex":((flg()&OctDec)?"oct":"dec")); break;
-    	    case TFld::Real:	n_e->setAttr_("tp","real");break;
-    	    case TFld::Boolean:	n_e->setAttr_("tp","bool");break;
+	    case TFld::Integer:	n_e->setAttr_("tp",(flg()&HexDec)?"hex":((flg()&OctDec)?"oct":"dec")); break;
+	    case TFld::Real:	n_e->setAttr_("tp","real");break;
+	    case TFld::Boolean:	n_e->setAttr_("tp","bool");break;
 	}
     }
     return n_e;
