@@ -1,7 +1,7 @@
 
 //OpenSCADA system file: tsys.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -26,6 +26,7 @@
 #include <sys/time.h>
 #include <sys/utsname.h>
 #include <errno.h>
+#include <string.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <getopt.h>
@@ -34,6 +35,7 @@
 #include <stdarg.h>
 #include <stdlib.h>
 #include <langinfo.h>
+#include <zlib.h>
 
 #include "../config.h"
 #include "terror.h"
@@ -41,33 +43,33 @@
 #include "tsys.h"
 
 //Continuously access variable
-TMess 	*Mess;
-TSYS  	*SYS;
+TMess	*Mess;
+TSYS	*SYS;
 bool TSYS::finalKill = false;
 
-TSYS::TSYS( int argi, char ** argb, char **env ) : 
+TSYS::TSYS( int argi, char ** argb, char **env ) :
     m_confFile("/etc/oscada.xml"), m_id("EmptySt"), m_name("Empty Station"),
-    m_user("root"),argc(argi), envp((const char **)env), argv((const char **)argb), stop_signal(0), 
+    m_user("root"),argc(argi), envp((const char **)env), argv((const char **)argb), stop_signal(0),
     m_sysOptCfg(false), mWorkDB(""), mSaveAtExit(false), mSavePeriod(0)
 {
     finalKill = false;
     SYS = this;		//Init global access value
     m_subst = grpAdd("sub_",true);
     nodeEn();
-    
+
     Mess = new TMess();
 
     if( getenv("USER") ) m_user = getenv("USER");
-    
+
     //Init system clock
     clkCalc();
-    
+
     signal(SIGINT,sighandler);
     signal(SIGTERM,sighandler);
     //signal(SIGCHLD,sighandler);
     signal(SIGALRM,sighandler);
     signal(SIGPIPE,sighandler);
-    signal(SIGFPE,sighandler);    
+    signal(SIGFPE,sighandler);
     //signal(SIGSEGV,sighandler);
     signal(SIGABRT,sighandler);
 }
@@ -86,7 +88,7 @@ TSYS::~TSYS(  )
     del("Transport");
     del("Security");
     del("BD");
-    
+
     delete Mess;
 }
 
@@ -105,8 +107,8 @@ string TSYS::setWorkDir( const string &wdir )
 string TSYS::int2str( int val, TSYS::IntView view )
 {
     char buf[STR_BUF_LEN];
-    if( view == TSYS::Dec )      snprintf(buf,sizeof(buf),"%d",val); 
-    else if( view == TSYS::Oct ) snprintf(buf,sizeof(buf),"%o",val); 
+    if( view == TSYS::Dec )      snprintf(buf,sizeof(buf),"%d",val);
+    else if( view == TSYS::Oct ) snprintf(buf,sizeof(buf),"%o",val);
     else if( view == TSYS::Hex ) snprintf(buf,sizeof(buf),"%x",val);
 
     return buf;
@@ -115,8 +117,8 @@ string TSYS::int2str( int val, TSYS::IntView view )
 string TSYS::uint2str( unsigned val, IntView view )
 {
     char buf[STR_BUF_LEN];
-    if( view == TSYS::Dec )      snprintf(buf,sizeof(buf),"%u",val); 
-    else if( view == TSYS::Oct ) snprintf(buf,sizeof(buf),"%o",val); 
+    if( view == TSYS::Dec )      snprintf(buf,sizeof(buf),"%u",val);
+    else if( view == TSYS::Oct ) snprintf(buf,sizeof(buf),"%o",val);
     else if( view == TSYS::Hex ) snprintf(buf,sizeof(buf),"%x",val);
 
     return buf;
@@ -128,7 +130,7 @@ string TSYS::ll2str( long long val, IntView view )
     if( view == TSYS::Dec )      snprintf(buf,sizeof(buf),"%lld",val);
     else if( view == TSYS::Oct ) snprintf(buf,sizeof(buf),"%llo",val);
     else if( view == TSYS::Hex ) snprintf(buf,sizeof(buf),"%llx",val);
-		
+
     return buf;
 }
 
@@ -147,7 +149,7 @@ string TSYS::addr2str( void *addr )
 
     return buf;
 }
-	    
+
 void *TSYS::str2addr( const string &str )
 {
     return (void *)strtoul(str.c_str(),NULL,16);
@@ -159,7 +161,7 @@ bool TSYS::strEmpty( const string &val )
     for( int i_s = 0; i_s < val.size(); i_s++ )
 	if( val[i_s] != ' ' && val[i_s] != '\n' && val[i_s] != '\t' )
 	    return false;
-    
+
     return true;
 }
 
@@ -181,36 +183,36 @@ string TSYS::optDescr( )
 	"    --Station=<name>   Station name.\n"
 	"    --demon            Start into demon mode.\n"
 	"    --MessLev=<level>  Process messages <level> (0-7).\n"
-    	"    --log=<direct>     Direct messages to:\n"
-    	"                         <direct> & 1 - syslogd;\n"
-    	"                         <direct> & 2 - stdout;\n"
-    	"                         <direct> & 4 - stderr;\n"
+	"    --log=<direct>     Direct messages to:\n"
+	"                         <direct> & 1 - syslogd;\n"
+	"                         <direct> & 2 - stdout;\n"
+	"                         <direct> & 4 - stderr;\n"
 	"                         <direct> & 8 - archive.\n"
-    	"----------- The config file station <%s> parameters -----------\n"
+	"----------- The config file station <%s> parameters -----------\n"
 	"Workdir    <path>	Work directory.\n"
-    	"MessLev    <level>     Messages <level> (0-7).\n"
-    	"LogTarget  <direction> Direct messages to:\n"
-    	"                           <direct> & 1 - syslogd;\n"
-    	"                           <direct> & 2 - stdout;\n"
-    	"                           <direct> & 4 - stderr;\n"
+	"MessLev    <level>     Messages <level> (0-7).\n"
+	"LogTarget  <direction> Direct messages to:\n"
+	"                           <direct> & 1 - syslogd;\n"
+	"                           <direct> & 2 - stdout;\n"
+	"                           <direct> & 4 - stderr;\n"
 	"                           <direct> & 8 - archive.\n"
 	"SysLang    <lang>	Internal language.\n"
-    	"WorkDB     <Type.Name> Work DB (type and name).\n"
+	"WorkDB     <Type.Name> Work DB (type and name).\n"
 	"SaveAtExit <true>      Save system at exit.\n"
 	"SavePeriod <sec>	Save system period.\n"
 	"SYSOptCfg  <true>      Get system options from DB.\n\n"),
 	PACKAGE_NAME,VERSION,buf.sysname,buf.release,nodePath().c_str());
-		
+
     return s_buf;
 }
 
 bool TSYS::cfgFileLoad()
-{	
-    bool cmd_help = false;    
+{
+    bool cmd_help = false;
 
     //================ Load parameters from commandline =========================
     int next_opt;
-    char *short_opt="h";
+    const char *short_opt="h";
     struct option long_opt[] =
     {
 	{"help"     ,0,NULL,'h'},
@@ -225,10 +227,10 @@ bool TSYS::cfgFileLoad()
 	next_opt=getopt_long(argc,(char * const *)argv,short_opt,long_opt,NULL);
 	switch(next_opt)
 	{
-	    case 'h': 
-		fprintf(stdout,optDescr().c_str()); 
+	    case 'h':
+		fprintf(stdout,optDescr().c_str());
 		Mess->setMessLevel(7);
-		cmd_help = true; 
+		cmd_help = true;
 		break;
 	    case 'f': m_confFile = optarg; break;
 	    case 's': m_id = optarg; break;
@@ -238,7 +240,7 @@ bool TSYS::cfgFileLoad()
 
     //Load config file
     int hd = open(m_confFile.c_str(),O_RDONLY);
-    if( hd < 0 ) 
+    if( hd < 0 )
 	mess_err(nodePath().c_str(),_("Config file <%s> error: %s"),m_confFile.c_str(),strerror(errno));
     else
     {
@@ -251,8 +253,8 @@ bool TSYS::cfgFileLoad()
 	string s_buf = buf;
 	free(buf);
 	try
-	{ 
-	    root_n.load(s_buf); 
+	{
+	    root_n.load(s_buf);
 	    if( root_n.name() == "OpenSCADA" )
 	    {
 		XMLNode *stat_n = NULL;
@@ -260,14 +262,14 @@ bool TSYS::cfgFileLoad()
 		    if( root_n.childGet(i_st)->name() == "station" )
 		    {
 			stat_n = root_n.childGet(i_st);
-    			if( stat_n->attr("id") == m_id ) break;
+			if( stat_n->attr("id") == m_id ) break;
 		    }
-                if( stat_n && stat_n->attr("id") != m_id )
-                {
+		if( stat_n && stat_n->attr("id") != m_id )
+		{
 		    mess_warning(nodePath().c_str(),_("Station <%s> into config file no present. Use <%s> station config!"),
-                        m_id.c_str(), stat_n->attr("id").c_str() );
-  		    m_id 	= stat_n->attr("id");
-		    m_name 	= stat_n->attr("name");
+			m_id.c_str(), stat_n->attr("id").c_str() );
+		    m_id	= stat_n->attr("id");
+		    m_name	= stat_n->attr("name");
 		}
 		if( !stat_n )	root_n.clear();
 	    } else root_n.clear();
@@ -276,7 +278,7 @@ bool TSYS::cfgFileLoad()
 	}
 	catch( TError err ) { mess_err(nodePath().c_str(),_("Load config file error: %s"),err.mess.c_str() ); }
     }
-    
+
     return cmd_help;
 }
 
@@ -285,7 +287,7 @@ void TSYS::cfgPrmLoad()
     //System parameters
     m_sysOptCfg = atoi(TBDS::genDBGet(nodePath()+"SYSOptCfg",TSYS::int2str(m_sysOptCfg),"root",true).c_str());
     chdir(TBDS::genDBGet(nodePath()+"Workdir","","root",sysOptCfg()).c_str());
-    
+
     mWorkDB = TBDS::genDBGet(nodePath()+"WorkDB","*.*","root",sysOptCfg());
     setSaveAtExit( atoi(TBDS::genDBGet(nodePath()+"SaveAtExit","0","root").c_str()) );
     setSavePeriod( atoi(TBDS::genDBGet(nodePath()+"SavePeriod","0","root").c_str()) );
@@ -293,13 +295,13 @@ void TSYS::cfgPrmLoad()
 
 void TSYS::load_()
 {
-    static bool first_load = true;    
-    
+    static bool first_load = true;
+
     bool cmd_help = cfgFileLoad();
     mess_info(nodePath().c_str(),_("Load!"));
     cfgPrmLoad();
-    Mess->load();	//Messages load    
-    
+    Mess->load();	//Messages load
+
     if( first_load )
     {
 	//- Create subsystems -
@@ -312,16 +314,16 @@ void TSYS::load_()
 	add( new TSpecialS() );
 	add( new TUIS() );
 	add( new TModSchedul() );
-	
-	//- Load modules -        
-	modSchedul().at().load(); 
+
+	//- Load modules -
+	modSchedul().at().load();
 	modSchedul().at().loadLibS();
-	
+
 	//- First DB subsystem load -
 	db().at().load();
-	
+
 	//- Second load for load from generic DB -
-	cfgPrmLoad(); 
+	cfgPrmLoad();
 	Mess->load();
     }
 
@@ -329,10 +331,10 @@ void TSYS::load_()
     vector<string> lst;
     list(lst);
     for( unsigned i_a=0; i_a < lst.size(); i_a++ )
-        try{ at(lst[i_a]).at().load(); }
-        catch(TError err) 
-        { 
-    	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
+	try { at(lst[i_a]).at().load(); }
+	catch(TError err)
+	{
+	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
 	    mess_err(nodePath().c_str(),_("Error load subsystem <%s>."),lst[i_a].c_str());
 	}
 
@@ -343,16 +345,16 @@ void TSYS::load_()
 void TSYS::save_( )
 {
     char buf[STR_BUF_LEN];
-    
+
     mess_info(nodePath().c_str(),_("Save!"));
-    
+
     //System parameters
     getcwd(buf,sizeof(buf));
     TBDS::genDBSet(nodePath()+"Workdir",buf);
     TBDS::genDBSet(nodePath()+"WorkDB",mWorkDB);
     TBDS::genDBSet(nodePath()+"SaveAtExit",TSYS::int2str(saveAtExit()));
-    TBDS::genDBSet(nodePath()+"SavePeriod",TSYS::int2str(savePeriod()));    
-    
+    TBDS::genDBSet(nodePath()+"SavePeriod",TSYS::int2str(savePeriod()));
+
     Mess->save();       //Messages load
 }
 
@@ -360,46 +362,46 @@ int TSYS::start(  )
 {
     vector<string> lst;
     list(lst);
-    
+
     mess_info(nodePath().c_str(),_("Start!"));
     for( unsigned i_a=0; i_a < lst.size(); i_a++ )
 	try{ at(lst[i_a]).at().subStart(); }
-	catch(TError err) 
-	{ 
-	    mess_err(err.cat.c_str(),"%s",err.mess.c_str()); 
-	    mess_err(nodePath().c_str(),_("Error start subsystem <%s>."),lst[i_a].c_str()); 
-	}	    
-    
+	catch(TError err)
+	{
+	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
+	    mess_err(nodePath().c_str(),_("Error start subsystem <%s>."),lst[i_a].c_str());
+	}
+
     cfgFileScan( true );
-    unsigned int i_cnt = 1;    
-    while( !stop_signal )	
+    unsigned int i_cnt = 1;
+    while( !stop_signal )
     {
 	//- CPU frequency calc
 	if( !(i_cnt%(10*1000/STD_WAIT_DELAY)) )	clkCalc( );
 	//- Config file change periodic check -
-	if( !(i_cnt%(10*1000/STD_WAIT_DELAY)) )	cfgFileScan( );	
+	if( !(i_cnt%(10*1000/STD_WAIT_DELAY)) )	cfgFileScan( );
 	//- Periodic changes saving to DB -
 	if( savePeriod() && !(i_cnt%(savePeriod()*1000/STD_WAIT_DELAY)) )
 	    save();
-	    
-       	usleep( STD_WAIT_DELAY*1000 );
+
+	usleep( STD_WAIT_DELAY*1000 );
 	i_cnt++;
     }
-    
-    mess_info(nodePath().c_str(),_("Stop!"));  
+
+    mess_info(nodePath().c_str(),_("Stop!"));
     if( saveAtExit() || savePeriod() )	save();
     for( int i_a=lst.size()-1; i_a >= 0; i_a-- )
 	try { at(lst[i_a]).at().subStop(); }
-	catch(TError err) 
-	{ 
+	catch(TError err)
+	{
 	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
 	    mess_err(nodePath().c_str(),_("Error stop subsystem <%s>."),lst[i_a].c_str());
 	}
 
-    return stop_signal;       
+    return stop_signal;
 }
 
-void TSYS::stop( )    
+void TSYS::stop( )
 { 
     stop_signal = SIGINT;
 }
@@ -408,8 +410,8 @@ void TSYS::sighandler( int signal )
 {
     switch(signal)
     {
-	case SIGINT: 
-	    SYS->stop_signal=signal; 
+	case SIGINT:
+	    SYS->stop_signal=signal;
 	    break;
 	case SIGTERM:
 	    mess_warning(SYS->nodePath().c_str(),_("Have get a Terminate signal. Server been stoped!"));
@@ -426,7 +428,7 @@ void TSYS::sighandler( int signal )
 		mess_info(SYS->nodePath().c_str(),_("Free child process %d!"),pid);
 	    break;
 	}
-	case SIGPIPE:	
+	case SIGPIPE:
 	    //mess_warning(SYS->nodePath().c_str(),_("Broken PIPE signal!"));
 	    break;
 	case SIGSEGV:
@@ -435,7 +437,7 @@ void TSYS::sighandler( int signal )
 	case SIGABRT:
 	    mess_emerg(SYS->nodePath().c_str(),_("OpenSCADA aborted!"));
 	    break;
-	case SIGALRM:	break;    
+	case SIGALRM:	break;
 	default:
 	    mess_warning(SYS->nodePath().c_str(),_("Unknown signal %d!"),signal);
     }
@@ -445,7 +447,7 @@ void TSYS::cfgFileScan( bool first )
 {
     static string cfg_fl;
     static struct stat f_stat;
-    
+
     struct stat f_stat_t;
     bool   up = false;
 
@@ -457,7 +459,7 @@ void TSYS::cfgFileScan( bool first )
     else up = true;
     cfg_fl = cfgFile();
     stat(cfg_fl.c_str(),&f_stat);
-    
+
     if( up && !first )	{ modifG(); load(); }
 }
 
@@ -471,19 +473,19 @@ long long TSYS::curTime()
 string TSYS::fNameFix( const string &fname )
 {
     string tmp;
-    char   buf[1024];   //!!!!
-    
+    char   buf[STR_BUF_LEN];
+
     for( string::const_iterator it = fname.begin(); it != fname.end(); it++ )
-    {	
+    {
 	if( *(it) == '.' && *(it+1) == '.' && *(it+2) == '/')
 	{
 	    string cur = getcwd(buf,sizeof(buf));
 	    if(chdir("..") != 0) return "";
 	    tmp += getcwd(buf,sizeof(buf));
-	    chdir(cur.c_str());       	
+	    chdir(cur.c_str());
 	    it++;
 	}
-	else if( *(it) == '.' && *(it+1) == '/' ) 
+	else if( *(it) == '.' && *(it+1) == '/' )
 	    tmp += getcwd(buf,sizeof(buf));
 	else tmp += *(it);
     }
@@ -493,7 +495,7 @@ string TSYS::fNameFix( const string &fname )
 bool TSYS::eventWait( bool &m_mess_r_stat, bool exempl, const string &loc, time_t tm )
 {
     time_t t_tm, s_tm;
-    
+
     t_tm = s_tm = time(NULL);
     while( m_mess_r_stat != exempl )
     {
@@ -502,10 +504,10 @@ bool TSYS::eventWait( bool &m_mess_r_stat, bool exempl, const string &loc, time_
 	if( tm && ( c_tm > s_tm+tm) )
 	{
 	    mess_crit(loc.c_str(),_("Timeouted !!!"));
-    	    return true;
+	    return true;
 	}
 	//Make messages
-	if( c_tm > t_tm+1 )  //1sec 
+	if( c_tm > t_tm+1 )  //1sec
 	{
 	    t_tm = c_tm;
 	    mess_info(loc.c_str(),_("Wait event..."));
@@ -518,55 +520,55 @@ bool TSYS::eventWait( bool &m_mess_r_stat, bool exempl, const string &loc, time_
 string TSYS::strSepParse( const string &path, int level, char sep, int *off )
 {
     int an_dir = off ? *off : 0;
-    int	t_lev = 0;
+    int t_lev = 0;
     int t_dir;
-    
-    if( an_dir >= path.size() )	return "";
+
+    if( an_dir >= path.size() ) return "";
     while(true)
     {
-        t_dir = path.find(sep,an_dir);
-        if( t_dir == string::npos )	
-	{ 
-	    if( off ) *off = path.size(); 
+	t_dir = path.find(sep,an_dir);
+	if( t_dir == string::npos )
+	{
+	    if( off ) *off = path.size();
 	    return (t_lev == level) ? path.substr(an_dir) : "";
 	}
-        else if( t_lev == level )
+	else if( t_lev == level )
 	{
 	    if( off ) *off = t_dir+1;
-	    return path.substr(an_dir,t_dir-an_dir); 
+	    return path.substr(an_dir,t_dir-an_dir);
 	}
-        an_dir = t_dir+1;
+	an_dir = t_dir+1;
 	t_lev++;
     }
     return "";
-}		
+}
 
 string TSYS::pathLev( const string &path, int level, bool encode, int *off )
 {
     int an_dir = off ? *off : 0;
     int t_lev = 0;
     int t_dir;
-    
+
     //- First separators pass -
-    while( an_dir<path.size() && path[an_dir]=='/' ) an_dir++;    
+    while( an_dir<path.size() && path[an_dir]=='/' ) an_dir++;
     if( an_dir >= path.size() ) return "";
     //- Path level process -
     while(true)
     {
 	t_dir = path.find("/",an_dir);
 	if( t_dir == string::npos )
-        {
-            if( off ) *off = path.size();
-            return (t_lev == level) ? ( encode ? TSYS::strDecode(path.substr(an_dir),TSYS::PathEl) : path.substr(an_dir) ) : "";
-        }
+	{
+	    if( off ) *off = path.size();
+	    return (t_lev == level) ? ( encode ? TSYS::strDecode(path.substr(an_dir),TSYS::PathEl) : path.substr(an_dir) ) : "";
+	}
 	else if( t_lev == level )
 	{
 	    if( off ) *off = t_dir;
 	    return encode ? TSYS::strDecode(path.substr(an_dir,t_dir-an_dir),TSYS::PathEl) : path.substr(an_dir,t_dir-an_dir);
 	}
-        an_dir = t_dir;
+	an_dir = t_dir;
 	t_lev++;
-        while( an_dir<path.size() && path[an_dir]=='/' ) an_dir++;
+	while( an_dir<path.size() && path[an_dir]=='/' ) an_dir++;
     }
 }
 
@@ -577,7 +579,7 @@ string TSYS::path2sepstr( const string &path, char sep )
     while( !(curv=TSYS::pathLev(path,0,false,&off)).empty() )
 	rez+=curv+sep;
     if(!rez.empty())	rez.resize(rez.size()-1);
-    
+
     return rez;
 }
 
@@ -595,71 +597,71 @@ string TSYS::strEncode( const string &in, TSYS::Code tp, const string &symb )
 {
     int i_sz;
     string sout;
-    
-    switch(tp)	
+
+    switch(tp)
     {
 	case TSYS::PathEl:
 	    sout = in;
-            for( i_sz = 0; i_sz < sout.size(); i_sz++ )
-                switch(sout[i_sz])
-                {
+	    for( i_sz = 0; i_sz < sout.size(); i_sz++ )
+		switch( sout[i_sz] )
+		{
 		    case '/': sout.replace(i_sz,1,"%2f"); i_sz+=2; break;
 		    case '%': sout.replace(i_sz,1,"%25"); i_sz+=2; break;
 		}
-	    break;	
+	    break;
 	case TSYS::HttpURL:
 	    sout = in;
-    	    for( i_sz = 0; i_sz < sout.size(); i_sz++ )
-        	switch(sout[i_sz])
-                {
+	    for( i_sz = 0; i_sz < sout.size(); i_sz++ )
+		switch( sout[i_sz] )
+		{
 		    case '%': sout.replace(i_sz,1,"%25"); i_sz+=2; break;
 		    case ' ': sout.replace(i_sz,1,"%20"); i_sz+=2; break;
 		    case '\t': sout.replace(i_sz,1,"%09"); i_sz+=2; break;
 		    default:
-                	if( sout[i_sz]&0x80 )
-                        {
-                            char buf[4];
+			if( sout[i_sz]&0x80 )
+			{
+			    char buf[4];
 			    snprintf(buf,sizeof(buf),"%%%02X",(unsigned char)sout[i_sz]);
 			    sout.replace(i_sz,1,buf);
 			    i_sz+=2;
-                            break;
-                	}
+			    break;
+			}
 		}
-	    break;	
+	    break;
 	case TSYS::Html:
-	    sout.reserve(in.size()+10);	
+	    sout.reserve(in.size()+10);
 	    for( i_sz = 0; i_sz < in.size(); i_sz++ )
-		switch(in[i_sz])
-		{ 
-		    case '>': 	sout+="&gt;";	break;
-		    case '<': 	sout+="&lt;"; 	break;
-		    case '"': 	sout+="&quot;";	break;
-		    case '&': 	sout+="&amp;"; 	break;
-		    case '\'': 	sout+="&#039;";	break;
-		    default: 	sout+=in[i_sz];
+		switch( in[i_sz] )
+		{
+		    case '>':	sout+="&gt;";	break;
+		    case '<':	sout+="&lt;";	break;
+		    case '"':	sout+="&quot;";	break;
+		    case '&':	sout+="&amp;";	break;
+		    case '\'':	sout+="&#039;";	break;
+		    default:	sout+=in[i_sz];
 		}
-	    break;	
+	    break;
 	case TSYS::JavaSc:
 	    sout.reserve(in.size()+10);
 	    for( i_sz = 0; i_sz < in.size(); i_sz++ )
-		switch(in[i_sz])
-                {
-            	    case '\n':	sout+="\\n"; break;
-		    default:    sout+=in[i_sz];
-		}	
+		switch( in[i_sz] )
+		{
+		    case '\n':	sout+="\\n";	break;
+		    default:	sout+=in[i_sz];
+		}
 	    break;
 	case TSYS::SQL:
-	    sout.reserve(in.size()+10);		
+	    sout.reserve(in.size()+10);
 	    for( i_sz = 0; i_sz < in.size(); i_sz++ )
-		switch(in[i_sz])
+		switch( in[i_sz] )
 		{
 		    case '\'':	sout+="\\'";	break;
 		    case '\"':	sout+="\\\"";	break;
-		    case '`':  	sout+="\\`";	break;
+		    case '`':	sout+="\\`";	break;
 		    case '\\':	sout+="\\\\";	break;
-		    default:    sout+=in[i_sz];
-		}	    	
-            break;
+		    default:	sout+=in[i_sz];
+		}
+	    break;
 	case TSYS::Custom:
 	    sout.reserve(in.size()+10);
 	    for( i_sz = 0; i_sz < in.size(); i_sz++ )
@@ -669,8 +671,8 @@ string TSYS::strEncode( const string &in, TSYS::Code tp, const string &symb )
 		    if( in[i_sz] == symb[i_smb] )
 		    {
 			char buf[4];
-                        sprintf(buf,"%%%02X",(unsigned char)in[i_sz]);
-                        sout+=buf;
+			sprintf(buf,"%%%02X",(unsigned char)in[i_sz]);
+			sout += buf;
 			break;
 		    }
 		if( i_smb >= symb.size() ) sout+=in[i_sz];
@@ -679,12 +681,12 @@ string TSYS::strEncode( const string &in, TSYS::Code tp, const string &symb )
 	case TSYS::base64:
 	{
 	    sout.reserve(in.size()+in.size()/4+in.size()/57+10);
-	    char *base64alph = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+	    const char *base64alph = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 	    for( i_sz = 0; i_sz < in.size(); i_sz+=3 )
 	    {
 		if(i_sz && !(i_sz%57))	sout.push_back('\n');
 		sout.push_back(base64alph[(unsigned char)in[i_sz]>>2]);
-		if((i_sz+1) >= in.size()) 
+		if((i_sz+1) >= in.size())
 		{
 		    sout.push_back(base64alph[((unsigned char)in[i_sz]&0x03)<<4]);
 		    sout+="==";
@@ -692,12 +694,12 @@ string TSYS::strEncode( const string &in, TSYS::Code tp, const string &symb )
 		else
 		{
 		    sout.push_back(base64alph[(((unsigned char)in[i_sz]&0x03)<<4)|((unsigned char)in[i_sz+1]>>4)]);
-		    if((i_sz+2) >= in.size()) 
+		    if((i_sz+2) >= in.size())
 		    {
 			sout.push_back(base64alph[((unsigned char)in[i_sz+1]&0x0F)<<2]);
 			sout.push_back('=');
 		    }
-		    else 
+		    else
 		    {
 			sout.push_back(base64alph[(((unsigned char)in[i_sz+1]&0x0F)<<2)|((unsigned char)in[i_sz+2]>>6)]);
 			sout.push_back(base64alph[(unsigned char)in[i_sz+2]&0x3F]);
@@ -717,13 +719,13 @@ string TSYS::strEncode( const string &in, TSYS::Code tp, const string &symb )
 
 unsigned char TSYS::getBase64Code(unsigned char asymb) 
 {
-    switch(asymb)
+    switch( asymb )
     {
 	case 'A' ... 'Z':	return asymb-(unsigned char)'A';
 	case 'a' ... 'z':	return 26+asymb-(unsigned char)'a';
 	case '0' ... '9':	return 52+asymb-(unsigned char)'0';
-	case '+': return 62;		    
-	case '/': return 63;		
+	case '+':		return 62;
+	case '/':		return 63;
     }
 }
 
@@ -733,11 +735,11 @@ string TSYS::strDecode( const string &in, TSYS::Code tp )
     string sout;
 
     switch(tp)
-    {    
+    {
 	case TSYS::PathEl: case TSYS::HttpURL: case TSYS::Custom:
-	    sout.reserve(in.size());	    
+	    sout.reserve(in.size());
 	    for( i_sz = 0; i_sz < in.size(); i_sz++ )
-		switch(in[i_sz])
+		switch( in[i_sz] )
 		{
 		    case '%':
 			if( i_sz+2 < in.size() )
@@ -771,10 +773,77 @@ string TSYS::strDecode( const string &in, TSYS::Code tp )
 	    }
 	    break;
     }
-	    
+
     return sout;
 }
 
+string TSYS::strCompr( const string &in, int lev )
+{
+    z_stream strm;
+
+    if( in.empty() )	return "";
+
+    strm.zalloc = Z_NULL;
+    strm.zfree = Z_NULL;
+    strm.opaque = Z_NULL;
+
+    if( deflateInit(&strm,lev) != Z_OK ) return "";
+
+    uLongf comprLen = deflateBound(&strm,in.size());
+    char out[comprLen];
+
+    strm.next_in = (Bytef*)in.data();
+    strm.avail_in = (uInt)in.size();
+    strm.next_out = (Bytef*)out;
+    strm.avail_out = comprLen;
+
+    if( deflate(&strm, Z_FINISH) != Z_STREAM_END )
+    {
+	deflateEnd(&strm);
+	return "";
+    }
+
+    comprLen = strm.total_out;
+
+    deflateEnd(&strm);
+
+    return string(out,comprLen);
+}
+
+string TSYS::strUncompr( const string &in )
+{
+    int ret;
+    z_stream strm;
+    unsigned char out[STR_BUF_LEN];
+    string rez;
+
+    if( in.empty() )	return "";
+
+    strm.zalloc = Z_NULL;
+    strm.zfree = Z_NULL;
+    strm.opaque = Z_NULL;
+
+    if( inflateInit(&strm) != Z_OK )	return "";
+
+    strm.avail_in = in.size();
+    strm.next_in = (Bytef*)in.data();
+    do
+    {
+	strm.avail_out = sizeof(out);
+	strm.next_out = out;
+	ret=inflate(&strm,Z_NO_FLUSH);
+	if( ret == Z_STREAM_ERROR || ret == Z_NEED_DICT || ret == Z_DATA_ERROR || ret == Z_MEM_ERROR )
+	    break;
+	rez.append((char*)out,sizeof(out)-strm.avail_out);
+    } while( strm.avail_out == 0 );
+
+    inflateEnd(&strm);
+
+    if( ret != Z_STREAM_END )	return "";
+
+    return rez;
+}
+
 long TSYS::HZ()
 {
     return sysconf(_SC_CLK_TCK);
@@ -783,14 +852,14 @@ long TSYS::HZ()
 void TSYS::cntrCmdProc( XMLNode *opt )
 {
     char buf[STR_BUF_LEN];
-    
+
     //Get page info
     if( opt->name() == "info" )
     {
 	snprintf(buf,sizeof(buf),_("%s station: \"%s\""),PACKAGE_NAME,name().c_str());
 	ctrMkNode("oscada_cntr",opt,-1,"/",buf,0444);
-	if(ctrMkNode("branches",opt,-1,"/br","",0444))	
-	    ctrMkNode("grp",opt,-1,"/br/sub_",_("Subsystem"),0444,"root","root",1,"idm","1");	
+	if(ctrMkNode("branches",opt,-1,"/br","",0444))
+	    ctrMkNode("grp",opt,-1,"/br/sub_",_("Subsystem"),0444,"root","root",1,"idm","1");
 	if(TUIS::icoPresent(id())) ctrMkNode("img",opt,-1,"/ico","",0444);
 	if(ctrMkNode("area",opt,-1,"/gen",_("Station"),0444))
 	{
@@ -800,19 +869,23 @@ void TSYS::cntrCmdProc( XMLNode *opt )
 	    ctrMkNode("fld",opt,-1,"/gen/host",_("Host name"),0444,"root","root",1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/gen/user",_("System user"),0444,"root","root",1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/gen/sys",_("Operation system"),0444,"root","root",1,"tp","str");
-    	    ctrMkNode("fld",opt,-1,"/gen/frq",_("Frequency (MHZ)"),0444,"root","root",1,"tp","real");
+	    ctrMkNode("fld",opt,-1,"/gen/frq",_("Frequency (MHZ)"),0444,"root","root",1,"tp","real");
 	    ctrMkNode("fld",opt,-1,"/gen/clk_res",_("Realtime clock resolution (msec)"),0444,"root","root",1,"tp","real");
 	    ctrMkNode("fld",opt,-1,"/gen/in_charset",_("Internal charset"),0440,"root","root",1,"tp","str");
 	    ctrMkNode("fld",opt,-1,"/gen/config",_("Config file"),0440,"root","root",1,"tp","str");
-	    ctrMkNode("fld",opt,-1,"/gen/workdir",_("Work directory"),0664,"root","root",1,"tp","str");
-	    ctrMkNode("fld",opt,-1,"/gen/wrk_db",_("Work DB (module.bd)"),0660,"root",db().at().subId().c_str(),1,"tp","str");
-	    ctrMkNode("fld",opt,-1,"/gen/saveExit",_("Save system at exit"),0664,"root","root",1,"tp","bool");
-	    ctrMkNode("fld",opt,-1,"/gen/savePeriod",_("Save system period"),0664,"root","root",1,"tp","dec");	    
+	    ctrMkNode("fld",opt,-1,"/gen/workdir",_("Work directory"),0440,"root","root",1,"tp","str");
+	    ctrMkNode("fld",opt,-1,"/gen/wrk_db",_("Work DB"),0664,"root",db().at().subId().c_str(),4,"tp","str","dest","select","select","/db/list",
+		"help",_("Work DB address in format [<DB module>.<DB name>].\nChange it field if you want save or reload all system from other DB."));
+	    ctrMkNode("fld",opt,-1,"/gen/saveExit",_("Save system at exit"),0664,"root","root",2,"tp","bool",
+		"help",_("Select for automatic system saving to DB on exit."));
+	    ctrMkNode("fld",opt,-1,"/gen/savePeriod",_("Save system period"),0664,"root","root",2,"tp","dec",
+		"help",_("Use no zero period (seconds) for periodic saving of changed systems parts to DB."));
 	    ctrMkNode("fld",opt,-1,"/gen/lang",_("Language"),0664,"root","root",1,"tp","str");
 	    if(ctrMkNode("area",opt,-1,"/gen/mess",_("Messages"),0444))
 	    {
-		ctrMkNode("fld",opt,-1,"/gen/mess/lev",_("Least level"),0664,"root","root",2,"tp","dec","len","1");
- 		ctrMkNode("fld",opt,-1,"/gen/mess/log_sysl",_("To syslog"),0664,"root","root",1,"tp","bool");
+		ctrMkNode("fld",opt,-1,"/gen/mess/lev",_("Least level"),0664,"root","root",3,
+		    "tp","dec","len","1","help",_("Least messages level which process by the system."));
+		ctrMkNode("fld",opt,-1,"/gen/mess/log_sysl",_("To syslog"),0664,"root","root",1,"tp","bool");
 		ctrMkNode("fld",opt,-1,"/gen/mess/log_stdo",_("To stdout"),0664,"root","root",1,"tp","bool");
 		ctrMkNode("fld",opt,-1,"/gen/mess/log_stde",_("To stderr"),0664,"root","root",1,"tp","bool");
 		ctrMkNode("fld",opt,-1,"/gen/mess/log_arch",_("To archive"),0664,"root","root",1,"tp","bool");
@@ -823,58 +896,58 @@ void TSYS::cntrCmdProc( XMLNode *opt )
 	if(ctrMkNode("area",opt,-1,"/hlp",_("Help")))
 	    ctrMkNode("fld",opt,-1,"/hlp/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","10");
 	return;
-    }    
-    
+    }
+
     //Process command to page
     string a_path = opt->attr("path");
     if( a_path == "/ico" && ctrChkNode(opt) )
     {
 	string itp;
-        opt->setText(TSYS::strEncode(TUIS::icoGet(id(),&itp),TSYS::base64));
-        opt->setAttr("tp",itp);	
-    }	
-    else if( a_path == "/gen/host" && ctrChkNode(opt) )	
+	opt->setText(TSYS::strEncode(TUIS::icoGet(id(),&itp),TSYS::base64));
+	opt->setAttr("tp",itp);
+    }
+    else if( a_path == "/gen/host" && ctrChkNode(opt) )
     {
 	utsname ubuf; uname(&ubuf);
 	opt->setText(ubuf.nodename);
     }
-    else if( a_path == "/gen/sys" && ctrChkNode(opt) )  	
+    else if( a_path == "/gen/sys" && ctrChkNode(opt) )
     {
 	utsname ubuf; uname(&ubuf);
-	opt->setText(string(ubuf.sysname)+"-"+ubuf.release);	
+	opt->setText(string(ubuf.sysname)+"-"+ubuf.release);
     }
     else if( a_path == "/gen/user" && ctrChkNode(opt) )	opt->setText(m_user);
     else if( a_path == "/gen/prog" && ctrChkNode(opt) )	opt->setText(PACKAGE_NAME);
     else if( a_path == "/gen/ver" && ctrChkNode(opt) )	opt->setText(VERSION);
     else if( a_path == "/gen/stat" && ctrChkNode(opt) )	opt->setText(name());
     else if( a_path == "/gen/frq" && ctrChkNode(opt) )	opt->setText(TSYS::real2str((float)sysClk()/1000000.,6));
-    else if( a_path == "/gen/clk_res" && ctrChkNode(opt) )	
+    else if( a_path == "/gen/clk_res" && ctrChkNode(opt) )
     {
-        struct timespec tmval;
-        clock_getres(CLOCK_REALTIME,&tmval);
-        opt->setText(TSYS::real2str((float)tmval.tv_nsec/1000000.,4));
+	struct timespec tmval;
+	clock_getres(CLOCK_REALTIME,&tmval);
+	opt->setText(TSYS::real2str((float)tmval.tv_nsec/1000000.,4));
     }
     else if( a_path == "/gen/in_charset" && ctrChkNode(opt) )	opt->setText(Mess->charset());
     else if( a_path == "/gen/config" && ctrChkNode(opt) )	opt->setText(m_confFile);
     else if( a_path == "/gen/wrk_db" )
-    { 
-	if( ctrChkNode(opt,"get",0660,"root",db().at().subId().c_str(),SEQ_RD) ) opt->setText(mWorkDB); 
-	if( ctrChkNode(opt,"set",0660,"root",db().at().subId().c_str(),SEQ_WR) ) setWorkDB(opt->text());
+    {
+	if( ctrChkNode(opt,"get",0664,"root",db().at().subId().c_str(),SEQ_RD) ) opt->setText(mWorkDB);
+	if( ctrChkNode(opt,"set",0664,"root",db().at().subId().c_str(),SEQ_WR) ) setWorkDB(opt->text());
     }
     else if( a_path == "/gen/saveExit" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",db().at().subId().c_str(),SEQ_RD) ) opt->setText( int2str(saveAtExit()) ); 
+	if( ctrChkNode(opt,"get",0664,"root",db().at().subId().c_str(),SEQ_RD) ) opt->setText( int2str(saveAtExit()) );
 	if( ctrChkNode(opt,"set",0664,"root",db().at().subId().c_str(),SEQ_WR) ) setSaveAtExit( atoi(opt->text().c_str()) );
     }
     else if( a_path == "/gen/savePeriod" )
     {
-	if( ctrChkNode(opt,"get",0664,"root",db().at().subId().c_str(),SEQ_RD) ) opt->setText( int2str(savePeriod()) ); 
+	if( ctrChkNode(opt,"get",0664,"root",db().at().subId().c_str(),SEQ_RD) ) opt->setText( int2str(savePeriod()) );
 	if( ctrChkNode(opt,"set",0664,"root",db().at().subId().c_str(),SEQ_WR) ) setSavePeriod( atoi(opt->text().c_str()) );
-    }    
+    }
     else if( a_path == "/gen/workdir" )
     {
-	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(workDir());
-	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	setWorkDir(opt->text().c_str());
+	if( ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) )	opt->setText(workDir());
+	if( ctrChkNode(opt,"set",0440,"root","root",SEQ_WR) )	setWorkDir(opt->text().c_str());
     }
     else if( a_path == "/gen/lang" )
     {
@@ -888,17 +961,17 @@ void TSYS::cntrCmdProc( XMLNode *opt )
     }
     else if( a_path == "/gen/mess/log_sysl" )
     {
-    	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText((Mess->logDirect()&0x01)?"1":"0");
+	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText((Mess->logDirect()&0x01)?"1":"0");
 	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	Mess->setLogDirect( atoi(opt->text().c_str())?Mess->logDirect()|0x01:Mess->logDirect()&(~0x01) );
     }
-    else if( a_path == "/gen/mess/log_stdo" )	
+    else if( a_path == "/gen/mess/log_stdo" )
     {
 	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText((Mess->logDirect()&0x02)?"1":"0");
 	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	Mess->setLogDirect( atoi(opt->text().c_str())?Mess->logDirect()|0x02:Mess->logDirect()&(~0x02) );
     }
     else if( a_path == "/gen/mess/log_stde" )
     {
-    	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText((Mess->logDirect()&0x04)?"1":"0");
+	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText((Mess->logDirect()&0x04)?"1":"0");
 	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	Mess->setLogDirect( atoi(opt->text().c_str())?Mess->logDirect()|0x04:Mess->logDirect()&(~0x04) );
     }
     else if( a_path == "/gen/mess/log_arch" )
@@ -908,9 +981,9 @@ void TSYS::cntrCmdProc( XMLNode *opt )
     }
     else if( (a_path == "/br/sub_" || a_path == "/subs/br") && ctrChkNode(opt,"get",0444,"root","root",SEQ_RD) )
     {
-        vector<string> lst;
-        list(lst);
-        for( unsigned i_a=0; i_a < lst.size(); i_a++ )
+	vector<string> lst;
+	list(lst);
+	for( unsigned i_a=0; i_a < lst.size(); i_a++ )
 	    opt->childAdd("el")->setAttr("id",lst[i_a])->setText(at(lst[i_a]).at().subName());
     }
     else if( a_path == "/hlp/g_help" && ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) ) opt->setText(optDescr());
