@@ -1,7 +1,7 @@
 
 //OpenSCADA system file: xml.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2007 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2008 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -28,15 +28,15 @@
 //*************************************************
 const char *XMLNode::o_name = "XMLNode";
 
-XMLNode &XMLNode::operator=(XMLNode &prm)    
+XMLNode &XMLNode::operator=(XMLNode &prm)
 {
     //- Delete self children and atributes -
     n_attr.clear();
     v_attr.clear();
     for( int i_ch = 0; i_ch < m_children.size(); i_ch++ )
 	delete m_children[i_ch];
-    m_children.clear();    	
-    
+    m_children.clear();
+
     //- Copy params (name,text and atributes) -
     setName( prm.name() );
     setText( prm.text() );
@@ -48,7 +48,7 @@ XMLNode &XMLNode::operator=(XMLNode &prm)
     //- Recursive copy children -
     for( int i_ch = 0; i_ch < prm.childSize(); i_ch++ )
 	*childAdd() = *prm.childGet(i_ch);
-    
+
     return *this;
 }
 
@@ -61,7 +61,7 @@ XMLNode* XMLNode::childAdd( const string &name )
 {
     XMLNode *n = new XMLNode( name );
     childAdd( n );
-    
+
     return n;
 }
 
@@ -71,20 +71,20 @@ void XMLNode::childDel( const unsigned id )
     delete m_children[id];
     m_children.erase( m_children.begin()+id );
 }
-	
+
 void XMLNode::childClean( const string &name )
 {
     for( int i_ch = 0; i_ch < m_children.size(); i_ch++ )
-    	if( !name.size() || m_children[i_ch]->name() == name )
+	if( !name.size() || m_children[i_ch]->name() == name )
 	    childDel(i_ch--);
 }
 
 int XMLNode::childIns( unsigned id, XMLNode * n )
 {
-    if( n ) 
+    if( n )
     {
-	if( id > childSize() ) id = childSize();    
-    	m_children.insert( m_children.begin()+id, n );
+	if( id > childSize() ) id = childSize();
+	m_children.insert( m_children.begin()+id, n );
 	return id;
     }
 }
@@ -93,7 +93,7 @@ XMLNode* XMLNode::childIns( unsigned id, const string &name )
 {
     XMLNode *n = new XMLNode( name );
     childIns( id, n );
-    
+
     return n;
 }
 
@@ -109,7 +109,7 @@ XMLNode* XMLNode::childGet( const string &name, const int numb ) const
     for( int i_ch = 0, i_n = 0; i_ch < childSize(); i_ch++)
 	if( childGet(i_ch)->name() == name && i_n++ == numb ) 
 	    return childGet(i_ch);
-	    
+
     throw TError(o_name,"Child %s:%d no found!",name.c_str(),numb);
 }
 
@@ -117,7 +117,7 @@ XMLNode* XMLNode::childGet( const string &attr, const string &val, bool noex ) c
 {
     for( unsigned i_f = 0; i_f < childSize(); i_f++)
 	if( childGet(i_f)->attr(attr) == val ) return childGet(i_f);
-	
+
     if( noex ) return NULL;
     throw TError(o_name,"Child with attribut %s=%s no present.",attr.c_str(),val.c_str());
 }
@@ -126,7 +126,7 @@ void XMLNode::attrList( vector<string> & list ) const
 {
     list.clear();
     for(unsigned i_opt = 0; i_opt < n_attr.size(); i_opt++)
-	list.push_back(n_attr[i_opt]);    
+	list.push_back(n_attr[i_opt]);
 }
 
 void XMLNode::attrClear( )
@@ -153,13 +153,13 @@ XMLNode* XMLNode::setAttr( const string &name, const string &val )
     for(unsigned i_opt = 0; i_opt < n_attr.size(); i_opt++)
 	if(n_attr[i_opt] == name)
 	{
-	    v_attr[i_opt] = val;  
+	    v_attr[i_opt] = val;
 	    return this;
 	}
 
     n_attr.push_back(name);
     v_attr.push_back(val);
-    
+
     return this;
 }
 
@@ -169,11 +169,11 @@ XMLNode* XMLNode::clear()
     setText("");
 
     for( unsigned i_ch = 0; i_ch < m_children.size(); i_ch++ )
-    	if( m_children[i_ch] ) delete m_children[i_ch];    
+	if( m_children[i_ch] ) delete m_children[i_ch];
 
     m_children.clear();
     current_node = NULL;
-    
+
     return this;
 }
 
@@ -182,14 +182,14 @@ string XMLNode::save( unsigned char flg )
     string xml = ((flg&XMLNode::BrOpenPrev)?"\n<":"<") + encode( name() );
 
     for(unsigned i_atr = 0; i_atr < n_attr.size(); i_atr++)
-	xml = xml + " " + n_attr[i_atr] + "=\"" + Mess->codeConvOut("UTF8",encode(v_attr[i_atr])) + "\"";
-    	
+	xml = xml + " " + n_attr[i_atr] + "=\"" + Mess->codeConvOut("UTF-8",encode(v_attr[i_atr])) + "\"";
+
     if(!childSize() && !text().size())
 	xml+= (flg&(XMLNode::BrOpenPast|XMLNode::BrClosePast))?"/>\n":"/>";
-    else	
+    else
     {
-	xml = xml + ((flg&XMLNode::BrOpenPast)?">\n":">") + 
-		    Mess->codeConvOut("UTF8",encode(text())) + 
+	xml = xml + ((flg&XMLNode::BrOpenPast)?">\n":">") +
+		    Mess->codeConvOut("UTF-8",encode(text())) +
 		    ((flg&XMLNode::BrTextPast)?"\n":"");
 
 	for( int child_index = 0; child_index < childSize(); child_index++ )
@@ -206,17 +206,17 @@ string XMLNode::save( unsigned char flg )
 string XMLNode::encode( const string &s ) const
 {
     string sout = s;
-    
+
     for( int i_sz = 0; i_sz < sout.size(); i_sz++ )
-        switch(sout[i_sz])
-        {
-            case '>': sout.replace(i_sz,1,"&gt;"); i_sz+=3; break;
-            case '<': sout.replace(i_sz,1,"&lt;"); i_sz+=3; break;
-            case '"': sout.replace(i_sz,1,"&quot;"); i_sz+=5; break;
-            case '&': sout.replace(i_sz,1,"&amp;"); i_sz+=4; break;
-            case '\'': sout.replace(i_sz,1,"&#039;"); i_sz+=5; break;
-        }
-	
+	switch(sout[i_sz])
+	{
+	    case '>': sout.replace(i_sz,1,"&gt;"); i_sz+=3; break;
+	    case '<': sout.replace(i_sz,1,"&lt;"); i_sz+=3; break;
+	    case '"': sout.replace(i_sz,1,"&quot;"); i_sz+=5; break;
+	    case '&': sout.replace(i_sz,1,"&amp;"); i_sz+=4; break;
+	    case '\'': sout.replace(i_sz,1,"&#039;"); i_sz+=5; break;
+	}
+
     return sout;
 }
 
@@ -229,21 +229,22 @@ void XMLNode::load( const string &s )
 
     XML_SetElementHandler( p, start_element, end_element );
     XML_SetCharacterDataHandler( p, characters );
-    XML_SetUserData( p, this );    
+    XML_SetUserData( p, this );
 
     if( !XML_Parse( p, s.c_str(), s.size(), true ) )
     {
 	XML_ParserFree( p );
-        throw TError(o_name,"Parse error at line %d --- %s", XML_GetCurrentLineNumber(p), XML_ErrorString(XML_GetErrorCode(p)) );
+	throw TError(o_name,"Parse error at line %d --- %s. Source string: '%s'",
+	    XML_GetCurrentLineNumber(p), XML_ErrorString(XML_GetErrorCode(p)), ((s.size()>1024)?s.substr(0,1024)+"...":s).c_str());
     }
-    XML_ParserFree( p );    
+    XML_ParserFree( p );
 }
-							  
+
 void XMLNode::start_element( void *data, const char *el, const char **attr )
 {
     XMLNode *p = (XMLNode*)data;
     XMLNode *n = p;
-    
+
     if( p->current_node )
     {
 	n = new XMLNode();
@@ -254,7 +255,7 @@ void XMLNode::start_element( void *data, const char *el, const char **attr )
     while(*attr)
     {
 	n->n_attr.push_back(*attr++);
-	n->v_attr.push_back(Mess->codeConvIn("UTF8",*attr++));
+	n->v_attr.push_back(Mess->codeConvIn("UTF-8",*attr++));
     }
 
     p->node_stack.push_back(n);
@@ -264,7 +265,7 @@ void XMLNode::start_element( void *data, const char *el, const char **attr )
 void XMLNode::end_element( void *data, const char *el )
 {
     XMLNode *p = (XMLNode*)data;
-    p->current_node->setText(Mess->codeConvIn("UTF8",p->current_node->text()));
+    p->current_node->setText(Mess->codeConvIn("UTF-8",p->current_node->text()));
     if( !p->node_stack.empty() ) p->node_stack.pop_back();
     if( p->node_stack.empty() ) p->current_node = NULL;
     else p->current_node = p->node_stack[p->node_stack.size()-1];
@@ -272,11 +273,11 @@ void XMLNode::end_element( void *data, const char *el )
 
 void XMLNode::characters( void *userData, const XML_Char *s, int len )
 {
-    XMLNode *p = (XMLNode*)userData;    
+    XMLNode *p = (XMLNode*)userData;
     //if( !len )	return;
     if( p->current_node->m_text.size() ) p->current_node->m_text.append(s,len);
     else
-        for(int i_ch = 0; i_ch < len; i_ch++)
+	for(int i_ch = 0; i_ch < len; i_ch++)
 	{
 	    if(s[i_ch] == ' ' || s[i_ch] == '\n' || s[i_ch] == '\t' )	continue;
 	    p->current_node->m_text.assign(s+i_ch,len-i_ch);
